@@ -22,7 +22,7 @@
 //
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
- #include "archive.h"
+#include "archive.h"
 #include <wx/colour.h>
 #include <wx/xml/xml.h>
 #include "serialized_object.h"
@@ -31,8 +31,8 @@
 
 namespace
 {
-	const wxChar breakpointName[] = wxT("Breakpoint");
-	const wxChar breakpointArrayName[] = wxT("BreakpointArray");
+const wxChar breakpointName[] = wxT("Breakpoint");
+const wxChar breakpointArrayName[] = wxT("BreakpointArray");
 }
 
 //helper functions
@@ -57,6 +57,31 @@ static wxXmlNode *FindNodeByName(const wxXmlNode *parent, const wxString &tagNam
 }
 
 static void SetNodeContent(wxXmlNode *node, const wxString &text)
+{
+	wxXmlNode *n = node->GetChildren();
+	wxXmlNode *contentNode = NULL;
+	while (n) {
+		if (n->GetType() == wxXML_TEXT_NODE || n->GetType() == wxXML_CDATA_SECTION_NODE) {
+			contentNode = n;
+			break;
+		}
+		n = n->GetNext();
+	}
+
+	if (contentNode) {
+		// remove old node
+		node->RemoveChild(contentNode);
+		delete contentNode;
+	}
+
+	if(!text.IsEmpty()){
+		contentNode = new wxXmlNode(wxXML_TEXT_NODE, wxEmptyString, text);
+		node->AddChild( contentNode );
+
+	}
+}
+
+static void SetCDATANodeContent(wxXmlNode* node, const wxString& text)
 {
 	wxXmlNode *n = node->GetChildren();
 	wxXmlNode *contentNode = NULL;
@@ -75,8 +100,10 @@ static void SetNodeContent(wxXmlNode *node, const wxString &text)
 		delete contentNode;
 	}
 
-	contentNode = new wxXmlNode(wxXML_TEXT_NODE, wxEmptyString, text);
-	node->AddChild( contentNode );
+	if(!text.IsEmpty()){
+		contentNode = new wxXmlNode(wxXML_CDATA_SECTION_NODE, wxEmptyString, text);
+		node->AddChild( contentNode );
+	}
 }
 
 // class Tab info
@@ -172,8 +199,7 @@ bool Archive::Write(const wxString &name, std::vector<TabInfo>& _vTabInfoArr)
 	node->AddProperty(wxT("Name"), name);
 
 	//add an entry for each wxString in the array
-	for(size_t i=0; i<_vTabInfoArr.size(); i++)
-	{
+	for (size_t i=0; i<_vTabInfoArr.size(); i++) {
 		wxXmlNode *child = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, wxT("TabInfo"));
 		Archive arch;
 		arch.SetXmlNode(child);
@@ -375,7 +401,9 @@ bool Archive::Read(const wxString &name, int &value)
 
 	long v;
 	bool res = ReadSimple(v, wxT("int"), name);
-	value = v;
+	if ( res ) {
+		value = v;
+	}
 	return res;
 }
 
@@ -398,7 +426,9 @@ bool Archive::Read(const wxString &name, bool &value)
 {
 	long v;
 	bool res = ReadSimple(v, wxT("bool"), name);
-	v  == 0 ? value = false : value = true;
+	if( res ) {
+		v  == 0 ? value = false : value = true;
+	}
 	return res;
 }
 
@@ -414,6 +444,36 @@ bool Archive::Write(const wxString &name, const wxString &str)
 	return true;
 }
 
+bool Archive::WriteCData(const wxString &name, const wxString &str)
+{
+	if (!m_root) {
+		return false;
+	}
+
+	wxXmlNode *node = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, wxT("CData"));
+	m_root->AddChild(node);
+
+	SetCDATANodeContent(node, str);
+	node->AddProperty(wxT("Name"), name);
+	return true;
+}
+
+bool Archive::ReadCData(const wxString &name, wxString &value)
+{
+	if (!m_root) {
+		return false;
+	}
+
+	wxXmlNode *node = FindNodeByName(m_root, wxT("CData"), name);
+	if (node) {
+		// get the content node
+		value = node->GetNodeContent();
+		value.Trim().Trim(false);
+		return true;
+	}
+	return false;
+}
+
 bool Archive::Read(const wxString &name, wxString &value)
 {
 	if (!m_root) {
@@ -421,7 +481,7 @@ bool Archive::Read(const wxString &name, wxString &value)
 	}
 	wxXmlNode *node = FindNodeByName(m_root, wxT("wxString"), name);
 	if (node) {
-		value = node->GetPropVal(wxT("Value"), wxEmptyString);
+		value = node->GetPropVal(wxT("Value"), value);
 		return true;
 	}
 	return false;
