@@ -45,12 +45,15 @@ CppCommentCreator::~CppCommentCreator()
 
 wxString CppCommentCreator::CreateComment()
 {
-	if ( m_tag->GetKind() == wxT("class"))
+	if ( m_tag->GetKind() == wxT("class") || m_tag->GetKind() == wxT("struct") )
 		return wxT("$(ClassPattern)\n");
+		
 	else if (m_tag->GetKind() == wxT("function"))
 		return FunctionComment();
+		
 	else if (m_tag->GetKind() == wxT("prototype"))
 		return FunctionComment();
+		
 	return wxEmptyString;
 }
 
