@@ -31,23 +31,15 @@
 #include "singleton.h"
 #include "entry.h"
 #include <wx/filename.h>
-#include "db_record.h"
 #include "expression_result.h"
 #include "variable.h"
 #include "function.h"
-
-#ifdef WXMAKINGDLL_CODELITE
-#    define WXDLLIMPEXP_CL WXEXPORT
-#elif defined(WXUSINGDLL_CODELITE)
-#    define WXDLLIMPEXP_CL WXIMPORT
-#else
-#    define WXDLLIMPEXP_CL
-#endif
+#include "comment.h"
 
 enum SearchFlags
 {
-	PartialMatch = 1,
-	ExactMatch = 2,
+	PartialMatch        = 1,
+	ExactMatch          = 2,
 	IgnoreCaseSensitive = 4
 };
 
@@ -64,7 +56,7 @@ class TagsManager;
  * \date 09-02-2006
  * \author Eran
  */
-class WXDLLIMPEXP_CL Language
+class Language
 {
 	friend class Singleton<Language>;
 
@@ -121,7 +113,7 @@ public:
 	 * \param name file name
 	 * \param comments [output] returned vector of comments
 	 */
-	void ParseComments(const wxFileName &fileName, std::vector<DbRecordPtr>* comments);
+	void ParseComments(const wxFileName &fileName, std::vector<CommentPtr>* comments);
 
 	//==========================================================
 	// New API based on the yacc grammar files
@@ -224,8 +216,9 @@ private:
 	 * \brief Attempt to fix template results
 	 * \param typeName the type name that was detected by the parser
 	 * \param typeScope the type scope
+	 * \param moreScopes additional scopes (they are collected by parsing any 'using namespace XXX' lines in the code
 	 */
-	bool OnTemplates(wxString &typeName, wxString &typeScope, Variable &parent);
+	bool OnTemplates(wxString &typeName, wxString &typeScope, Variable &parent, const std::vector<wxString> &moreScopes);
 
 	/**
 	 * \brief attempt to expand 'typedef' to their actual value
