@@ -30,13 +30,14 @@
 
 TagsOptionsData::TagsOptionsData()
 		: SerializedObject()
-		, m_ccFlags(CC_DISP_FUNC_CALLTIP | CC_LOAD_EXT_DB | CC_CPP_KEYWORD_ASISST | CC_COLOUR_WORKSPACE_TAGS | CC_COLOUR_VARS)
-		, m_ccColourFlags(CC_COLOUR_DEFAULT)
-		, m_fileSpec(wxT("*.cpp;*.cc;*.cxx;*.h;*.hpp;*.c;*.c++"))
-		, m_minWordLen(3)
+		, m_ccFlags       (CC_DISP_FUNC_CALLTIP | CC_LOAD_EXT_DB | CC_CPP_KEYWORD_ASISST | CC_COLOUR_VARS | CC_ACCURATE_SCOPE_RESOLVING)
+		, m_ccColourFlags (CC_COLOUR_DEFAULT)
+		, m_fileSpec(wxT  ("*.cpp;*.cc;*.cxx;*.h;*.hpp;*.c;*.c++;*.tcc"))
+		, m_minWordLen    (3)
+		, m_parserEnabled (true)
+		, m_maxItemToColour(1000)
 {
 	m_languages.Add(wxT("C++"));
-//	m_languages.Add(wxT("Java"));
 }
 
 TagsOptionsData::~TagsOptionsData()
@@ -51,34 +52,35 @@ void TagsOptionsData::Serialize(Archive &arch)
 		m_prep.RemoveAt(where);
 	}
 
-	arch.Write(wxT("m_ccFlags"), m_ccFlags);
-	arch.Write(wxT("m_ccColourFlags"), m_ccColourFlags);
-	arch.Write(wxT("m_prep"), m_prep);
-	arch.Write(wxT("m_fileSpec"), m_fileSpec);
-	arch.Write(wxT("m_languages"), m_languages);
-	arch.Write(wxT("m_minWordLen"), m_minWordLen);
+	arch.Write(wxT("m_ccFlags"),           m_ccFlags);
+	arch.Write(wxT("m_ccColourFlags"),     m_ccColourFlags);
+	arch.Write(wxT("m_prep"),              m_prep);
+	arch.Write(wxT("m_fileSpec"),          m_fileSpec);
+	arch.Write(wxT("m_languages"),         m_languages);
+	arch.Write(wxT("m_minWordLen"),        m_minWordLen);
+	arch.Write(wxT("m_parserSearchPaths"), m_parserSearchPaths);
+	arch.Write(wxT("m_parserEnabled"),     m_parserEnabled);
+	arch.Write(wxT("m_parserExcludePaths"),m_parserExcludePaths);
+	arch.Write(wxT("m_maxItemToColour"),   m_maxItemToColour);
 }
 
 void TagsOptionsData::DeSerialize(Archive &arch)
 {
-	arch.Read(wxT("m_ccFlags"), m_ccFlags);
-	if( !arch.Read(wxT("m_ccColourFlags"), m_ccColourFlags) ) {
-		m_ccColourFlags = CC_COLOUR_DEFAULT;
-	}
-
-	arch.Read(wxT("m_prep"), m_prep);
-	arch.Read(wxT("m_fileSpec"), m_fileSpec);
-	arch.Read(wxT("m_languages"), m_languages);
-	arch.Read(wxT("m_minWordLen"), m_minWordLen);
-
+	arch.Read(wxT("m_ccFlags"),            m_ccFlags);
+	arch.Read(wxT("m_ccColourFlags"),      m_ccColourFlags);
+	arch.Read(wxT("m_prep"),               m_prep);
+	arch.Read(wxT("m_fileSpec"),           m_fileSpec);
+	arch.Read(wxT("m_languages"),          m_languages);
+	arch.Read(wxT("m_minWordLen"),         m_minWordLen);
+	arch.Read(wxT("m_parserSearchPaths"),  m_parserSearchPaths);
+	arch.Read(wxT("m_parserEnabled"),      m_parserEnabled);
+	arch.Read(wxT("m_parserExcludePaths"), m_parserExcludePaths);
+	arch.Read(wxT("m_maxItemToColour"),    m_maxItemToColour);
 	// Hack: remove "extern"
 	int where = m_prep.Index(wxT("extern"));
 	if (where != wxNOT_FOUND) {
 		m_prep.RemoveAt(where);
 	}
-
-	//TODO :: remove the CC_CACHE_WORKSPACE_TAGS
-	m_ccFlags &= ~(CC_CACHE_WORKSPACE_TAGS);
 }
 
 wxString TagsOptionsData::ToString() const
