@@ -35,15 +35,13 @@ enum CodeCompletionOpts {
 	CC_DISP_TYPE_INFO             = 0x00000004,
 	CC_DISP_FUNC_CALLTIP          = 0x00000008,
 	CC_LOAD_EXT_DB                = 0x00000010,
-	CC_LOAD_EXT_DB_TO_MEMORY      = 0x00000020,
 	CC_PARSE_EXT_LESS_FILES       = 0x00000040,
 	CC_COLOUR_VARS                = 0x00000080,
 	CC_COLOUR_WORKSPACE_TAGS      = 0x00000100,
 	CC_CPP_KEYWORD_ASISST         = 0x00000200,
-	CC_CACHE_WORKSPACE_TAGS       = 0x00000400,
 	CC_DISABLE_AUTO_PARSING       = 0x00000800,
 	CC_MARK_TAGS_FILES_IN_BOLD    = 0x00001000,
-	CC_USE_FULL_RETAGGING         = 0x00002000,
+//	CC_USE_FULL_RETAGGING         = 0x00002000,
 	CC_RETAG_WORKSPACE_ON_STARTUP = 0x00004000,
 	CC_ACCURATE_SCOPE_RESOLVING   = 0x00008000
 };
@@ -61,20 +59,25 @@ enum CodeCompletionColourOpts {
 	CC_COLOUR_ENUMERATOR        = 0x00000200,
 	CC_COLOUR_VARIABLE          = 0x00000400,
 	CC_COLOUR_MEMBER            = 0x00000800,
+	CC_COLOUR_ALL               = CC_COLOUR_CLASS | CC_COLOUR_STRUCT | CC_COLOUR_FUNCTION |
+	CC_COLOUR_ENUM | CC_COLOUR_UNION | CC_COLOUR_PROTOTYPE | CC_COLOUR_TYPEDEF | CC_COLOUR_MACRO |
+	CC_COLOUR_NAMESPACE | CC_COLOUR_ENUMERATOR | CC_COLOUR_VARIABLE | CC_COLOUR_MEMBER,
 	CC_COLOUR_DEFAULT           = CC_COLOUR_CLASS | CC_COLOUR_STRUCT | CC_COLOUR_FUNCTION |
-									CC_COLOUR_ENUM | CC_COLOUR_PROTOTYPE
+	CC_COLOUR_ENUM | CC_COLOUR_PROTOTYPE
 };
 
 class TagsOptionsData : public SerializedObject
 {
-	size_t m_ccFlags;
-	size_t m_ccColourFlags;
-
-	wxArrayString m_prep;
-	wxString m_fileSpec;
-	wxArrayString m_languages;
-	int m_minWordLen;
-
+	size_t           m_ccFlags;
+	size_t           m_ccColourFlags;
+	wxArrayString    m_prep;
+	wxString         m_fileSpec;
+	wxArrayString    m_languages;
+	int              m_minWordLen;
+	wxArrayString    m_parserSearchPaths;
+	wxArrayString    m_parserExcludePaths;
+	bool             m_parserEnabled;
+	int              m_maxItemToColour;
 public:
 	TagsOptionsData();
 	virtual ~TagsOptionsData();
@@ -132,6 +135,30 @@ public:
 	const size_t& GetCcColourFlags() const {
 		return m_ccColourFlags;
 	}
+	void SetParserEnabled(const bool& parserEnabled) {
+		this->m_parserEnabled = parserEnabled;
+	}
+	void SetParserSearchPaths(const wxArrayString& parserSearchPaths) {
+		this->m_parserSearchPaths = parserSearchPaths;
+	}
+	const bool& GetParserEnabled() const {
+		return m_parserEnabled;
+	}
+	const wxArrayString& GetParserSearchPaths() const {
+		return m_parserSearchPaths;
+	}
+	void SetParserExcludePaths(const wxArrayString& parserExcludePaths) {
+		this->m_parserExcludePaths = parserExcludePaths;
+	}
+	const wxArrayString& GetParserExcludePaths() const {
+		return m_parserExcludePaths;
+	}
+	void SetMaxItemToColour(int maxItemToColour) {
+		this->m_maxItemToColour = maxItemToColour;
+	}
+	int GetMaxItemToColour() const {
+		return m_maxItemToColour;
+	}
 };
 
 #endif //TAGS_OPTIONS_DATA_H
