@@ -2,40 +2,24 @@
 #define __ContinuousBuild__
 
 #include "plugin.h"
+#include "buildprocess.h"
 #include "compiler.h"
 
 class wxEvtHandler;
 class ContinousBuildPane;
 class ShellCommand;
 
-struct CurrentBuildFile {
-	wxString file;
-	wxString project;
-    wxString config;
-	wxArrayString output;
-
-	void Clear() {
-		output.Clear();
-		file.Clear();
-		project.Clear();
-        config.Clear();
-	}
-};
-
 class ContinuousBuild : public IPlugin
 {
 	ContinousBuildPane *m_view;
-	wxEvtHandler *m_topWin;
-	ShellCommand *m_shellProcess;
-	wxArrayString m_files;
-	CurrentBuildFile m_currentBuildInfo;
+	wxEvtHandler *      m_topWin;
+	BuildProcess        m_buildProcess;
+	wxArrayString       m_files;
+	bool                m_buildInProgress;
 
 public:
-	void DoBuild(const wxString &fileName);
-	void DoReportErrors();
-	bool IsCompilable(const wxString &fileName);
-	CompilerPtr DoGetCompiler();
-	
+	void        DoBuild(const wxString &fileName);
+
 public:
 	ContinuousBuild(IManager *manager);
 	~ContinuousBuild();
@@ -54,10 +38,11 @@ public:
 	// Event handlers
 	DECLARE_EVENT_TABLE()
 
-	void OnFileSaved(wxCommandEvent &e);
-	void OnShellAddLine(wxCommandEvent &e);
-	void OnShellBuildStarted(wxCommandEvent &e);
-	void OnShellProcessEnded(wxCommandEvent &e);
+	void OnFileSaved           (wxCommandEvent &e);
+	void OnIgnoreFileSaved     (wxCommandEvent &e);
+	void OnStopIgnoreFileSaved (wxCommandEvent &e);
+	void OnBuildProcessEnded   (wxCommandEvent &e);
+	void OnBuildProcessOutput  (wxCommandEvent &e);
 };
 
 #endif //ContinuousBuild
