@@ -32,6 +32,7 @@
 #include "wx/arrstr.h"
 #include "wx/event.h"
 #include "vector"
+#include "macros.h"
 
 enum DebuggerCommands {
 	DBG_PAUSE = 0,
@@ -42,12 +43,36 @@ enum DebuggerCommands {
 };
 
 // Breakpoint types. If you add more, LEditor::FillBPtoMarkerArray will also need altering
-enum BP_type { /*Convenient return-codes*/ BP_type_invalid = -1, BP_type_none = 0, /*Real breakpoint-types*/ BP_FIRST_ITEM, BP_type_break = BP_FIRST_ITEM,
-        BP_type_cmdlistbreak, BP_type_condbreak, BP_type_ignoredbreak, BP_type_tempbreak, BP_LAST_MARKED_ITEM = BP_type_tempbreak,
-        BP_type_watchpt, BP_LAST_ITEM = BP_type_watchpt
-             };
+enum BreakpointType {
+	/*Convenient return-codes*/
+	BP_type_invalid = -1,
+	BP_type_none = 0,
+	/*Real breakpoint-types*/
+	BP_FIRST_ITEM,
+	BP_type_break = BP_FIRST_ITEM,
+	BP_type_cmdlistbreak,
+	BP_type_condbreak,
+	BP_type_ignoredbreak,
+	BP_type_tempbreak,
+	BP_LAST_MARKED_ITEM = BP_type_tempbreak,
+	BP_type_watchpt,
+	BP_LAST_ITEM = BP_type_watchpt
+};
+
 // Watchpoint subtypes: write,read and both
-enum WP_type { WP_watch, WP_rwatch, WP_awatch };
+enum WatchpointType {
+	WP_watch,
+	WP_rwatch,
+	WP_awatch
+};
+
+// The breakpoint origin:
+// Can be from the Editor (user clicked 'F9')
+// or from any other source (direct command to gdb, from the start up command etc)
+enum BreakpointOrigin {
+	BO_Editor,
+	BO_Other
+};
 
 //-------------------------------------------------------
 // Data structures used by the debugger
@@ -68,43 +93,96 @@ struct StackEntry {
 };
 
 struct ThreadEntry {
-	bool 	 	active;
-	long 		dbgid;
-	wxString	more;
+	bool     active;
+	long     dbgid;
+	wxString more;
 };
 
-typedef std::vector<StackEntry> StackEntryArray;
-typedef std::vector<ThreadEntry> ThreadEntryArray;
+struct VariableObjChild {
+	int      numChilds;    // If this child has children (i.e. is this child a node or leaf)
+	wxString varName;      // the name of the variable object node
+	wxString gdbId;        // A unique name given by gdb which holds this node information for further queries
+	wxString value;
+	bool     isAFake;      // Sets to true of this variable object is a fake node
+	VariableObjChild() : numChilds(0), isAFake(false) {}
+};
+
+struct VariableObject {
+	bool     isPtr;         // if this variable object is of type pointer
+	bool     isPtrPtr;      // if this variable object is of type pointer pointer
+	wxString gdbId;         // GDB unique identifier for this variable object
+	wxString typeName;      // the type of this variable object
+	int      numChilds;     // Number of children
+
+	VariableObject() : isPtr(false), isPtrPtr(false), numChilds(0) {}
+};
+
+struct LocalVariable {
+	wxString name;
+	wxString value;
+	wxString type;
+	bool     updated;
+	wxString gdbId; // Mac generates variable object for locals as well...
+
+	LocalVariable() : updated(false) {}
+	~LocalVariable() {}
+};
+
+typedef std::vector<VariableObjChild> VariableObjChildren;
+typedef std::vector<StackEntry>       StackEntryArray;
+typedef std::vector<ThreadEntry>      ThreadEntryArray;
+typedef std::vector<LocalVariable>    LocalVariables;
 
 class BreakpointInfo: public SerializedObject
 {
 public:
 	// Where the bp is: file/lineno, function name (e.g. main()) or the memory location
-	wxString file;
-	int lineno;
-	wxString watchpt_data;
-	wxString function_name;
-	bool regex;							// Is the function_name a regex?
-	int memory_address;
-
+	wxString               file;
+	int                    lineno;
+	wxString               watchpt_data;
+	wxString               function_name;
+	bool                   regex;            // Is the function_name a regex?
+	wxString               memory_address;
 	// How to identify the bp. Because the debugger won't always be running, we need an internal id as well as the debugger's one
-	int internal_id;
-	int debugger_id;	// -1 signifies not set
+	int                    internal_id;
+	int                    debugger_id;	// -1 signifies not set
+	BreakpointType         bp_type;  // Is it a plain vanilla breakpoint, or a temporary one, or a watchpoint, or...
+	unsigned int           ignore_number; // 0 means 'not ignored'. >0 is the number of times the bp must be hit before it becomes enabled
+	bool                   is_enabled;
+	bool                   is_temp;
+	WatchpointType         watchpoint_type;	// If this is a watchpoint, holds which sort it is
+	wxString               commandlist;
+	wxString               conditions;
+	wxString               at;
+	wxString               what;
+	BreakpointOrigin       origin;
 
+	BreakpointInfo(const BreakpointInfo& BI ):
+			file(BI.file),
+			lineno(BI.lineno),
+			watchpt_data(BI.watchpt_data),
+			function_name(BI.function_name),
+			regex(BI.regex),
+			memory_address(BI.memory_address),
+			internal_id(BI.internal_id),
+			debugger_id(BI.debugger_id),
+			bp_type(BI.bp_type),
+			ignore_number(BI.ignore_number),
+			is_enabled(BI.is_enabled),
+			is_temp (BI.is_temp),
+			watchpoint_type(BI.watchpoint_type),
+			commandlist(BI.commandlist),
+			conditions(BI.conditions),
+			at(BI.at),
+			what(BI.what),
+			origin(BI.origin) {}
 
-	enum BP_type bp_type;  // Is it a plain vanilla breakpoint, or a temporary one, or a watchpoint, or...
-	unsigned int ignore_number; // 0 means 'not ignored'. >0 is the number of times the bp must be hit before it becomes enabled
-	bool is_enabled;
-	bool is_temp;
-	enum WP_type watchpoint_type;	// If this is a watchpoint, holds which sort it is
-	wxString commandlist;
-	wxString conditions;
+	BreakpointInfo() : lineno(-1), regex(false), debugger_id(-1), bp_type(BP_type_break),
+			ignore_number(0), is_enabled(true), is_temp(false), watchpoint_type(WP_watch), origin(BO_Other) {}
 
-	BreakpointInfo() : lineno(-1), regex(false), memory_address(-1), debugger_id(-1), bp_type(BP_type_break),
-			ignore_number(0), is_enabled(true), is_temp(false), watchpoint_type(WP_watch)	{}
-	BreakpointInfo(const BreakpointInfo& BI ) {
-		*this = BI;
-	}
+//	BreakpointInfo(const BreakpointInfo& BI ) {
+//		*this = BI;
+//	}
 
 	bool IsConditional() {
 		return ! conditions.IsEmpty();
@@ -119,26 +197,29 @@ public:
 	}
 
 	BreakpointInfo& operator=(const BreakpointInfo& BI) {
-		file = BI.file;
-		lineno = BI.lineno;
-		function_name = BI.function_name;
-		memory_address = BI.memory_address;
-		bp_type = BI.bp_type;
-		watchpoint_type = BI.watchpoint_type;
-		watchpt_data = BI.watchpt_data;
-		commandlist = BI.commandlist;
-		regex = BI.regex;
-		is_temp = BI.is_temp;
-		internal_id = BI.internal_id;
-		debugger_id = BI.debugger_id;
-		is_enabled = BI.is_enabled;
-		ignore_number = BI.ignore_number;
-		conditions = BI.conditions;
+		file             = BI.file;
+		lineno           = BI.lineno;
+		watchpt_data     = BI.watchpt_data;
+		function_name    = BI.function_name;
+		regex            = BI.regex;
+		memory_address   = BI.memory_address;
+		internal_id      = BI.internal_id;
+		debugger_id      = BI.debugger_id;
+		bp_type          = BI.bp_type;
+		ignore_number    = BI.ignore_number;
+		is_enabled       = BI.is_enabled;
+		is_temp          = BI.is_temp;
+		watchpoint_type  = BI.watchpoint_type;
+		commandlist      = BI.commandlist;
+		conditions       = BI.conditions;
+		at               = BI.at;                 // Provided by the debugger, no need to serialize
+		what             = BI.what;               // Provided by the debugger, no need to serialize
+		origin           = BI.origin;
 		return *this;
 	}
 
 	bool operator==(const BreakpointInfo& BI) {
-		return ((file == BI.file) && (lineno == BI.lineno) && (function_name == BI.function_name) && (memory_address == BI.memory_address)
+		return ((origin == BI.origin) && (what == BI.what) && (at == BI.at) && (file == BI.file) && (lineno == BI.lineno) && (function_name == BI.function_name) && (memory_address == BI.memory_address)
 		        && (bp_type == BI.bp_type) &&  (watchpt_data == BI.watchpt_data)&& (is_enabled == BI.is_enabled)
 		        && (ignore_number == BI.ignore_number) && (conditions == BI.conditions) && (commandlist == BI.commandlist) && (is_temp == BI.is_temp)
 		        && (bp_type==BP_type_watchpt ? (watchpoint_type == BI.watchpoint_type) : true) && (!function_name.IsEmpty() ? (regex == BI.regex) : true));
@@ -154,14 +235,14 @@ protected:
 		arch.Write(wxT("bp_type"), bp_type);
 		arch.Write(wxT("watchpoint_type"), watchpoint_type);
 		arch.Write(wxT("watchpt_data"), watchpt_data);
-		arch.Write(wxT("commandlist"), commandlist);
+		// WriteCDate tends to write white-space even for empty commandlists
+		arch.WriteCData(wxT("commandlist"), commandlist.Trim().Trim(false));
 		arch.Write(wxT("regex"), regex);
 		arch.Write(wxT("is_temp"), is_temp);
-		arch.Write(wxT("internal_id"), internal_id);
-		arch.Write(wxT("debugger_id"), debugger_id);
 		arch.Write(wxT("is_enabled"), is_enabled);
 		arch.Write(wxT("ignore_number"), (int)ignore_number);
 		arch.Write(wxT("conditions"), conditions);
+		arch.Write(wxT("origin"),     (int)origin);
 	}
 
 	virtual void DeSerialize(Archive& arch) {
@@ -172,16 +253,14 @@ protected:
 		arch.Read(wxT("bp_type"), (int&)bp_type);
 		arch.Read(wxT("watchpoint_type"), (int&)watchpoint_type);
 		arch.Read(wxT("watchpt_data"), watchpt_data);
-		arch.Read(wxT("commandlist"), commandlist);
+		arch.ReadCData(wxT("commandlist"), commandlist);
+		commandlist.Trim().Trim(false); // ReadCData tends to add white-space to the commands e.g. a terminal \n
 		arch.Read(wxT("regex"), regex);
 		arch.Read(wxT("is_temp"), is_temp);
-		arch.Read(wxT("internal_id"), internal_id);
-		arch.Read(wxT("debugger_id"), debugger_id);
-		arch.Read(wxT("is_enabled"), is_enabled);
 		arch.Read(wxT("ignore_number"), (int&)ignore_number);
 		arch.Read(wxT("conditions"), conditions);
+		arch.Read(wxT("origin"),     (int&)origin);
 	}
-	//
 };
 
 /**
@@ -248,10 +327,15 @@ public:
 	bool      enableDebugLog;
 	bool      enablePendingBreakpoints;
 	bool      breakAtWinMain;
-	bool      resolveThis;
 	bool      showTerminal;
 	wxString  consoleCommand;
 	bool      useRelativeFilePaths;
+	bool      catchThrow;
+	bool      showTooltips;
+	bool      debugAsserts;
+	wxString  startupCommands;
+	int       maxDisplayStringSize;
+	bool      resolveLocals;
 
 public:
 	DebuggerInformation()
@@ -260,19 +344,50 @@ public:
 			, enableDebugLog(false)
 			, enablePendingBreakpoints(true)
 			, breakAtWinMain(false)
-			, resolveThis(false)
 			, showTerminal(false)
-#if defined(__WXGTK__)
-			, consoleCommand(wxT("xterm -title '$(TITLE)' -e '$(CMD)'"))
-#elif defined(__WXMAC__)
-			, consoleCommand(wxT("osascript -e 'tell application \"Terminal\"' -e 'activate' -e 'do script with command \"$(CMD)\"' -e 'end tell'"))
-#else
-			, consoleCommand(wxT(""))
-#endif
-			, useRelativeFilePaths(false) {}
+			, consoleCommand(TERMINAL_CMD)
+			, useRelativeFilePaths(false)
+			, catchThrow(false)
+			, showTooltips(false)
+			, debugAsserts(false)
+			, startupCommands(wxEmptyString)
+			, maxDisplayStringSize(200)
+			, resolveLocals (false) {}
 	~DebuggerInformation() {}
 };
 
+class IDebugger;
+
+// sent by debugger start events
+class DebuggerStartupInfo
+{
+public:
+	enum DebugType {
+		DebugProject  = 1,
+		AttachProcess = 2,
+		QuickDebug    = 3
+	};
+
+	long pid;
+	wxString project;
+	IDebugger *debugger;
+
+	DebuggerStartupInfo()
+			: pid(wxNOT_FOUND), project(wxEmptyString), debugger(NULL) {}
+
+	DebuggerStartupInfo(long pid)
+			: pid(pid), project(wxEmptyString), debugger(NULL) {}
+
+	DebuggerStartupInfo(const wxString &project)
+			: pid(wxNOT_FOUND), project(project), debugger(NULL) {}
+
+	DebugType GetDebugType() const {
+		return pid != wxNOT_FOUND ? AttachProcess :
+		       project.IsEmpty()  ? QuickDebug :
+		       DebugProject;
+	}
+};
+
 class IDebuggerObserver;
 class EnvironmentConfig;
 
@@ -281,7 +396,7 @@ class EnvironmentConfig;
 //-------------------------------------------------------
 /**
  * \ingroup Interfaces
- * Defines the debugger interface
+ * Defines the *GDB* debugger interface
  *
  * \version 1.0
  * first version
@@ -294,26 +409,67 @@ class EnvironmentConfig;
 class IDebugger
 {
 protected:
-	IDebuggerObserver *m_observer;
+	IDebuggerObserver * m_observer;
 	DebuggerInformation m_info;
-	EnvironmentConfig *m_env;
+	EnvironmentConfig * m_env;
+	wxString            m_name;
+	bool                m_isRemoteDebugging;
 
 public:
-	IDebugger() : m_env(NULL) {};
+	IDebugger() : m_env(NULL), m_isRemoteDebugging(false) {};
 	virtual ~IDebugger() {};
-	virtual void SetObserver(IDebuggerObserver *observer) {
+
+	void SetName(const wxString &name) {
+		m_name = name;
+	}
+	wxString GetName() const {
+		return m_name;
+	}
+
+	void SetObserver(IDebuggerObserver *observer) {
 		m_observer = observer;
 	}
-	virtual IDebuggerObserver *GetObserver() {
+
+	IDebuggerObserver *GetObserver() {
 		return m_observer;
 	}
-	virtual void SetEnvironment(EnvironmentConfig *env) {
+
+	void SetEnvironment(EnvironmentConfig *env) {
 		m_env = env;
 	}
-	virtual void SetDebuggerInformation(const DebuggerInformation& info) {
+
+	void SetDebuggerInformation(const DebuggerInformation& info) {
 		m_info = info;
 	}
 
+	DebuggerInformation GetDebuggerInformation() {
+		return m_info;
+	}
+
+	void SetIsRemoteDebugging(bool isRemoteDebugging) {
+		this->m_isRemoteDebugging = isRemoteDebugging;
+	}
+	
+	bool GetIsRemoteDebugging() const {
+		return m_isRemoteDebugging;
+	}
+	
+	/**
+	 * \brief Sets the logging level 'on the fly'
+	 * \param level the new level
+	 */
+	void EnableLogging(bool level) {
+		m_info.enableDebugLog = level;
+	}
+
+	/**
+	 * \brief Gets the current logging level
+	 * \return the current level
+	 */
+	bool IsLoggingEnabled() const {
+		return m_info.enableDebugLog;
+	}
+
 	//-------------------------------------------------------------
 	// Debugger operations
 	//-------------------------------------------------------------
@@ -400,17 +556,29 @@ public:
 	 */
 	virtual bool Break(const BreakpointInfo& bp) = 0;
 	/**
-	 * \brief remove breakpoint from given file and line
-	 */
-	virtual bool RemoveBreak(const wxString &file, long lineno) = 0;
-	/**
-	 * \brief remove breakpoint by its internal ID
+	 * \brief remove breakpoint by its ID
 	 */
 	virtual bool RemoveBreak(int bid) = 0;
 	/**
 	 * \brief clear all breakpoints set (gdb's 'clear' command)
 	 */
 	virtual bool RemoveAllBreaks() = 0;
+	/**
+	 * \brief Enable or Disable a breakpoint
+	 */
+	virtual bool SetEnabledState(const int bid, const bool enable) = 0;
+	/**
+	 * \brief Set this breakpoint's Ignore count
+	 */
+	virtual bool SetIgnoreLevel(const int bid, const int ignorecount) = 0;
+	/**
+	 * \brief Set this breakpoint's condition
+	 */
+	virtual bool SetCondition(const BreakpointInfo& bp) = 0;
+	/**
+	 * \brief Set a command-list for this breakpoint
+	 */
+	virtual bool SetCommands(const BreakpointInfo& bp) = 0;
 	/**
 	 * \brief ask the debugger to query about its file & line. Once the result arrives, the observer's UpdateFileLine() will be invoked
 	 */
@@ -419,12 +587,7 @@ public:
 	 * \brief executes a command that does not yield any output from the debugger
 	 */
 	virtual bool ExecuteCmd(const wxString &cmd) = 0;
-	/**
-	 * \brief excute a command and waits from its answer from the debugger
-	 * \param command command to execute
-	 * \param output the debugger response
-	 */
-	virtual bool ExecSyncCmd(const wxString &command, wxString &output) = 0;
+
 	/**
 	 * \brief ask the debugger to print the current stack local variable. When the results arrives, the observer->UpdateLocals() is called
 	 * \return true on success, false otherwise
@@ -444,10 +607,9 @@ public:
 	virtual bool SetFrame(int frame) = 0;
 	/**
 	 * \brief return list of threads.
-	 * \param threads [output]
 	 * \return true on success, false otherwise
 	 */
-	virtual bool ListThreads(ThreadEntryArray &threads) = 0;
+	virtual bool ListThreads() = 0;
 	/**
 	 * \brief select threadId to be active
 	 * \param threadId thread id
@@ -458,44 +620,78 @@ public:
 	 * \brief the Poke() method is called at Idle() time by the application so the debugger can read the actual debugger process output and process it
 	 */
 	virtual void Poke() = 0;
-	/**
-	 * \brief return tip for a give expression
-	 * \param expression expression to evaluate
-	 * \param evaluated evaluated value from the debugger
-	 * \return
-	 */
-	virtual bool GetTip(const wxString &expression, wxString &evaluated) = 0;
 
 	/**
-	 * \brief resolve expression and return its actual type
+	 * @brief return string to show the user as tip for expression. this is an async call. When this function is done, it will trigger a call to
+	 * IDebuggerObserver::UpdateTip()
+	 * @param dbgCommand debugger command to evaluate the tip (e.g. "print")
+	 * @param expression expression to evaluate
+	 * @return true if evaluation succeeded, false otherwise
+	 */
+	virtual bool GetAsciiViewerContent(const wxString &dbgCommand, const wxString &expression) = 0;
+
+	/**
+	 * \brief resolve expression and return its actual type this is an async call. When this function is done, it will trigger a call to
+	 * IDebuggerObserver::UpdateTypeResolved()
 	 * \param expression expression to evaluate
-	 * \param type the type [output]
 	 * \return true on success, false otherwise
 	 */
-	virtual bool ResolveType(const wxString &expression, wxString &type) = 0;
+	virtual bool ResolveType(const wxString &expression) = 0;
 
 	//We provide two ways of evulating an expressions:
 	//The short one, which returns a string, and long one
 	//which returns a tree of the result
 	virtual bool EvaluateExpressionToString(const wxString &expression, const wxString &format) = 0;
-	virtual bool EvaluateExpressionToTree(const wxString &expression) = 0;
 
 	/**
 	 * \brief a request to display memory from address -> address + count. This is a synchronous call
 	 * \param address starting address
 	 * \param count address range
-	 * \param output [output] string containing the formatted result
 	 * \return true on success, false otherwise
 	 */
-	virtual bool WatchMemory(const wxString &address, size_t count, wxString &output) = 0;
+	virtual bool WatchMemory(const wxString &address, size_t count) = 0;
 
 	/**
 	 * \brief set memory at given address and of size count. the value to set must be a space delimited
 	 * hex values (e.g. '0x01 0x02 0x03')
 	 */
 	virtual bool SetMemory(const wxString &address, size_t count, const wxString &hex_value) = 0;
-};
 
+	/**
+	 * \brief have the debugger list all breakpoints
+	 */
+	virtual void BreakList() = 0;
+
+	// ----------------------------------------------------------------------------------------
+	// Variable object manipulation (GDB only)
+	// If you wish to implement a debugger other than
+	// GDB, implement all the 'Variable Object' releated method
+	// with an empty implementation
+	// ----------------------------------------------------------------------------------------
+	/**
+	 * @brief list the children of a variable object
+	 * @param name
+	 */
+	virtual bool ListChildren(const wxString& name, int userReason) = 0;
+
+	/**
+	 * @brief create variable object from a given expression
+	 * @param expression
+	 */
+	virtual bool CreateVariableObject(const wxString &expression, int userReason) = 0;
+
+	/**
+	 * @brief delete variable object
+	 * @param name
+	 */
+	virtual bool DeleteVariableObject(const wxString &name) = 0;
+
+	/**
+	 * @brief evaluate variable object
+	 * @param name variable object
+	 */
+	virtual bool EvaluateVariableObject(const wxString &name, int userReason) = 0;
+};
 
 //-----------------------------------------------------------
 // Each debugger module must implement these two functions
