@@ -42,28 +42,6 @@
 
 class IManager;
 
-class ParseThreadEventData
-{
-	wxString m_fileName;
-	std::vector<std::pair<wxString, TagEntry> >  m_items;
-public:
-	ParseThreadEventData() {}
-	~ParseThreadEventData() {}
-
-	void SetFileName(const wxString& fileName) {
-		this->m_fileName = fileName.c_str();
-	}
-	void SetItems(const std::vector<std::pair<wxString, TagEntry> >& items) {
-		this->m_items = items;
-	}
-	const wxString& GetFileName() const {
-		return m_fileName;
-	}
-	const std::vector<std::pair<wxString, TagEntry> >& GetItems() const {
-		return m_items;
-	}
-};
-
 /**
  * Possible popup menu
  */
@@ -101,6 +79,10 @@ enum {
 	wxEVT_FILE_EXP_INIT_DONE,
 	//clientData is NULL
 	wxEVT_FILE_EXP_REFRESHED,
+	// an attempt to open a file using double click / ENTER was made
+	// on an item in the 'File Explorer' OR from the 'Workspace' tree
+	// clientData is the full path of the file (wxString*)
+	wxEVT_TREE_ITEM_FILE_ACTIVATED,
 
 	//clientData is list of files added to project (wxArrayString*)
 	wxEVT_PROJ_FILE_ADDED,
@@ -114,10 +96,20 @@ enum {
 	//clientData is the selected word (wxString*)
 	wxEVT_CCBOX_SELECTION_MADE,
 
-	//clientData is fileName (wxString*)
+	// the following 2 events are used as "transaction"
+	// the first event indicates that any "wxEVT_FILE_SAVED" event sent from this point
+	// is due to build process which is about to starte
+	// the later event, indicates the end of that transaction
+	wxEVT_FILE_SAVE_BY_BUILD_END,
+	wxEVT_FILE_SAVE_BY_BUILD_START,
+
+	// clientData is fileName (wxString*)
 	wxEVT_FILE_SAVED,
-	//clientData is list of files which have been retagged (std::vector<wxFileName>*)
+	// clientData is list of files which have been retagged (std::vector<wxFileName>*)
 	wxEVT_FILE_RETAGGED,
+	// clientData is wxArrayString*: Item(0) = oldName
+	//                               Item(1) = newName
+	wxEVT_FILE_RENAMED,
 	//clientData is ParseThreadEventData*
 	wxEVT_SYNBOL_TREE_UPDATE_ITEM,
 	//clientData is ParseThreadEventData*
@@ -132,11 +124,27 @@ enum {
 	//clientData is NULL
 	wxEVT_ALL_EDITORS_CLOSED,
 
+	// This event is sent when the user clicks inside an editor
+	// this event can not be Veto()
+	// clientData is NULL. You may query the clicked editor by calling to
+	// IManager::GetActiveEditor()
+	wxEVT_EDITOR_CLICKED,
+
+	// User dismissed the Editor's settings dialog with
+	// Apply or OK (Settings | Editor)
+	// clientData is NULL
+	wxEVT_EDITOR_SETTINGS_CHANGED,
 
 	// This event is sent from plugins to the application to tell it to reload
 	// any open files (and re-tag them as well)
 	wxEVT_CMD_RELOAD_EXTERNALLY_MODIFIED,
 
+	// Same as wxEVT_CMD_RELOAD_EXTERNALLY_MODIFIED
+	// just without prompting the user
+	// this event only reload code files without
+	// any reload to the workspace / project
+	wxEVT_CMD_RELOAD_EXTERNALLY_MODIFIED_NOPROMPT,
+
 	// Sent by the project settings dialogs to indicate that
 	// the project configurations are saved
 	// clientData is the project name (wxString*)
@@ -184,8 +192,49 @@ enum {
 	// event.GetString() returns the selected configuration
 	// the return answer is done by simply avoid calling event.Skip() (which will result in ProcessEvent() == true)
 	wxEVT_GET_IS_PLUGIN_MAKEFILE,
-	wxEVT_GET_IS_PLUGIN_BUILD
+	wxEVT_GET_IS_PLUGIN_BUILD,
 
+	/**
+	 * Debug related events
+	 */
+
+	// sent when the debugger is about to start
+	// clientData is a pointer to a DebuggerStartupInfo structure
+	wxEVT_DEBUG_STARTING,
+
+	// sent right after the debugger started; program is not running yet
+	// clientData is a pointer to a DebuggerStartupInfo structure
+	wxEVT_DEBUG_STARTED,
+
+	// sent just before the debugger stops
+	// clientData is NULL
+	wxEVT_DEBUG_ENDING,
+
+	// sent after the debugger stopped
+	// clientData is NULL
+	wxEVT_DEBUG_ENDED,
+
+	/**
+	 ** Build events (additional)
+	 **/
+	// These events allows the plugins to concatenate a string
+	// to the compilation/link line of the default build system
+	// By using the event.SetString()/event.GetString()
+	// Note, that the since all multiple plugins
+	// might be interesting with this feature, it is recommened
+	// to use it like this:
+	// wxString content = event.GetString();
+	// content << wxT(" -DMYMACRO ");
+	// event.SetString( content );
+	// event.Skip();
+	wxEVT_GET_ADDITIONAL_COMPILEFLAGS,
+	wxEVT_GET_ADDITIONAL_LINKFLAGS,
+
+	// Sent to the plugins to request to export the makefile
+	// for the project + configuration
+	// clientData is the builded project name (wxString*)
+	// event.GetString() returns the selected configuration
+	wxEVT_PLUGIN_EXPORT_MAKEFILE
 };
 
 //------------------------------------------------------------------
