@@ -24,6 +24,7 @@
 //////////////////////////////////////////////////////////////////////////////
 
 #include <wx/socket.h>
+#include <wx/splash.h>
 #include "evnvarlist.h"
 #include "environmentconfig.h"
 #include "conffilelocator.h"
@@ -33,6 +34,7 @@
 #include <wx/filefn.h>
 #include "dirsaver.h"
 
+
 #include "xmlutils.h"
 #include "editor_config.h"
 #include <wx/xrc/xmlres.h>
@@ -47,12 +49,17 @@
 #include "globals.h"
 #include "wx/tokenzr.h"
 #include "wx/dir.h"
-#include "splashscreen.h"
 #include <wx/stdpaths.h>
 
 #define __PERFORMANCE
 #include "performance.h"
 
+//////////////////////////////////////////////
+// Define the version string for this codelite
+//////////////////////////////////////////////
+extern wxChar *SvnRevision;
+wxString CODELITE_VERSION_STR = wxString::Format(wxT("v2.2.0.%s"), SvnRevision);
+
 #if defined(__WXMAC__)||defined(__WXGTK__)
 #include <signal.h> // sigprocmask
 #endif
@@ -61,8 +68,6 @@
 #include <wx/msw/registry.h> //registry keys
 #endif
 
-extern wxChar *SvnRevision;
-
 #ifdef __WXMAC__
 #include <mach-o/dyld.h>
 
@@ -87,6 +92,47 @@ wxString MacGetBasePath()
 	return rest;
 }
 #endif
+//-------------------------------------------------
+// helper method to draw the revision + version
+// on our splash screen
+//-------------------------------------------------
+static wxBitmap clDrawSplashBitmap(const wxBitmap& bitmap, const wxString &mainTitle)
+{
+	wxBitmap bmp ( bitmap.GetWidth(), bitmap.GetHeight()  );
+    wxMemoryDC dcMem;
+	
+    dcMem.SelectObject( bmp );
+	dcMem.DrawBitmap  ( bitmap, 0, 0, true);
+
+	//write the main title & subtitle
+	wxCoord w, h;
+	wxFont font = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
+	wxFont smallfont = wxSystemSettings::GetFont(wxSYS_DEFAULT_GUI_FONT);
+	font.SetPointSize(12);
+	smallfont.SetPointSize(10);
+	dcMem.SetFont(font);
+	dcMem.GetMultiLineTextExtent(mainTitle, &w, &h);
+	wxCoord bmpW = bitmap.GetWidth();
+
+	//draw shadow
+	dcMem.SetTextForeground(wxT("WHITE"));
+	
+	wxCoord textX = (bmpW - w)/2;
+	dcMem.DrawText(mainTitle, textX, 11);
+	dcMem.SelectObject(wxNullBitmap);
+	return bmp;
+}
+
+//-----------------------------------------------------
+// Splashscreen
+//-----------------------------------------------------
+class clSplashScreen : public wxSplashScreen
+{
+public:
+	clSplashScreen(const wxBitmap& bmp) : wxSplashScreen(bmp, wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_TIMEOUT, 5000, NULL, wxID_ANY, wxDefaultPosition, wxDefaultSize, wxBORDER_SIMPLE| wxFRAME_NO_TASKBAR| wxSTAY_ON_TOP)
+	{
+	}
+};
 
 #if wxVERSION_NUMBER < 2900
 static const wxCmdLineEntryDesc cmdLineDesc[] = {
@@ -266,9 +312,11 @@ bool App::OnInit()
 	}
 
 	ManagerST::Get()->SetInstallDir( homeDir );
-	EditorConfig::Init( SvnRevision );
 #endif
 
+	// Update codelite revision and Version
+	EditorConfig::Init( SvnRevision, wxT("2.0.2") );
+
 	wxString curdir = wxGetCwd();
 	::wxSetWorkingDirectory(homeDir);
 	// Load all of the XRC files that will be used. You can put everything
@@ -323,6 +371,7 @@ bool App::OnInit()
 		// read the installation path of MinGW & WX
 		wxRegKey rk(wxT("HKEY_CURRENT_USER\\Software\\CodeLite"));
 		if(rk.Exists()) {
+			m_parserPaths.Clear();
 			wxString strWx, strMingw;
 			if(rk.HasValue(wxT("wx"))){
 				rk.QueryValue(wxT("wx"), strWx);
@@ -332,27 +381,27 @@ bool App::OnInit()
 				rk.QueryValue(wxT("mingw"), strMingw);
 			}
 
-			long up;
-			if( !cfg->GetLongValue(wxT("UpdateWxPaths"), up) ){
-				if(strWx.IsEmpty() == false) {
-					// we have WX installed on this machine, set the path of WXWIN & WXCFG to point to it
-					EvnVarList vars;
-					EnvironmentConfig::Instance()->Load();
+			EvnVarList vars;
+			EnvironmentConfig::Instance()->Load();
+			EnvironmentConfig::Instance()->ReadObject(wxT("Variables"), &vars);
 
-					EnvironmentConfig::Instance()->ReadObject(wxT("Variables"), &vars);
-					StringMap varMap = vars.GetVariables();
-					varMap[wxT("WXWIN")] = strWx;
-					varMap[wxT("WXCFG")] = wxT("gcc_dll\\mswu");
+			if(strWx.IsEmpty() == false) {
+				// we have WX installed on this machine, set the path of WXWIN & WXCFG to point to it
+				std::map<wxString, wxString> envs = vars.GetVariables(wxT("Default"));
+				if(envs.find(wxT("WXWIN")) == envs.end())
+					vars.AddVariable(wxT("Default"), wxT("WXWIN"), strWx);
 
-					vars.SetVariables(varMap);
+				if(envs.find(wxT("WXCFG")) == envs.end())
+					vars.AddVariable(wxT("Default"), wxT("WXCFG"), wxT("gcc_dll\\mswu"));
 
-					EnvironmentConfig::Instance()->WriteObject(wxT("Variables"), &vars);
-					cfg->SaveLongValue(wxT("UpdateWxPaths"), 1);
-				}
+				EnvironmentConfig::Instance()->WriteObject(wxT("Variables"), &vars);
+				wxSetEnv(wxT("WX_INCL_HOME"), strWx + wxT("\\include"));
 			}
 
 			if(strMingw.IsEmpty() == false) {
+				// Add the installation include paths
 				pathEnv << wxT(";") << strMingw << wxT("\\bin");
+				wxSetEnv(wxT("MINGW_INCL_HOME"), strMingw);
 			}
 		}
 
@@ -366,20 +415,18 @@ bool App::OnInit()
 	cfg->ReadObject(wxT("GeneralInfo"), &inf);
 
 	bool showSplash = inf.GetFlags() & CL_SHOW_SPLASH ? true : false;
+	
+	m_splash = NULL;
 	if (showSplash) {
 		wxBitmap bitmap;
 		wxString splashName(mgr->GetStarupDirectory() + wxT("/images/splashscreen.png"));
 		if (bitmap.LoadFile(splashName, wxBITMAP_TYPE_PNG)) {
-			wxString mainTitle;
-			mainTitle << wxT("v1.0.") << SvnRevision;
-			m_splash = new SplashScreen(bitmap, mainTitle, wxEmptyString,
-			                            wxSPLASH_CENTRE_ON_SCREEN|wxSPLASH_TIMEOUT,
-			                            3000, NULL, -1, wxDefaultPosition, wxDefaultSize,
-			                            style);
-			wxTheApp->Yield();
+			wxString mainTitle = CODELITE_VERSION_STR;
+			wxBitmap splash = clDrawSplashBitmap(bitmap, mainTitle);
+			m_splash = new clSplashScreen(splash);
 		}
 	}
-
+	
 	// Create the main application window (a dialog in this case)
 	// NOTE: Vertical dimension comprises the caption bar.
 	//       Horizontal dimension has to take into account the thin
@@ -390,15 +437,9 @@ bool App::OnInit()
 
 	// update the accelerators table
 	ManagerST::Get()->UpdateMenuAccelerators();
-
 	m_pMainFrame->Show(TRUE);
 	SetTopWindow(m_pMainFrame);
 
-	if ( showSplash ) {
-		m_splash->Close(true);
-		m_splash->Destroy();
-	}
-
 	long lineNumber(0);
 	parser.Found(wxT("l"), &lineNumber);
 	if(lineNumber > 0){
@@ -551,3 +592,4 @@ bool App::CheckSingularity(const wxCmdLineParser &parser, const wxString &curdir
 	}
 	return true;
 }
+
