@@ -23,6 +23,7 @@
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 #include "wx/numdlg.h"
+#include "wx/regex.h"
 #include "breakpointsmgr.h"
 #include "debuggermanager.h"
 #include "manager.h"
@@ -36,6 +37,7 @@ bool BreakptMgr::AddBreakpointByLineno(const wxString& file, const int lineno, c
 {
 	BreakpointInfo bp;
 	bp.Create(file, lineno, GetNextID());
+	bp.origin = BO_Editor;
 	if (is_temp) {
 		bp.bp_type = BP_type_tempbreak;
 		bp.is_temp = true;
@@ -47,15 +49,20 @@ bool BreakptMgr::AddBreakpointByLineno(const wxString& file, const int lineno, c
 
 bool BreakptMgr::AddBreakpoint(const BreakpointInfo &bp)
 {
+	if(bp.file.IsEmpty() && bp.function_name.IsEmpty() && bp.memory_address.IsEmpty() == false) {
+		// no function nor file?
+		// do nothing then
+		return true;
+	}
+
 	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
 	if (dbgr && dbgr->IsRunning()) {
 		// If the debugger is already running, tell it we want a new bp
 		// If not, they'll all be added together when the debugger does start
 		bool contIsNeeded = PauseDebuggerIfNeeded();
+
 		dbgr->Break(bp);
-		// dbgr->Break(bp) doesn't set the ignore/disabled/etc states
-		// but we can't do it now, as we don't yet know the debugger_id
-		// However it will happen later, in SetBreakpointDebuggerID
+
 		if (contIsNeeded) {
 			dbgr->Continue();
 		}
@@ -73,7 +80,11 @@ bool BreakptMgr::AddBreakpoint(const BreakpointInfo &bp)
 		}
 	}
 
-	if( !alreadyExist ) m_bps.push_back(newBreakpoint);
+	// Add this breakpoint to the list only if it does not already exist
+	// in the breakpoint list
+	if( !alreadyExist ) {
+		m_bps.push_back(newBreakpoint);
+	}
 
 	DeleteAllBreakpointMarkers();
 	RefreshBreakpointMarkers();
@@ -96,6 +107,10 @@ void BreakptMgr::AddBreakpoint()
 	}
 
 	if (AddBreakpoint(dlg.b)) {
+		IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
+		if ((!dlg.b.is_enabled) && dbgr && dbgr->IsRunning()) {
+			SetBPEnabledState(dlg.b.debugger_id, dlg.b.is_enabled);
+		}
 		wxString msg;
 		if (dlg.b.bp_type == BP_type_watchpt) {
 			msg = _("Watchpoint successfully added");
@@ -127,7 +142,7 @@ unsigned int BreakptMgr::GetBreakpoints(std::vector<BreakpointInfo>& li, const w
 
 // Try to find a breakpoint on this line/file to match the data
 // Returns whether >0 was found, with matches stored in the vector
-bool BreakptMgr::GetMatchingBreakpoints(std::vector<BreakpointInfo>& li, const wxString &fileName, const int lineno, enum BP_type bp_type)
+bool BreakptMgr::GetMatchingBreakpoints(std::vector<BreakpointInfo>& li, const wxString &fileName, const int lineno, enum BreakpointType bp_type)
 {
 	std::vector<BreakpointInfo> allonline;	// Start by finding all on the line
 	if ( ! GetBreakpoints(allonline, fileName, lineno) ) {
@@ -200,14 +215,11 @@ void BreakptMgr::DeleteAllBreakpointMarkers()
 // Refresh all line-type breakpoint markers in all editors
 void BreakptMgr::RefreshBreakpointMarkers()
 {
-	std::set<wxString> filenames = GetFilesWithBreakpointMarkers();
-	std::set<wxString>::iterator filenames_iter = filenames.begin();
-	for (; filenames_iter != filenames.end(); ++filenames_iter) {
-		LEditor* editor = Frame::Get()->GetMainBook()->FindEditor(*filenames_iter);
-		if (editor) {
-			DoRefreshFileBreakpoints(editor);
-		}
-	}
+	std::vector<LEditor*> editors;
+	Frame::Get()->GetMainBook()->GetAllEditors( editors );
+
+	for(size_t i=0; i<editors.size(); i++)
+		DoRefreshFileBreakpoints(editors.at(i));
 }
 
 // Delete all breakpoint markers for this file, then re-mark with the currently-correct marker
@@ -215,10 +227,9 @@ void BreakptMgr::DoRefreshFileBreakpoints(LEditor* editor)
 {
 	// Load the file's line-type bps into fi, and make a set of their line-numbers
 	std::multimap<int, BreakpointInfo> bps;
-	std::vector<BreakpointInfo>::iterator iter = m_bps.begin();
-	for (; iter != m_bps.end(); ++iter) {
-		BreakpointInfo b = *iter;
-		if ((b.file == editor->GetFileName().GetFullPath()) && (b.lineno != -1)) {
+	for(size_t i=0; i<m_bps.size(); i++) {
+		BreakpointInfo b = m_bps.at(i);
+		if ((editor->GetFileName() == b.file) && (b.lineno != -1)) {
 			bps.insert(std::pair<int, BreakpointInfo>(b.lineno, b));
 		}
 	}
@@ -253,21 +264,23 @@ void BreakptMgr::DoProvideBestBP_Type(LEditor* editor, const std::vector<Breakpo
 	}
 	// If there's an enabled bp of any sort, it beats all disabled ones
 	// Otherwise, BP_type_break > BP_type_tempbreak > BP_type_cmdlistbreak > BP_type_condbreak > BP_type_ignoredbreak
-	int values[BP_LAST_MARKED_ITEM+2]; // Allow for BP_type_invalid = -1 & BP_type_none = 0
+	int values[BP_LAST_MARKED_ITEM+1]; // Allow for BP_type_none = 0
 	values[BP_type_break] = 100;
 	values[BP_type_tempbreak] = 90;
 	values[BP_type_cmdlistbreak] = 80;
 	values[BP_type_condbreak] = 70;
 	values[BP_type_ignoredbreak] = 60;
 	values[BP_type_none] = 0;
-	values[BP_type_invalid] = -1;
 
-	BP_type best = BP_type_none;
+	BreakpointType best = BP_type_none;
 	int best_value = 0;
 	bool is_disabled = false;
 	std::vector<BreakpointInfo>::const_iterator iter = li.begin();
 	for (; iter != li.end(); ++iter) {
-		BP_type bpt = iter->bp_type;
+		BreakpointType bpt = iter->bp_type;
+		if (bpt == BP_type_invalid) {
+			continue;
+		}
 		int val = values[bpt];
 		if (!iter->is_enabled) {
 			val /= 2;	// Halving the value for disability means that abled always outranks disabled, without otherwise interfering with the order
@@ -291,18 +304,53 @@ void BreakptMgr::DoProvideBestBP_Type(LEditor* editor, const std::vector<Breakpo
 }
 
 // Invalidates each debugger_id
-void BreakptMgr::ClearBP_debugger_ids()
+void BreakptMgr::DebuggerStopped()
 {
-	std::vector<BreakpointInfo>::iterator iter = m_bps.begin();
-	for (; iter != m_bps.end(); iter++) {
-		BreakpointInfo bp = *iter;
+	std::vector<BreakpointInfo> newList;
+	for (size_t i=0; i<m_bps.size(); i++) {
+		BreakpointInfo &bp = m_bps.at(i);
 		bp.debugger_id = -1;
+
+		// We collect all the breakpoints which their origin
+		// was the editor
+		if(bp.origin == BO_Editor) {
+			newList.push_back(bp);
+		}
 	}
+
+	for (size_t i=0; i<m_pendingBreakpointsList.size(); i++) {
+		BreakpointInfo &bp = m_pendingBreakpointsList.at(i);
+		bp.debugger_id = -1;
+
+		// We collect all the breakpoints which their origin
+		// was the editor
+		if(bp.origin == BO_Editor) {
+			newList.push_back(bp);
+		}
+	}
+
+	m_bps.clear();
+	m_pendingBreakpointsList.clear();
+
+	m_bps = newList;
+	UnDisableAllBreakpoints();
+}
+
+// Since a bp can't be created disabled, enable them all here when the debugger stops
+// That way they're guaranteed all to be enabled when it starts again
+void BreakptMgr::UnDisableAllBreakpoints()
+{
+	for (size_t i=0; i<m_bps.size(); i++) {
+		BreakpointInfo &bp = m_bps.at(i);
+		bp.is_enabled = true;
+	}
+
+	RefreshBreakpointMarkers(); // Make this visible to the user
 }
 
 bool BreakptMgr::DelBreakpoint(const int id)
 {
-	int index = FindBreakpointById(id);
+	int index = FindBreakpointById(id, m_bps);
 	if (index == wxNOT_FOUND) {
 		return false;
 	}
@@ -346,11 +394,53 @@ int BreakptMgr::DelBreakpointByLineno(const wxString& file, const int lineno)
 	return (DelBreakpoint(bid) == true ? true : -1); // Use -1 as an arbitrary failed-to-delete flag
 }
 
+void BreakptMgr::ApplyPendingBreakpoints()
+{
+	if (!PendingBreakpointsExist()) {
+		return; // Nothing to do
+	}
+
+	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
+	if (!(dbgr && dbgr->IsRunning())) {
+		return; // If the debugger isn't running, there's no point (and we shouldn't have reached here anyway)
+	}
+	bool contIsNeeded = PauseDebuggerIfNeeded();
+
+	for (size_t i=m_pendingBreakpointsList.size(); i>0; --i) {
+		BreakpointInfo bp = m_pendingBreakpointsList.at(i-1);
+
+		// First check to see if the debugger already accepted this one
+		// The answer should be no, as acceptance should have removed it from this list
+		int index = FindBreakpointById(bp.internal_id, m_bps);
+		if (index != wxNOT_FOUND) {
+			// Hmm. See if there's a valid debugger_id. If so, the bp *was* accepted, and shouldn't be on the pending list
+			if (m_bps.at(index).debugger_id != -1) {
+				m_pendingBreakpointsList.erase(m_pendingBreakpointsList.begin()+i-1);
+			}
+			continue;	// The bp hasn't been assessed yet; it's probably pointless to try to reapply it atm
+		}
+
+		// This bp didn't 'take' the first time; "..try, try again" :p
+		dbgr->Break(bp);
+		m_bps.push_back(bp);
+	}
+
+	if (contIsNeeded) {
+		dbgr->Continue();
+	}
+
+	RefreshBreakpointMarkers();
+}
+
 void BreakptMgr::DelAllBreakpoints()
 {
 	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
-	if (dbgr) {
+	if (dbgr && dbgr->IsRunning()) {
+		bool contIsNeeded = PauseDebuggerIfNeeded();
 		dbgr->RemoveAllBreaks();
+		if (contIsNeeded) {
+			dbgr->Continue();
+		}
 	}
 
 	// Delete all markers before clearing m_bps, otherwise we won't know which files they were in
@@ -358,8 +448,33 @@ void BreakptMgr::DelAllBreakpoints()
 	m_bps.clear();
 }
 
-// The debugger labels each breakpoint with an id. DbgGdb::Poke grabs this
-// and passes it here via DbgCmdHandlerBp::ProcessOutput and Manager::UpdateBpAdded.
+// Toggle a breakpoint's enabled state
+bool BreakptMgr::ToggleEnabledStateByLineno(const wxString& file, const int lineno)
+{
+	wxString msg(_("Select the breakpoint that you want to alter"));
+	int bid = GetDesiredBreakpointIfMultiple(file, lineno, msg);
+	if (bid == wxID_CANCEL || bid == BP_type_none) {
+		return false;
+	}
+
+	int index = FindBreakpointById(bid, m_bps);
+
+	// sanity
+	if (index < 0 || index >= (int)m_bps.size()) {
+		wxLogMessage(wxT("ToggleEnabledStateByLineno(): Insane index"));
+		return false;
+	}
+
+	if (! SetBPEnabledState(bid, !m_bps.at(index).is_enabled)) {
+		return false;
+	}
+
+	m_bps.at(index).is_enabled = ! m_bps.at(index).is_enabled;
+	RefreshBreakpointMarkers();
+	return true;
+}
+
+// The debugger labels each breakpoint with an id and passes it here via Manager::UpdateBpAdded.
 // By storing it as BreakpointInfo::debugger_id, we can be certain that this bp refers to the gdb breakpoint
 void BreakptMgr::SetBreakpointDebuggerID(const int internal_id, const int debugger_id)
 {
@@ -377,16 +492,21 @@ void BreakptMgr::SetBreakpointDebuggerID(const int internal_id, const int debugg
 
 				// add message to the debugger tab
 				ManagerST::Get()->UpdateAddLine(msg);
+				DeleteAllBreakpointMarkers(); // Must do this before the erase, otherwise the last-bp-in-file will be missed
 				m_bps.erase(iter);
 
 				// update the UI as well
 				Frame::Get()->GetDebuggerPane()->GetBreakpointView()->Initialize();
-
+				RefreshBreakpointMarkers();
 				return;
 			}
 			// Otherwise store the valid debugger_id
 			iter->debugger_id = debugger_id;
-
+			// Remove the bp from  the 'pending' array
+			int index = FindBreakpointById(internal_id, m_pendingBreakpointsList);
+			if (index != wxNOT_FOUND) {
+				m_pendingBreakpointsList.erase(m_pendingBreakpointsList.begin()+index);
+			}
 			// update the UI as well
 			Frame::Get()->GetDebuggerPane()->GetBreakpointView()->Initialize();
 			return;
@@ -395,6 +515,44 @@ void BreakptMgr::SetBreakpointDebuggerID(const int internal_id, const int debugg
 	wxLogMessage(wxT("SetBreakpointDebuggerID(): Failed to match internal_id to debugger_id"));
 }
 
+// Set a breakpoint's ignore count
+bool BreakptMgr::IgnoreByLineno(const wxString& file, const int lineno)
+{
+	wxString msg(_("Select the breakpoint to have its ignore-count changed"));
+	int bid = GetDesiredBreakpointIfMultiple(file, lineno, msg);
+	if (bid == wxID_CANCEL || bid == BP_type_none) {
+		return false;
+	}
+
+	int index = FindBreakpointById(bid, m_bps);
+
+	// sanity
+	if (index < 0 || index >= (int)m_bps.size()) {
+		wxLogMessage(wxT("IgnoreByLineno(): Insane index"));
+		return false;
+	}
+
+	BreakpointInfo bp = m_bps.at(index);
+	if (bp.bp_type == BP_type_invalid) {
+		return false;
+	}
+
+	long newvalue = wxGetNumberFromUser( _("Please enter the new ignore-count"), wxT(""), wxT("Set ignore-count"), bp.ignore_number, 0, 1000000);
+	if ((newvalue == -1) || (newvalue == (long)bp.ignore_number)) {
+		return false;
+	}
+
+	if (! SetBPIgnoreCount(bid, newvalue)) {
+		return false;
+	}
+
+	m_bps.at(index).ignore_number = newvalue;
+	// Explicitly set the best type, in case the user just reset a previously-ignored bp
+	SetBestBPType(m_bps.at(index));
+	RefreshBreakpointMarkers();
+	return true;
+}
+
 void BreakptMgr::EditBreakpointByLineno(const wxString& file, const int lineno)
 {
 	wxString msg(_("Select the breakpoint to edit"));
@@ -403,7 +561,7 @@ void BreakptMgr::EditBreakpointByLineno(const wxString& file, const int lineno)
 		return;
 	}
 
-	int index = FindBreakpointById(bid);
+	int index = FindBreakpointById(bid, m_bps);
 	if (index == wxNOT_FOUND) {
 		return;
 	}
@@ -448,17 +606,44 @@ void BreakptMgr::EditBreakpoint(int index, bool &bpExist)
 		return ;
 	}
 
+	// We've got our altered dlg.b If the debugger's running, we can update it now
+	// Otherwise, it'll be automatically inserted correctly when the debugger starts
 	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
 	if (dbgr && dbgr->IsRunning()) {
-		// Update the bp by deleting/replacing
+		if (CanThisBreakpointBeUpdated(dlg.b, bp)) {
+			if (dlg.b.ignore_number != bp.ignore_number) {
+				if (! SetBPIgnoreCount(dlg.b.debugger_id, dlg.b.ignore_number)) {
+					return;	// Harsh, but what else can one do?
+				}
+			}
+			if (dlg.b.is_enabled != bp.is_enabled) {
+				if (! SetBPEnabledState(dlg.b.debugger_id, dlg.b.is_enabled)) {
+					return ;
+				}
+			}
+			if (dlg.b.conditions != bp.conditions) {
+				if (! SetBPConditon(dlg.b)) {
+					return ;
+				}
+			}
+			if (dlg.b.commandlist != bp.commandlist) {
+				if (! SetBPCommands(dlg.b)) {
+					return ;
+				}
+			}
+		} else {
+			// If it can't be updated (because gdb wouldn't be able to cope with the change), replace
 		bool contIsNeeded = PauseDebuggerIfNeeded();
 		dbgr->RemoveBreak(bp.debugger_id);
 		dbgr->Break(dlg.b);
-
+			// dbgr->Break(bp) doesn't set the ignore/disabled/etc states
+			// but we can't do it now, as we don't yet know the debugger_id
+			// However it will happen later, in SetBreakpointDebuggerID
 		if (contIsNeeded) {
 			dbgr->Continue();
 		}
 	}
+	}
 
 	// Replace the old data with the new, in m_bps
 	m_bps.at(index) = dlg.b;
@@ -466,25 +651,88 @@ void BreakptMgr::EditBreakpoint(int index, bool &bpExist)
 	RefreshBreakpointMarkers();
 }
 
-/*------------------------------- Implementation -------------------------------*/
-
-// Tell the debugger about the ignore count, conditions etc
-void BreakptMgr::DoBreakpointExtras(BreakpointInfo &bp)
+void BreakptMgr::ReconcileBreakpoints(const std::vector<BreakpointInfo>& li)
 {
-//	if (bp.ignore_number) {
-//		SetBPIgnoreCount(bp.debugger_id, bp.ignore_number);
-//	}
-//	if (! bp.is_enabled) {
-//		SetBPEnabledState(bp.debugger_id, bp.is_enabled);
-//	}
-//	if (! bp.conditions.IsEmpty()) {
-//		SetBPConditon(bp);
-//	}
-//	if (! bp.commandlist.IsEmpty()) {
-//		SetBPCommands(bp);
-//	}
+	std::vector<BreakpointInfo> updated_bps;
+	std::vector<BreakpointInfo>::const_iterator li_iter = li.begin();
+	for (; li_iter != li.end(); ++li_iter) {
+		int index = FindBreakpointById(li_iter->debugger_id, m_bps);
+		if (index == wxNOT_FOUND) {
+			// This will happen e.g. if a bp was auto-set on Main()
+			// If so, its internal_id will be invalid
+			BreakpointInfo bp = *li_iter;
+			bp.internal_id = GetNextID();
+			updated_bps.push_back(bp);
+
+		} else {
+			// We've match the debugger_id from -break-list with a bp
+			// Update the ignore-count, then store it in a new vector
+			BreakpointInfo bp = m_bps.at(index);
+			bp.ignore_number  = li_iter->ignore_number;
+			bp.what           = li_iter->what;
+			bp.at             = li_iter->at;
+
+			// Remove it from the m_bps list
+			m_bps.erase(m_bps.begin()+index);
+
+			SetBestBPType(bp);  // as this might have just changed
+			updated_bps.push_back(bp);
+		}
+	}
+	// All the still-existing bps have been added to updated_bps
+	// So throw away m_bps (which will contain stale bps) and replace with the new vector
+	// First though, delete all markers. Otherwise, if the last in a file has been deleted...
+	DeleteAllBreakpointMarkers();
+
+	// All the stale breakpoints should be assigned to the 'm_pendingBreakpointList'
+	m_pendingBreakpointsList = m_bps;
+
+	m_bps.clear();
+	SetBreakpoints(updated_bps);
+
+	RefreshBreakpointMarkers();
+	// update the Breakpoints pane too
+	Frame::Get()->GetDebuggerPane()->GetBreakpointView()->Initialize();
 }
 
+	// When a a breakpoint is hit, see if it's got a command-list that needs faking
+void BreakptMgr::BreakpointHit(int id)
+{
+	int index = FindBreakpointById(id, m_bps);
+	if ((index == wxNOT_FOUND) || (index >= FIRST_INTERNAL_ID)) {
+		return;
+	}
+
+	BreakpointInfo bp = m_bps.at(index);
+	if (bp.commandlist.IsEmpty()) {
+		return;
+	}
+
+	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
+	if (dbgr && dbgr->IsRunning()) {
+		// A likely command, presumably at the end of the command-list, is 'continue' or 'cont'
+		// Filter this out and do it separately, otherwise Manager::UpdateLostControl isn't called to blank the indicator
+		static wxRegEx reContinue(wxT("(([[:space:]]|(^))((cont$)|(continue)))"));
+		bool needsCont = false;
+		wxString commands = bp.commandlist;
+		if (reContinue.IsValid() && reContinue.Matches(commands)) {
+			size_t start, len;
+			if (reContinue.GetMatch(&start,&len)) {
+				commands = commands.Left(start);
+				needsCont = true;
+			}
+		}
+		if (! commands.IsEmpty()) {	// Just in case someone's _only_ command is 'continue' !
+			dbgr->ExecuteCmd(commands);
+		}
+		if (needsCont) {
+			dbgr->Continue();
+		}
+	}
+}
+
+/*------------------------------- Implementation -------------------------------*/
+
 // Get a breakpoint on this line. If multiple bps, ask the user to select
 int BreakptMgr::GetDesiredBreakpointIfMultiple(const wxString &fileName, const int lineno, const wxString msg /*=wxT("")*/)
 {
@@ -536,7 +784,7 @@ std::set<wxString> BreakptMgr::GetFilesWithBreakpointMarkers()
 	for (; iter != m_bps.end(); ++iter) {
 		// If this bp is a lineno or function type, add its file to the set
 		wxString fileName = iter->file;
-		if ((!fileName.IsEmpty()) && (iter->memory_address == -1)) {
+		if ((!fileName.IsEmpty()) && (iter->memory_address.IsEmpty())) {
 			filenames.insert(fileName);
 		}
 	}
@@ -544,15 +792,15 @@ std::set<wxString> BreakptMgr::GetFilesWithBreakpointMarkers()
 	return filenames;
 }
 
-int BreakptMgr::FindBreakpointById(const int id)
+int BreakptMgr::FindBreakpointById(const int id, const std::vector<BreakpointInfo>& li)
 {
-	std::vector<BreakpointInfo>::iterator iter = m_bps.begin();
-	for (; iter != m_bps.end(); ++iter) {
+	std::vector<BreakpointInfo>::const_iterator iter = li.begin();
+	for (; iter != li.end(); ++iter) {
 		BreakpointInfo b = *iter;
 		// If we were passed an id > 10000, it must be internal
 		int this_id = (id > FIRST_INTERNAL_ID ? b.internal_id : b.debugger_id);
 		if (id == this_id) {
-			return (int)(iter - m_bps.begin());
+			return (int)(iter - li.begin());
 		}
 	}
 
@@ -572,6 +820,70 @@ bool BreakptMgr::CanThisBreakpointBeUpdated(const BreakpointInfo &a, const Break
 	return (temp == b);
 }
 
+bool BreakptMgr::SetBPIgnoreCount(const int bid, const int ignorecount)
+{
+	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
+	if (dbgr && dbgr->IsRunning()) {
+		// If the debugger is already running, tell it about the new ignore level
+		// If not, it'll happen automatically when the debugger does start
+		bool contIsNeeded = PauseDebuggerIfNeeded();
+		bool result = dbgr->SetIgnoreLevel(bid, ignorecount);
+		if (contIsNeeded) {
+			dbgr->Continue();
+		}
+		return result;
+	}
+	return true;
+}
+
+bool BreakptMgr::SetBPCommands(const BreakpointInfo& bp)
+{
+	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
+	if (dbgr && dbgr->IsRunning()) {
+		// If the debugger is already running, tell it about the commands
+		// If not, it'll happen automatically when the debugger does start
+		bool contIsNeeded = PauseDebuggerIfNeeded();
+		bool result = dbgr->SetCommands(bp);
+		if (contIsNeeded) {
+			dbgr->Continue();
+		}
+		return result;
+	}
+	return false;
+}
+
+bool BreakptMgr::SetBPConditon(const BreakpointInfo& bp)
+{
+	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
+	if (dbgr && dbgr->IsRunning()) {
+		// If the debugger is already running, tell it about the condition
+		// If not, it'll happen automatically when the debugger does start
+		bool contIsNeeded = PauseDebuggerIfNeeded();
+		bool result = dbgr->SetCondition(bp);
+		if (contIsNeeded) {
+			dbgr->Continue();
+		}
+		return result;
+	}
+	return false;
+}
+
+bool BreakptMgr::SetBPEnabledState(const int bid, const bool enable)
+{
+	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
+	if (dbgr && dbgr->IsRunning()) {
+		// If the debugger is already running, tell it about the new ignore level
+		// If not, it'll happen automatically when the debugger does start
+		bool contIsNeeded = PauseDebuggerIfNeeded();
+		bool result = dbgr->SetEnabledState(bid, enable);
+		if (contIsNeeded) {
+			dbgr->Continue();
+		}
+		return result;
+	}
+	return true;
+}
+
 void BreakptMgr::SetBestBPType(BreakpointInfo& bp)
 {
 	if (bp.bp_type == BP_type_watchpt) {
@@ -639,7 +951,101 @@ void BreakptMgr::SaveSession(SessionEntry& session)
 void BreakptMgr::LoadSession(const SessionEntry& session)
 {
 	const std::vector<BreakpointInfo>& breakpoints = session.GetBreakpoints();
-	for (std::vector<BreakpointInfo>::const_iterator itr = breakpoints.begin(); itr != breakpoints.end(); ++itr)
-		AddBreakpoint(*itr);
+	for (std::vector<BreakpointInfo>::const_iterator itr = breakpoints.begin(); itr != breakpoints.end(); ++itr) {
+		BreakpointInfo bp = *itr;
+		bp.internal_id = GetNextID();
+		bp.origin      = BO_Editor;
+		AddBreakpoint(bp);
+	}
+	RefreshBreakpointMarkers();
 }
 
+void BreakptMgr::DragBreakpoint(LEditor* editor, int line, wxBitmap bitmap)
+{
+	// See if there's a bp marker under the cursor. If so, let the user drag it
+	std::vector<BreakpointInfo> lineBPs;
+	if (GetBreakpoints(lineBPs, editor->GetFileName().GetFullPath(), line+1) == 0) {
+		return;
+	}
+
+	m_dragImage = new myDragImage(editor, bitmap, lineBPs);
+	m_dragImage->StartDrag();
+}
+
+void BreakptMgr::DropBreakpoint(std::vector<BreakpointInfo>& BPs, int newline)
+{
+	// We've received back the vector of bps that we passed to DragBreakpoint()
+	// We need to remove one from m_bps, and from the debugger if running,
+	// then add it back with the lineno altered to newline
+	wxString msg(_("Select the breakpoint that you want to move"));
+	int bid = GetDesiredBreakpointIfMultiple(BPs[0].file, BPs[0].lineno, msg);
+	if (bid == wxID_CANCEL || bid == BP_type_none) {
+		return;
+	}
+
+	int index = FindBreakpointById(bid, m_bps);
+	BreakpointInfo bp = *(m_bps.begin()+index);
+	bp.lineno = newline+1;
+	// AddBreakpoint() doesn't do the disabled state: it can't, and there's no good way round this
+	// So if the dragged bp was disabled, we need to enable it here, otherwise it retains its disabled icon
+	bp.is_enabled = true;
+
+	if (DelBreakpoint(bid)) {
+		AddBreakpoint(bp);
+		Frame::Get()->GetDebuggerPane()->GetBreakpointView()->Initialize();
+	}
+}
+
+//---------------------------------------------------------
+
+myDragImage::myDragImage(LEditor* ed, wxBitmap btmp, std::vector<BreakpointInfo>& BPs)
+	: wxDragImage(btmp, wxCURSOR_POINT_LEFT), editor(ed), bitmap(btmp), lineBPs(BPs)
+{
+	// In theory, we could pass a blank cursor to wxDragImage::BeginDrag
+	// In practice, that doesn't seem to work, so do it here, and undo it in OnEndDrag()
+	oldcursor = editor->SetCursor(wxCursor(wxCURSOR_BLANK));
+}
+
+bool myDragImage::StartDrag()
+{
+	BeginDrag( wxPoint(0, 0),editor, false );
+	wxPoint pt = editor->ScreenToClient(wxGetMousePosition());
+	// Store the initial x position, as we know it must be in the bp margin
+	m_startx = pt.x;
+	Move(pt);
+	Show();
+
+	return false;
+}
+
+void myDragImage::OnMotion(wxMouseEvent& event)
+{
+	Move(event.GetPosition());
+	event.Skip();
+}
+
+void myDragImage::OnEndDrag(wxMouseEvent& event)
+{
+	Hide();
+	EndDrag();
+	editor->SetCursor(oldcursor);
+	editor->Disconnect(wxEVT_MOTION, wxMouseEventHandler(myDragImage::OnMotion), NULL, this);
+	editor->Disconnect(wxEVT_LEFT_UP, wxMouseEventHandler(myDragImage::OnEndDrag), NULL, this);
+
+	// If the cursor is within spitting distance of the bp margin, assume it's a genuine drop
+	wxPoint pt = event.GetPosition();
+	if ((pt.x < 0) || ((pt.x - m_startx) > 10)) {
+		return;
+	}
+	// Find the desired new line, and check it's different from the previous
+	long pos = editor->PositionFromPoint(pt);
+	int newline = editor->LineFromPosition(pos);
+	if ((newline+1) != lineBPs[0].lineno) {
+		ManagerST::Get()->GetBreakpointsMgr()->DropBreakpoint(lineBPs, newline);
+	}
+}
+
+void BreakptMgr::RefreshBreakpointsForEditor(LEditor* editor)
+{
+	DoRefreshFileBreakpoints( editor );
+}
