@@ -33,14 +33,21 @@
 #include "cl_editor.h"
 #include "sessionmanager.h"
 #include "wx/arrstr.h"
+#include "wx/dragimag.h"
+
+class myDragImage;
 
 #define FIRST_INTERNAL_ID 10000
 
 class BreakptMgr
 {
-	std::vector<BreakpointInfo> m_bps;
+	std::vector<BreakpointInfo> m_bps;        // The vector of breakpoints
+	std::vector<BreakpointInfo> m_pendingBreakpointsList; // These are any breakpoints that the debugger won't (yet) accept (often because they're in a plugin)
+
 	int NextInternalID;		// Used to give each bp a unique internal ID. Start at 10k to avoid confusion with gdb's IDs
 
+	myDragImage* m_dragImage;
+
 	// Delete all breakpoint markers for this file, then re-mark with the currently-correct marker
 	void DoRefreshFileBreakpoints(LEditor* editor);
 	void DoProvideBestBP_Type(LEditor* editor, const std::vector<BreakpointInfo>& li);	// Tells the editor which is the most appropriate bp marker to show
@@ -50,17 +57,16 @@ class BreakptMgr
 	void DeleteAllBreakpointMarkers();
 	std::set<wxString> GetFilesWithBreakpointMarkers();
 
-	int FindBreakpointById(const int id);
+	/**
+	 * Return the index of the bp with the passed id, in the vector that will normally be m_bps
+	 */
+	int FindBreakpointById(const int id, const std::vector<BreakpointInfo>& li);
+
 	/**
 	 * Can gdb accept this alteration, or will be bp have to be replaced?
 	 */
 	bool CanThisBreakpointBeUpdated(const BreakpointInfo &bp1, const BreakpointInfo &bp2) const;
 
-	/**
-	 * Tell the debugger about the ignore count, conditions etc
-	 */
-	void DoBreakpointExtras(BreakpointInfo &bp);
-
 	/**
 	 * Get id of the breakpoint on this line. If multiple bps, ask the user to select
 	 */
@@ -71,6 +77,26 @@ class BreakptMgr
 	 */
 	void SetBestBPType(BreakpointInfo& bp);
 
+	/**
+	 * Ignore this bp
+	 */
+	bool SetBPIgnoreCount(const int bid, const int ignorecount);
+
+	/**
+	 * Enable or Disable this breakpoint
+	 */
+	bool SetBPEnabledState(const int bid, const bool enable);
+
+	/**
+	 * Set this breakpoint's condition to that in bp.condition
+	 */
+	bool SetBPConditon(const BreakpointInfo &bp);
+
+	/**
+	 * Set this breakpoint's command-list to that in bp.commandlist
+	 */
+	bool SetBPCommands(const BreakpointInfo &bp);
+
 	/**
 	 * Clear the list of breakpoints
 	 */
@@ -88,6 +114,11 @@ public:
 		Clear();
 	}
 
+	/**
+	 * @brief refresh the breakpoints marker for a given editor
+	 * @param editor
+	 */
+	void RefreshBreakpointsForEditor(LEditor* editor);
 
 	/**
 	 * Refresh all line-type breakpoint markers in all editors
@@ -100,6 +131,27 @@ public:
 	 */
 	void SetBreakpoints(const std::vector<BreakpointInfo>& bps);
 
+	/**
+	 * @brief Store list of breakpoints in the pending-breakpoints list
+	 * @param bps
+	 */
+	void SetPendingBreakpoints(const std::vector<BreakpointInfo>& bps) {
+		m_pendingBreakpointsList.clear(); m_pendingBreakpointsList = bps;
+	}
+
+	/**
+	 * @brief Returns true if there are pending breakpoints
+	 */
+	bool PendingBreakpointsExist() {
+		return ! m_pendingBreakpointsList.empty();
+	}
+
+	/**
+	 * Send again to the debugger any breakpoints that weren't accepted by the debugger the first time
+	 *  (e.g. because they're inside a plugin)
+	 */
+	void ApplyPendingBreakpoints();
+
 	/**
 	 * @brief delete all stored breakpoints related to file. this method should does not update the
 	 * debugger, so it must be called *before* starting the debugger
@@ -136,11 +188,21 @@ public:
 	 */
 	int DelBreakpointByLineno(const wxString& file, const int lineno);
 
+	/**
+	 * Toggle a breakpoint's enabled state
+	 */
+	bool ToggleEnabledStateByLineno(const wxString& file, const int lineno);
+
 	/**
 	 * Summon the BreakptProperties dialog for a bp
 	 */
 	void EditBreakpointByLineno(const wxString& file, const int lineno);
 
+	/**
+	 * Set a breakpoint's ignore count
+	 */
+	bool IgnoreByLineno(const wxString& file, const int lineno);
+
 	/**
 	 * return list of breakpoints
 	 */
@@ -162,18 +224,30 @@ public:
 	 * return whether this line has a breakpoint of type bp_type
 	 * Any matches found are returned in li
 	 */
-	bool GetMatchingBreakpoints(std::vector<BreakpointInfo>& li, const wxString &fileName, const int lineno, enum BP_type bp_type);
+	bool GetMatchingBreakpoints(std::vector<BreakpointInfo>& li, const wxString &fileName, const int lineno, enum BreakpointType bp_type);
 
 	/**
 	 * Returns a string containing details of any breakpoints on this line
 	 */
 	wxString GetTooltip(const wxString& fileName, const int lineno);
 
+	/**
+	 * Update the m_bps with what the debugger really contains
+	 * from vector of breakpoints acquired from -break-list
+	 */
+	void ReconcileBreakpoints(const std::vector<BreakpointInfo>& li);
+
 	/**
 	 * Clears the debugger_ids of all breakpoints.
 	 * Called when the debugger has stopped, so they're  no longer valid
 	 */
-	void ClearBP_debugger_ids();
+	void DebuggerStopped();
+
+	/**
+	 * Since a bp can't be created disabled, enable them all here when the debugger stops
+	 * That way they're guaranteed all to be enabled when it starts again
+	 */
+	void UnDisableAllBreakpoints();
 
 	/**
 	 * remove all breakpoints
@@ -191,6 +265,28 @@ public:
 	 */
 	bool PauseDebuggerIfNeeded();
 
+	/**
+	 * Notification from the debugger that breakpoint id was just hit
+	 */
+	void BreakpointHit(int id);
+
+	/**
+	 * Starts 'drag'n'drop' for breakpoints
+	 */
+	void DragBreakpoint(LEditor* editor, int line, wxBitmap bitmap);
+
+	/**
+	 * The 'drop' bit of breakpoints 'drag'n'drop'
+	 */
+	void DropBreakpoint(std::vector<BreakpointInfo>& BPs, int newline);
+
+	/**
+	 * Getter for the myDragImage pointer
+	 */
+	myDragImage* GetDragImage() {
+		return m_dragImage;
+	}
+
 	/**
 	 * Get a unique id for a breakpoint, to use when the debugger isn't running
 	 */
@@ -206,4 +302,20 @@ public:
 	 */
 	 void LoadSession(const SessionEntry& session);
 };
+
+class myDragImage  :  public wxDragImage, public wxEvtHandler
+{
+	LEditor* editor;
+	wxBitmap bitmap;
+	std::vector<BreakpointInfo> lineBPs;
+	int m_startx; // The initial x position
+	wxCursor oldcursor;
+
+public:
+	myDragImage(LEditor* ed, wxBitmap bitmap, std::vector<BreakpointInfo>& BPs);
+	bool StartDrag();
+	void OnMotion(wxMouseEvent& event);
+	void OnEndDrag(wxMouseEvent& event);
+
+};
 #endif //BREAKPOINTS_MANAGER_H
