@@ -41,8 +41,18 @@
 #include "breakpointsmgr.h"
 #include "plugin.h"
 
+#define DEBUGGER_INDICATOR          11
+#define MATCH_INDICATOR             10
+#define USER_INDICATOR              3
+#define HYPERLINK_INDICATOR         4
+
 class wxFindReplaceDialog;
 class CCBox;
+class clEditorTipWindow;
+
+enum sci_annotation_styles {
+	eAnnotationStyleError = 128, eAnnotationStyleWarning
+};
 
 // NB The following are sci markers, which are zero based. So smt_bookmark is actually the eighth of them (important when masking it!)
 // If you add another type here, watch out for smt_LAST_BP_TYPE; and you need also to add to the enum 'marker_mask_type' below
@@ -67,23 +77,14 @@ enum calltip_type { ct_function_hover, ct_debugger, ct_function_proto, ct_breakp
 * Holds which marker and mask are associated with each breakpoint type
 */
 typedef struct _BPtoMarker {
-	enum BP_type bp_type;	// An enum of possible break/watchpoint types. In debugger.h
+	enum BreakpointType bp_type;	// An enum of possible break/watchpoint types. In debugger.h
 	sci_marker_types marker;
 	marker_mask_type mask;
 	sci_marker_types marker_disabled;
 	marker_mask_type mask_disabled;
 } BPtoMarker;
 
-/**
- * @class LEditorState
- * a container for the editor state (breakpoints, bookmarks and current position)
- */
-struct LEditorState {
-	std::vector<int> breakpoints;
-	std::vector<int> markers;
-	int caretPosition;
-};
-
+extern const wxEventType wxCMD_EVENT_REMOVE_MATCH_INDICATOR;
 /**
  * \ingroup LiteEditor
  * LEditor CodeLite editing component based on Scintilla
@@ -105,38 +106,50 @@ struct LEditorState {
  */
 class LEditor : public wxScintilla, public IEditor
 {
-	wxFileName m_fileName;
-	wxString m_project;
-	wxStopWatch m_watch;
-	ContextBasePtr m_context;
-	wxMenu *m_rightClickMenu;
-	std::vector<wxMenuItem*> m_dynItems;
-	std::vector<BPtoMarker> m_BPstoMarkers;
-
-	// static cache among editors to keep track of jumping between editors
-	static FindReplaceDialog *m_findReplaceDlg;
-	static FindReplaceData m_findReplaceData;
-	int m_lastMatchPos;
-	static std::map<wxString, int> ms_bookmarkShapes;
-	bool m_popupIsOn;
-	time_t m_modifyTime;
-	std::map<int, wxString> m_customCmds;
-	CCBox *m_ccBox;
-	bool m_isVisible;
-	int m_hyperLinkIndicatroStart;
-	int m_hyperLinkIndicatroEnd;
-	int m_hyperLinkType;
-	bool m_hightlightMatchedBraces;
-	bool m_autoAddMatchedBrace;
+	wxFileName                                  m_fileName;
+	wxString                                    m_project;
+	wxStopWatch                                 m_watch;
+	ContextBasePtr                              m_context;
+	wxMenu *                                    m_rightClickMenu;
+	std::vector<wxMenuItem*>                    m_dynItems;
+	std::vector<BPtoMarker>                     m_BPstoMarkers;
+	static FindReplaceDialog *                  m_findReplaceDlg;
+	static FindReplaceData                      m_findReplaceData;
+	int                                         m_lastMatchPos;
+	static std::map<wxString, int>              ms_bookmarkShapes;
+	bool                                        m_popupIsOn;
+	time_t                                      m_modifyTime;
+	std::map<int, wxString>                     m_customCmds;
+	CCBox *                                     m_ccBox;
+	bool                                        m_isVisible;
+	int                                         m_hyperLinkIndicatroStart;
+	int                                         m_hyperLinkIndicatroEnd;
+	int                                         m_hyperLinkType;
+	bool                                        m_hightlightMatchedBraces;
+	bool                                        m_autoAddMatchedBrace;
 	std::map<int, std::vector<BreakpointInfo> > m_breakpointsInfo;
-	bool m_autoAdjustHScrollbarWidth;
-	calltip_type m_calltipType;
+	bool                                        m_autoAdjustHScrollbarWidth;
+	calltip_type                                m_calltipType;
+	bool                                        m_reloadingFile;
+	bool                                        m_disableSmartIndent;
+	bool                                        m_disableSemicolonShift;
+	clEditorTipWindow*                        m_functionTip;
 
 public:
 	static FindReplaceData &GetFindReplaceData() {
 		return m_findReplaceData;
 	}
 
+	void SetReloadingFile(const bool& reloadingFile) {
+		this->m_reloadingFile = reloadingFile;
+	}
+	const bool& GetReloadingFile() const {
+		return m_reloadingFile;
+	}
+
+	clEditorTipWindow* GetFunctionTip() {
+		return m_functionTip;
+	}
 public:
 	/// Construct a LEditor object
 	LEditor(wxWindow* parent);
@@ -151,6 +164,13 @@ public:
 	// this function prompts the user for selecting file name
 	bool SaveFileAs();
 
+	void SetDisableSmartIndent(bool disableSmartIndent) {
+		this->m_disableSmartIndent = disableSmartIndent;
+	}
+
+	bool GetDisableSmartIndent() const {
+		return m_disableSmartIndent;
+	}
 	/**
 	 * @brief set the EOL mode of the file by applying this logic:
 	 * - if the file has content, use the current cotext EOL
@@ -159,18 +179,6 @@ public:
 	 */
 	void SetEOL();
 
-	/**
-	 * @brief save the editor current state (in terms of breakpoints, bookmarks & current position)
-	 * @param s state structure
-	 */
-	void GetEditorState(LEditorState &s);
-
-	/**
-	 * @brief set the editor current state (in terms of breakpoints, bookmarks & current position)
-	 * @param s state structure
-	 */
-	void SetEditorState(const LEditorState &s);
-
 	void CompleteWord();
 
 	// set this editor file name
@@ -265,8 +273,10 @@ public:
 	void FindNextMarker();
 	// Find previous marker and move cursor to that line
 	void FindPrevMarker();
+
 	// Replace all
 	bool ReplaceAll();
+	bool ReplaceAllExactMatch(const wxString &what, const wxString &replaceWith);
 	// mark all occurances
 	bool MarkAll();
 
@@ -279,8 +289,10 @@ public:
 
 	// Util function
 	int SafeGetChar(int pos);
-	wxChar PreviousChar(const int& pos, int &foundPos, bool wantWhitespace = false);
-	wxChar NextChar(const int& pos, int &foundPos);
+	wxChar   PreviousChar(const int& pos, int &foundPos, bool wantWhitespace = false);
+	wxString PreviousWord(int pos, int &foundPos);
+	wxChar   NextChar    (const int& pos, int &foundPos);
+
 	int  FindString (const wxString &str, int flags, const bool down, long pos);
 
 	bool FindAndSelect();
@@ -345,6 +357,16 @@ public:
 	 */
 	void AddOtherBreakpointType(wxCommandEvent &event);
 
+	/**
+	 * toggle whether the break point at the current line & file is enabled
+	 */
+	void ToggleBreakpointEnablement();
+
+	/**
+	 * Ignore the break point at the current line & file
+	 */
+	void OnIgnoreBreakpoint();
+
 	/**
 	 * Edit a breakpoint in the BreakptProperties dialog
 	 */
@@ -361,12 +383,23 @@ public:
 	 */
 	void DelBreakpoint(int lineno = -1);
 
+	/**
+	 * @brief search the editor for pattern. If pattern is found, the editor will then search for 'what'
+	 * inside the pattern and will select it
+	 * @param pattern pattern to search in the editor
+	 * @param what    sub string of pattern to select
+	 * @param pos     start the search from 'pos'
+	 * @param navmgr  Navigation manager to place browsing recrods
+	 * @return return true if a match was found, false otherwise
+	 */
+	virtual bool FindAndSelect(const wxString &pattern, const wxString &what, int pos, NavMgr *navmgr);
+
 	virtual void UpdateBreakpoints();
 
 	//--------------------------------
 	// breakpoint visualisation
 	//--------------------------------
-	virtual void SetBreakpointMarker(int lineno, BP_type bptype, bool is_disabled, const std::vector<BreakpointInfo>& li);
+	virtual void SetBreakpointMarker(int lineno, BreakpointType bptype, bool is_disabled, const std::vector<BreakpointInfo>& li);
 	virtual void DelAllBreakpointMarkers();
 
 	virtual void HighlightLine(int lineno);
@@ -380,7 +413,9 @@ public:
 	void DoCancelCalltip();
 	int  DoGetOpenBracePos();
 
-	calltip_type GetCalltipType() const {return m_calltipType;}
+	calltip_type GetCalltipType() const {
+		return m_calltipType;
+	}
 
 	//----------------------------------
 	//File modifications
@@ -489,6 +524,8 @@ public:
 	virtual wxString GetSelection();
 	virtual void SelectText(int startPos, int len);
 
+	virtual void SetLexerName(const wxString &lexerName);
+
 	// User Indicators API
 	virtual void SetUserIndicatorStyleAndColour(int style, const wxColour &colour);
 	virtual void SetUserIndicator(int startPos, int len);
@@ -522,6 +559,11 @@ public:
 	//----------------------------------------------------------------------------
 	//----------------------------------------------------------------------------
 
+	/**
+	 * Get editor options. Takes any workspace/project overrides into account
+	 */
+	OptionsConfigPtr GetOptions();
+
 	void SetIsVisible(const bool& isVisible) {
 		this->m_isVisible = isVisible;
 	}
@@ -533,7 +575,7 @@ public:
 
 private:
 	void FillBPtoMarkerArray();
-	BPtoMarker GetMarkerForBreakpt(enum BP_type bp_type);
+	BPtoMarker GetMarkerForBreakpt(enum BreakpointType bp_type);
 	void SetProperties();
 	void DefineMarker(int marker, int markerType, wxColor fore, wxColor back);
 	void SetLineNumberWidth();
@@ -546,13 +588,16 @@ private:
 	// Conevert FindReplaceDialog flags to wxSD flags
 	size_t SearchFlags(const FindReplaceData &data);
 
-	void AddDebuggerContextMenu(wxMenu *menu);
-	void RemoveDebuggerContextMenu(wxMenu *menu);
-	void DoBreakptContextMenu(wxPoint clientPt);
-	void DoMarkHyperlink(wxMouseEvent &event, bool isMiddle);
-	void DoQuickJump(wxMouseEvent &event, bool isMiddle);
+	void    AddDebuggerContextMenu(wxMenu *menu);
+	void    RemoveDebuggerContextMenu(wxMenu *menu);
+	void    DoBreakptContextMenu(wxPoint clientPt);
+	void    DoMarkHyperlink(wxMouseEvent &event, bool isMiddle);
+	void    DoQuickJump(wxMouseEvent &event, bool isMiddle);
+	bool    DoFindAndSelect(const wxString &pattern, const wxString &what, int start_pos, NavMgr *navmgr);
+	wxMenu* DoCreateDebuggerWatchMenu(const wxString &word);
 
 	DECLARE_EVENT_TABLE()
+	void OnRemoveMatchInidicator(wxCommandEvent &e);
 	void OnSavePoint(wxScintillaEvent &event);
 	void OnCharAdded(wxScintillaEvent& event);
 	void OnMarginClick(wxScintillaEvent& event);
@@ -572,7 +617,6 @@ private:
 	void OnFocusLost(wxFocusEvent &event);
 	void OnLeftDClick(wxScintillaEvent &event);
 	void OnPopupMenuUpdateUI(wxUpdateUIEvent &event);
-	void OnDbgQuickWatch(wxCommandEvent &event);
 	void OnDbgAddWatch(wxCommandEvent &event);
 	void OnDbgRunToCursor(wxCommandEvent &event);
 	void OnDbgCustomWatch(wxCommandEvent &event);
