@@ -25,6 +25,9 @@
 
 
 #include "pluginmanager.h"
+#include "cl_editor_tip_window.h"
+#include "implement_parent_virtual_functions.h"
+#include "debuggerasciiviewer.h"
 #include <wx/file.h>
 #include "threebuttondlg.h"
 #include "precompiled_header.h"
@@ -35,7 +38,6 @@
 #include <wx/progdlg.h>
 #include "renamesymboldlg.h"
 #include "cpptoken.h"
-#include "tokendb.h"
 #include "globals.h"
 #include "commentconfigdata.h"
 #include "editor_config.h"
@@ -63,6 +65,7 @@
 #include "workspacetab.h"
 #include "fileview.h"
 #include "refactorindexbuildjob.h"
+#include "new_quick_watch_dlg.h"
 
 //#define __PERFORMANCE
 #include "performance.h"
@@ -160,6 +163,8 @@ BEGIN_EVENT_TABLE(ContextCpp, wxEvtHandler)
 	EVT_MENU(XRCID("move_impl"),                    ContextCpp::OnMoveImpl)
 	EVT_MENU(XRCID("add_impl"),                     ContextCpp::OnAddImpl)
 	EVT_MENU(XRCID("add_multi_impl"),               ContextCpp::OnAddMultiImpl)
+	EVT_MENU(XRCID("add_virtual_impl"),             ContextCpp::OnOverrideParentVritualFunctions)
+	EVT_MENU(XRCID("add_pure_virtual_impl"),        ContextCpp::OnOverrideParentVritualFunctions)
 	EVT_MENU(XRCID("setters_getters"),              ContextCpp::OnGenerateSettersGetters)
 	EVT_MENU(XRCID("add_include_file"),             ContextCpp::OnAddIncludeFile)
 	EVT_MENU(XRCID("rename_function"),              ContextCpp::OnRenameFunction)
@@ -170,7 +175,6 @@ ContextCpp::ContextCpp(LEditor *container)
 		: ContextBase(container)
 		, m_rclickMenu(NULL)
 {
-	ApplySettings();
 	Initialize();
 }
 
@@ -315,28 +319,102 @@ void ContextCpp::AutoIndent(const wxChar &nChar)
 {
 	LEditor &rCtrl = GetCtrl();
 
+	if(rCtrl.GetDisableSmartIndent()) {
+		ContextBase::AutoIndent(nChar);
+		return;
+	}
+
 	int curpos = rCtrl.GetCurrentPos();
 	if (IsComment(curpos) && nChar == wxT('\n')) {
 		AutoAddComment();
 		return;
 	}
+
 	if (IsCommentOrString(curpos)) {
 		ContextBase::AutoIndent(nChar);
 		return;
 	}
+
 	int line = rCtrl.LineFromPosition(curpos);
 	if (nChar == wxT('\n')) {
-		int prevpos = wxNOT_FOUND;
-		wxChar ch = rCtrl.PreviousChar(curpos, prevpos);
+
+		int      prevpos (wxNOT_FOUND);
+		int      foundPos(wxNOT_FOUND);
+		wxString word;
+
+		wxChar ch = rCtrl.PreviousChar(curpos, prevpos );
+		word      = rCtrl.PreviousWord(curpos, foundPos);
+
+		// user hit ENTER after 'else'
+		if ( word == wxT("else") ) {
+			int prevLine = rCtrl.LineFromPosition(prevpos);
+			rCtrl.SetLineIndentation(line, rCtrl.GetIndent() + rCtrl.GetLineIndentation(prevLine));
+			rCtrl.SetCaretAt(rCtrl.GetLineIndentPosition(line));
+			rCtrl.ChooseCaretX(); // set new column as "current" column
+			return;
+		}
+
+		// User typed 'ENTER' immediatly after closing brace ')'
+		if ( prevpos != wxNOT_FOUND && ch == wxT(')') ) {
+
+			long openBracePos          (wxNOT_FOUND);
+			int  posWordBeforeOpenBrace(wxNOT_FOUND);
+
+			if (rCtrl.MatchBraceBack(wxT(')'), prevpos, openBracePos)) {
+				rCtrl.PreviousChar(openBracePos, posWordBeforeOpenBrace);
+				if (posWordBeforeOpenBrace != wxNOT_FOUND) {
+					word = rCtrl.PreviousWord(posWordBeforeOpenBrace, foundPos);
+
+					// c++ expression with single line and should be treated separatly
+					if ( word == wxT("if") || word == wxT("while") || word == wxT("for")) {
+						int prevLine = rCtrl.LineFromPosition(prevpos);
+						rCtrl.SetLineIndentation(line, rCtrl.GetIndent() + rCtrl.GetLineIndentation(prevLine));
+						rCtrl.SetCaretAt(rCtrl.GetLineIndentPosition(line));
+						rCtrl.ChooseCaretX(); // set new column as "current" column
+						return;
+					}
+				}
+			}
+		}
+
+		// User typed 'ENTER' immediatly after colons ':'
+		if ( prevpos != wxNOT_FOUND && ch == wxT(':') ) {
+			int  posWordBeforeColons(wxNOT_FOUND);
+
+			rCtrl.PreviousChar(prevpos, posWordBeforeColons);
+			if (posWordBeforeColons != wxNOT_FOUND) {
+				word = rCtrl.PreviousWord(posWordBeforeColons, foundPos);
+				int prevLine = rCtrl.LineFromPosition(posWordBeforeColons);
+
+				// If we found one of the following keywords, un-indent their line by (foldLevel - 1)*indentSize
+				if ( word == wxT("public") || word == wxT("private") || word == wxT("protected")) {
+
+					ContextBase::AutoIndent(nChar);
+
+					// Indent this line according to the block indentation level
+					int foldLevel = (rCtrl.GetFoldLevel(prevLine) & wxSCI_FOLDLEVELNUMBERMASK) - wxSCI_FOLDLEVELBASE;
+					if (foldLevel) {
+						rCtrl.SetLineIndentation(prevLine, ((foldLevel-1)*rCtrl.GetIndent()) );
+						rCtrl.ChooseCaretX();
+					}
+					return;
+				}
+			}
+		}
+
+		// use the previous line indentation level
 		if (prevpos == wxNOT_FOUND || ch != wxT('{') || IsCommentOrString(prevpos)) {
 			ContextBase::AutoIndent(nChar);
 			return;
 		}
-		//open brace? increase indent size
+
+		// Open brace? increase indent size
 		int prevLine = rCtrl.LineFromPosition(prevpos);
 		rCtrl.SetLineIndentation(line, rCtrl.GetIndent() + rCtrl.GetLineIndentation(prevLine));
 		rCtrl.SetCaretAt(rCtrl.GetLineIndentPosition(line));
+
 	} else if (nChar == wxT('}')) {
+
 		long matchPos = wxNOT_FOUND;
 		if (!rCtrl.MatchBraceBack(wxT('}'), rCtrl.PositionBefore(curpos), matchPos))
 			return;
@@ -344,8 +422,23 @@ void ContextCpp::AutoIndent(const wxChar &nChar)
 		if (secondLine == line)
 			return;
 		rCtrl.SetLineIndentation(line, rCtrl.GetLineIndentation(secondLine));
+
+	} else if (nChar == wxT('{')) {
+		wxString lineString = rCtrl.GetLine(line);
+		lineString.Trim().Trim(false);
+
+		int matchPos = wxNOT_FOUND;
+		wxChar previousChar = rCtrl.PreviousChar(rCtrl.PositionBefore(curpos), matchPos);
+		if(previousChar != wxT('{') && lineString == wxT("{")) {
+			// indent this line accroding to the previous line
+			int line = rCtrl.LineFromPosition(rCtrl.GetCurrentPos());
+			rCtrl.SetLineIndentation(line, rCtrl.GetLineIndentation(line-1));
+			rCtrl.ChooseCaretX();
+		}
 	}
-	rCtrl.ChooseCaretX(); // set new column as "current" column
+
+	// set new column as "current" column
+	rCtrl.ChooseCaretX();
 }
 
 bool ContextCpp::IsCommentOrString(long pos)
@@ -391,7 +484,7 @@ void ContextCpp::RemoveDuplicates(std::vector<TagEntryPtr>& src, std::vector<Tag
 	}
 }
 
-wxString ContextCpp::GetExpression(long pos, bool onlyWord, LEditor *editor)
+wxString ContextCpp::GetExpression(long pos, bool onlyWord, LEditor *editor, bool forCC)
 {
 	bool cont(true);
 	int depth(0);
@@ -531,6 +624,11 @@ wxString ContextCpp::GetExpression(long pos, bool onlyWord, LEditor *editor)
 
 	if (at < 0) at = 0;
 	wxString expr = ctrl->GetTextRange(at, pos);
+	if ( !forCC ) {
+		// If we do not require the expression for CodeCompletion
+		// return the un-touched buffer
+		return expr;
+	}
 
 	//remove comments from it
 	CppScanner sc;
@@ -578,7 +676,7 @@ void ContextCpp::AddMenuDynamicContent(wxMenu *menu)
 	wxString fileName;
 
 	LEditor &rCtrl = GetCtrl();
-	VALIDATE_PROJECT(rCtrl);
+//	VALIDATE_PROJECT(rCtrl);
 
 	wxString menuItemText;
 	wxString line = rCtrl.GetCurLine();
@@ -587,7 +685,7 @@ void ContextCpp::AddMenuDynamicContent(wxMenu *menu)
 	if (IsIncludeStatement(line, &fileName)) {
 
 		PrependMenuItemSeparator(menu);
-		menuItemText << wxT("Open Workspace File \"") << fileName << wxT("\"");
+		menuItemText << wxT("Open Include File \"") << fileName << wxT("\"");
 
 		PrependMenuItem(menu, menuItemText, wxCommandEventHandler(ContextCpp::OnContextOpenDocument));
 		m_selectedWord = fileName;
@@ -612,7 +710,6 @@ void ContextCpp::OnAddIncludeFile(wxCommandEvent &e)
 {
 	wxUnusedVar(e);
 	LEditor &rCtrl = GetCtrl();
-	VALIDATE_PROJECT(rCtrl);
 
 	//get expression
 	int pos = rCtrl.GetCurrentPos();
@@ -776,8 +873,6 @@ void ContextCpp::DisplayFilesCompletionBox(const wxString &word)
 
 	std::sort(files.begin(), files.end(), SFileSort());
 
-	wxLogMessage(wxString::Format(wxT("Completing: %s"), fileName.c_str()));
-
 	if ( files.empty() == false ) {
 		GetCtrl().RegisterImageForKind(wxT("FileCpp"),    m_cppFileBmp);
 		GetCtrl().RegisterImageForKind(wxT("FileHeader"), m_hFileBmp  );
@@ -998,6 +1093,37 @@ void ContextCpp::OnSwapFiles(wxCommandEvent &event)
 	SwapFiles(GetCtrl().GetFileName());
 }
 
+
+void ContextCpp::DoMakeDoxyCommentString(DoxygenComment& dc)
+{
+	LEditor &editor = GetCtrl();
+	CommentConfigData data;
+	EditorConfigST::Get()->ReadObject(wxT("CommentConfigData"), &data);
+
+	wxString blockStart(wxT("/**\n"));
+	if (!data.GetUseSlash2Stars()) {
+		blockStart = wxT("/*!\n");
+	}
+
+	//prepend the prefix to the
+	wxString classPattern = data.GetClassPattern();
+	wxString funcPattern  = data.GetFunctionPattern();
+
+	//replace $(Name) here **before** the call to ExpandAllVariables()
+	classPattern.Replace(wxT("$(Name)"), dc.name);
+	funcPattern.Replace(wxT("$(Name)"), dc.name);
+
+	classPattern = ExpandAllVariables(classPattern, WorkspaceST::Get(), editor.GetProjectName(), wxEmptyString, editor.GetFileName().GetFullPath());
+	funcPattern = ExpandAllVariables(funcPattern, WorkspaceST::Get(), editor.GetProjectName(), wxEmptyString, editor.GetFileName().GetFullPath());
+
+	dc.comment.Replace(wxT("$(ClassPattern)"), classPattern);
+	dc.comment.Replace(wxT("$(FunctionPattern)"), funcPattern);
+
+	//close the comment
+	dc.comment << wxT(" */\n");
+	dc.comment.Prepend(blockStart);
+}
+
 void ContextCpp::OnInsertDoxyComment(wxCommandEvent &event)
 {
 	wxUnusedVar(event);
@@ -1017,33 +1143,12 @@ void ContextCpp::OnInsertDoxyComment(wxCommandEvent &event)
 		keyPrefix = wxT('@');
 	}
 
-	wxString blockStart(wxT("/**\n"));
-	if (!data.GetUseSlash2Stars()) {
-		blockStart = wxT("/*!\n");
-	}
-
 	DoxygenComment dc = TagsManagerST::Get()->GenerateDoxygenComment(editor.GetFileName().GetFullPath(), lineno, keyPrefix);
 	//do we have a comment?
 	if (dc.comment.IsEmpty())
 		return;
 
-	//prepend the prefix to the
-	wxString classPattern = data.GetClassPattern();
-	wxString funcPattern  = data.GetFunctionPattern();
-
-	//replace $(Name) here **before** the call to ExpandAllVariables()
-	classPattern.Replace(wxT("$(Name)"), dc.name);
-	funcPattern.Replace(wxT("$(Name)"), dc.name);
-
-	classPattern = ExpandAllVariables(classPattern, WorkspaceST::Get(), editor.GetProjectName(), wxEmptyString, editor.GetFileName().GetFullPath());
-	funcPattern = ExpandAllVariables(funcPattern, WorkspaceST::Get(), editor.GetProjectName(), wxEmptyString, editor.GetFileName().GetFullPath());
-
-	dc.comment.Replace(wxT("$(ClassPattern)"), classPattern);
-	dc.comment.Replace(wxT("$(FunctionPattern)"), funcPattern);
-
-	//close the comment
-	dc.comment << wxT(" */\n");
-	dc.comment.Prepend(blockStart);
+	DoMakeDoxyCommentString(dc);
 
 	editor.InsertTextWithIndentation(dc.comment, lineno);
 
@@ -1141,7 +1246,8 @@ void ContextCpp::OnGenerateSettersGetters(wxCommandEvent &event)
 		return;
 
 	TagEntryPtr tag = classtags.at(0);
-	if (tag->GetFile() != editor.GetFileName().GetFullPath()) {
+	if (tag->GetFile().CmpNoCase(editor.GetFileName().GetFullPath()) != 0) {
+
 		wxString msg;
 		msg << wxT("This file does not seem to contain the declaration for '") << tag->GetName() << wxT("'\n");
 		msg << wxT("The declaration of '") << tag->GetName() << wxT("' is located at '") << tag->GetFile() << wxT("'\n");
@@ -1168,49 +1274,24 @@ void ContextCpp::OnGenerateSettersGetters(wxCommandEvent &event)
 		if (code.IsEmpty() == false) {
 			editor.InsertTextWithIndentation(code, lineno);
 		}
-		IPlugin *formatter = PluginManager::Get()->GetPlugin(wxT("CodeFormatter"));
-		if (formatter) {
-			// code formatter is available, format the current source file
-			wxCommandEvent e(wxEVT_COMMAND_MENU_SELECTED, XRCID("format_source"));
-			Frame::Get()->GetEventHandler()->AddPendingEvent(e);
-		}
+
+		if ( s_dlg->GetFormatText() )
+			DoFormatActiveEditor();
 	}
 }
 
 void ContextCpp::OnKeyDown(wxKeyEvent &event)
 {
 	LEditor &ctrl = GetCtrl();
-	int pos = ctrl.GetCurrentPos();
-
-	if (m_ct && m_ct->Count() && ctrl.CallTipActive() && ctrl.GetCalltipType() == ct_function_proto) {
-
+	if (ctrl.GetFunctionTip()->IsActive()) {
 		switch (event.GetKeyCode()) {
-
-		case WXK_UP: {
-			int index = DoGetCalltipParamterIndex();
-			int start(wxNOT_FOUND), len(wxNOT_FOUND);
-			wxString tip = m_ct->Prev();
-
-			if (index != wxNOT_FOUND) {
-				m_ct->GetHighlightPos(index, start, len);
-			}
-			ctrl.CallTipCancel();
-			ctrl.DoShowCalltip(pos, tip, ct_function_proto, start, len);
-		}
-		return;
-
-		case WXK_DOWN: {
-			int index = DoGetCalltipParamterIndex();
-			int start(wxNOT_FOUND), len(wxNOT_FOUND);
-			wxString tip = m_ct->Next();
-
-			if (index != wxNOT_FOUND) {
-				m_ct->GetHighlightPos(index, start, len);
-			}
-			ctrl.CallTipCancel();
-			ctrl.DoShowCalltip(pos, tip, ct_function_proto, start, len);
-		}
-		return;
+		case WXK_UP:
+			ctrl.GetFunctionTip()->SelectPrev(DoGetCalltipParamterIndex());
+			return;
+		
+		case WXK_DOWN:
+			ctrl.GetFunctionTip()->SelectNext(DoGetCalltipParamterIndex());
+			return;
 		}
 	}
 	event.Skip();
@@ -1289,13 +1370,24 @@ void ContextCpp::OnSciUpdateUI(wxScintillaEvent &event)
 void ContextCpp::OnDbgDwellEnd(wxScintillaEvent &event)
 {
 	wxUnusedVar(event);
-	Manager *mgr = ManagerST::Get();
-	mgr->DbgCancelQuickWatchTip();
+	// remove the debugger indicator
+	GetCtrl().SetIndicatorCurrent(DEBUGGER_INDICATOR);
+	GetCtrl().IndicatorClearRange(0, GetCtrl().GetLength());
 }
 
 void ContextCpp::OnDbgDwellStart(wxScintillaEvent & event)
 {
 	static wxRegEx reCppIndentifier(wxT("[a-zA-Z_][a-zA-Z0-9_]*"));
+
+	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
+	if (dbgr && dbgr->IsRunning() && ManagerST::Get()->DbgCanInteract()) {
+		if ( ManagerST::Get()->GetDisplayVariableDialog()->IsShown() ) {
+			// a 'Quick Show dialog' is already shown!
+			// dont show another tip
+			return;
+		}
+	}
+
 	wxPoint pt;
 	wxString word;
 	pt.x = event.GetX();
@@ -1308,92 +1400,39 @@ void ContextCpp::OnDbgDwellStart(wxScintillaEvent & event)
 			return;
 		}
 
-		long start(0), end(0);
+		long end(0);
 		long sel_start(0), sel_end(0);
 
-		start = ctrl.WordStartPosition(pos, true);
-		end   = ctrl.WordEndPosition(pos, true);
+		end = ctrl.WordEndPosition  (pos, true);
 
 		// if thers is no selected text, use the word calculated from the caret position
 		if (!ctrl.GetSelectedText().IsEmpty()) {
 			// selection is not empty, use it
 			sel_start = ctrl.GetSelectionStart();
-			sel_end = ctrl.GetSelectionEnd();
+			sel_end   = ctrl.GetSelectionEnd  ();
+			word = ctrl.GetTextRange(sel_start, sel_end);
+
+			// Mark the code we are going to try and show tip for
+			GetCtrl().SetIndicatorCurrent(DEBUGGER_INDICATOR);
+			GetCtrl().IndicatorFillRange(sel_start, sel_end - sel_start);
+
+		} else {
+			word = GetExpression(end, false, &GetCtrl(), false);
+			word.Trim().Trim(false);
+
+			// Mark the code we are going to try and show tip for
+			GetCtrl().SetIndicatorCurrent(DEBUGGER_INDICATOR);
+			GetCtrl().IndicatorFillRange(end - word.length(), word.Length());
 		}
 
-		// incase the cursor is placed inside the selected text,
-		// use the entire selected text and not only the "word"
-		if (pos >= sel_start && pos <= sel_end) {
-			start = sel_start;
-			end = sel_end;
-		}
-
-		word = ctrl.GetTextRange(start, end);
 		if (word.IsEmpty()) {
 			return;
 		}
-
-		//make sure that this is indeed a variable name
-		if (!reCppIndentifier.Matches(word)) {
-			return;
-		}
 	} else {
 		return;
 	}
 
-	IDebugger *dbgr = DebuggerMgr::Get().GetActiveDebugger();
-	if (dbgr) {
-		DebuggerSettingsData data;
-		DebuggerConfigTool::Get()->ReadObject(wxT("DebuggerCommands"), &data);
-		std::vector<DebuggerCmdData> cmds = data.GetCmds();
-		// before asking for the debugger for tooltip, try
-		// to query the debugger about the type of the
-		// evaluated string
-		wxString type;
-		wxString command(word);
-
-		if (dbgr->ResolveType(word, type)) {
-
-			// gdb returns usually expression like:
-			// const string &, so in order to get the actual type
-			// we construct a valid expression by appending a valid identifier followed by a semi colon.
-			wxString expression;
-			//	wxLogMessage(word + wxT(" resolved into: ") + type);
-
-			expression << wxT("/^");
-			expression << type;
-			expression << wxT(" someValidName;");
-			expression << wxT("$/");
-
-			Variable variable;
-			if (LanguageST::Get()->VariableFromPattern(expression, wxT("someValidName"), variable)) {
-				type = _U(variable.m_type.c_str());
-				for (size_t i=0; i<cmds.size(); i++) {
-					DebuggerCmdData cmd = cmds.at(i);
-					if (cmd.GetName() == type) {
-						// prepare the string to be evaluated
-						command = cmd.GetCommand();
-						command.Replace(wxT("$(Variable)"), word);
-						break;
-					}
-				}
-			}
-		} else {
-			//	wxLogMessage(wxT("ResolveType failed for ") + word);
-		}
-
-		wxString output;
-		if (dbgr->GetTip(command, output)) {
-			// cancel any old calltip and display the new one
-			ctrl.DoCancelCalltip();
-
-			// wxScintilla's tooltip does not present \t characters
-			// so we replace it with 4 spaces
-			output.Replace(wxT("\t"), wxT("    "));
-
-			ctrl.DoShowCalltip(event.GetPosition(), output, ct_debugger);
-		}
-	}
+	dbgr->CreateVariableObject( word, DBG_USERR_QUICKWACTH );
 }
 
 int ContextCpp::FindLineToAddInclude()
@@ -1569,6 +1608,61 @@ bool ContextCpp::DoGetFunctionBody(long curPos, long &blockStartPos, long &block
 	        (blockStartPos != wxNOT_FOUND);
 }
 
+void ContextCpp::OnOverrideParentVritualFunctions(wxCommandEvent& e)
+{
+	LEditor &rCtrl = GetCtrl();
+	VALIDATE_WORKSPACE();
+
+	// Get the text from the file start point until the current position
+	int      pos      = rCtrl.GetCurrentPos();
+	wxString context  = rCtrl.GetTextRange(0, pos);
+	bool     onlyPure = e.GetId() == XRCID("add_pure_virtual_impl");
+
+	wxString scopeName = TagsManagerST::Get()->GetScopeName(context);
+	if (scopeName.IsEmpty() || scopeName == wxT("<global>")) {
+		wxMessageBox(_("Cant resolve scope properly. Found <") + scopeName + wxT(">"), wxT("CodeLite"), wxICON_INFORMATION|wxOK);
+		return;
+	}
+
+	// get map of all unimlpemented methods
+	std::vector<TagEntryPtr> protos;
+	TagsManagerST::Get()->GetUnOverridedParentVirtualFunctions(scopeName, onlyPure, protos);
+
+	// No methods to add?
+	if (protos.empty())
+		return;
+
+	// Locate the swapped file
+	wxString targetFile(rCtrl.GetFileName().GetFullPath());
+	FindSwappedFile(rCtrl.GetFileName(), targetFile);
+
+	CommentConfigData data;
+	EditorConfigST::Get()->ReadObject(wxT("CommentConfigData"), &data);
+
+	//get doxygen comment based on file and line
+	wxChar keyPrefix(wxT('\\'));
+	if (data.GetUseShtroodel()) {
+		keyPrefix = wxT('@');
+	}
+
+	ImplementParentVirtualFunctionsDialog dlg(wxTheApp->GetTopWindow(), scopeName, protos, keyPrefix, this);
+	dlg.m_textCtrlImplFile->SetValue(targetFile);
+	if (dlg.ShowModal() == wxID_OK) {
+		wxString implFile = dlg.m_textCtrlImplFile->GetValue();
+		wxString impl     = dlg.GetImpl();
+		wxString decl     = dlg.GetDecl();
+		rCtrl.InsertText(rCtrl.GetCurrentPos(), decl);
+		if (dlg.m_checkBoxFormat->IsChecked())
+			DoFormatActiveEditor();
+
+		// Open teh implementation file and format it if needed
+		OpenFileAppendAndFormat(implFile, impl, dlg.m_checkBoxFormat->IsChecked());
+	}
+
+	// Restore this file to be the active one
+	Frame::Get()->GetMainBook()->OpenFile(GetCtrl().GetFileName().GetFullPath());
+}
+
 void ContextCpp::OnAddMultiImpl(wxCommandEvent &e)
 {
 	wxUnusedVar(e);
@@ -1612,14 +1706,14 @@ void ContextCpp::OnAddMultiImpl(wxCommandEvent &e)
 		targetFile = rCtrl.GetFileName().GetFullPath();
 	}
 
-	MoveFuncImplDlg *dlg = new MoveFuncImplDlg(NULL, body, targetFile);
-	if (dlg->ShowModal() == wxID_OK) {
+	MoveFuncImplDlg dlg(NULL, body, targetFile);
+	dlg.SetTitle(wxT("Implement All Un-Implemented Functions"));
+	if (dlg.ShowModal() == wxID_OK) {
 		//get the updated data
-		targetFile = dlg->GetFileName();
-		body = dlg->GetText();
+		targetFile = dlg.GetFileName();
+		body = dlg.GetText();
 		OpenFileAndAppend(targetFile, body);
 	}
-	dlg->Destroy();
 }
 
 void ContextCpp::OnAddImpl(wxCommandEvent &e)
@@ -1698,17 +1792,35 @@ void ContextCpp::OnAddImpl(wxCommandEvent &e)
 			targetFile = rCtrl.GetFileName().GetFullPath();
 		}
 
-		MoveFuncImplDlg *dlg = new MoveFuncImplDlg(NULL, body, targetFile);
-		if (dlg->ShowModal() == wxID_OK) {
+		MoveFuncImplDlg dlg(NULL, body, targetFile);
+		dlg.SetTitle(wxT("Add Function Implmentation"));
+		if (dlg.ShowModal() == wxID_OK) {
 			//get the updated data
-			targetFile = dlg->GetFileName();
-			body = dlg->GetText();
+			targetFile = dlg.GetFileName();
+			body       = dlg.GetText();
 			OpenFileAndAppend(targetFile, body);
 		}
-		dlg->Destroy();
 	}
 }
 
+void ContextCpp::DoFormatActiveEditor()
+{
+	IPlugin *formatter = PluginManager::Get()->GetPlugin(wxT("CodeFormatter"));
+	if (formatter) {
+		// code formatter is available, format the current source file
+		wxCommandEvent e(wxEVT_COMMAND_MENU_SELECTED, XRCID("format_source"));
+		Frame::Get()->GetEventHandler()->ProcessEvent(e);
+	}
+}
+
+bool ContextCpp::OpenFileAppendAndFormat(const wxString& fileName, const wxString& text, bool format)
+{
+	OpenFileAndAppend(fileName, text);
+	if (format)
+		DoFormatActiveEditor();
+	return true;
+}
+
 bool ContextCpp::OpenFileAndAppend ( const wxString &fileName, const wxString &text )
 {
 	LEditor *editor = Frame::Get()->GetMainBook()->OpenFile(fileName, wxEmptyString, 0);
@@ -1716,13 +1828,10 @@ bool ContextCpp::OpenFileAndAppend ( const wxString &fileName, const wxString &t
 		return false;
 
 	// if needed, append EOL
-	// in an ideal world, we would like that the file will be terminated with 2xEOL
 	if (editor->GetText().EndsWith(editor->GetEolString()) == false) {
 		editor->AppendText(editor->GetEolString());
 	}
-	if (editor->GetText().EndsWith(editor->GetEolString() + editor->GetEolString()) == false) {
-		editor->AppendText(editor->GetEolString());
-	}
+
 	int lineNum = editor->GetLineCount();
 	editor->GotoLine ( lineNum-1 );
 	editor->AppendText ( text );
@@ -1844,7 +1953,7 @@ void ContextCpp::ApplySettings()
 
 	// Update the control
 	LEditor &rCtrl = GetCtrl();
-	rCtrl.SetLexer(lexPtr->GetLexerId());
+	rCtrl.SetLexer((int)lexPtr->GetLexerId());
 
 	wxString keyWords = lexPtr->GetKeyWords(0);
 	keyWords.Replace(wxT("\n"), wxT(" "));
@@ -1905,6 +2014,7 @@ void ContextCpp::ApplySettings()
 
 	//delete uneeded commands
 	rCtrl.CmdKeyClear('/', wxSCI_SCMOD_CTRL);
+	rCtrl.CmdKeyClear('/', wxSCI_SCMOD_CTRL|wxSCI_SCMOD_SHIFT);
 
 	// update word characters to allow '~' as valid word character
 	rCtrl.SetWordChars(wxT("~_abcdefghijklmnopqrstuvwxyzABCDEFGHIJKLMNOPQRSTUVWXYZ0123456789"));
@@ -2040,14 +2150,6 @@ void ContextCpp::OnRenameFunction(wxCommandEvent& e)
 	std::list<CppToken>::iterator iter = tokens.begin();
 	int counter(0);
 
-	// enable tags caching to boost performance
-	TagsOptionsData data = TagsManagerST::Get()->GetCtagsOptions();
-	data.SetFlags(data.GetFlags() | CC_CACHE_WORKSPACE_TAGS);
-	TagsManagerST::Get()->SetCtagsOptions( data );
-
-	// clear the caching flag
-	data.SetFlags(data.GetFlags() & ~(CC_CACHE_WORKSPACE_TAGS));
-
 	for (; iter != tokens.end(); iter++) {
 		CppToken token = *iter;
 		editor->Create(wxEmptyString, token.getFilename());
@@ -2061,9 +2163,6 @@ void ContextCpp::OnRenameFunction(wxCommandEvent& e)
 			// user clicked 'Cancel'
 			prgDlg->Destroy();
 			editor->Destroy();
-
-			// restore CC flags
-			TagsManagerST::Get()->SetCtagsOptions( data );
 			return;
 		}
 
@@ -2092,9 +2191,6 @@ void ContextCpp::OnRenameFunction(wxCommandEvent& e)
 		}
 	}
 
-	// restore CC flags
-	TagsManagerST::Get()->SetCtagsOptions( data );
-
 	editor->Destroy();
 	prgDlg->Destroy();
 
@@ -2143,7 +2239,7 @@ void ContextCpp::ReplaceInFiles ( const wxString &word, std::list<CppToken> &li
 			file_name = token.getFilename();
 		}
 		LEditor *editor = Frame::Get()->GetMainBook()->OpenFile(token.getFilename(), wxEmptyString, 0);
-		if (editor != NULL && editor->GetFileName().GetFullPath() == wxFileName(token.getFilename()).GetFullPath()) {
+		if (editor != NULL && (editor->GetFileName().GetFullPath().CmpNoCase(token.getFilename()) == 0) ) {
 			editor->SetSelection ( token.getOffset(), token.getOffset()+token.getName().Len() );
 			if ( editor->GetSelectionStart() != editor->GetSelectionEnd() ) {
 				editor->ReplaceSelection ( word );
@@ -2247,6 +2343,7 @@ void ContextCpp::OnRetagFile(wxCommandEvent& e)
 	}
 
 	RetagFile();
+	ctrl.SetActive();
 }
 
 void ContextCpp::RetagFile()
@@ -2257,16 +2354,14 @@ void ContextCpp::RetagFile()
 	// incase this file is not cache this function does nothing
 	TagsManagerST::Get()->ClearCachedFile(ctrl.GetFileName().GetFullPath());
 
-	// clear all the queries which holds reference to this file
-	TagsManagerST::Get()->GetWorkspaceTagsCache()->DeleteByFilename(ctrl.GetFileName().GetFullPath());
-
-	// clear also the swapped file
-	wxString targetFile;
-	if (FindSwappedFile(ctrl.GetFileName(), targetFile) && targetFile.IsEmpty() == false) {
-		TagsManagerST::Get()->GetWorkspaceTagsCache()->DeleteByFilename(targetFile);
-	}
-
-	ctrl.SetActive();
+//	// clear all the queries which holds reference to this file
+//	TagsManagerST::Get()->GetWorkspaceTagsCache()->DeleteByFilename(ctrl.GetFileName().GetFullPath());
+//
+//	// clear also the swapped file
+//	wxString targetFile;
+//	if (FindSwappedFile(ctrl.GetFileName(), targetFile) && targetFile.IsEmpty() == false) {
+//		TagsManagerST::Get()->GetWorkspaceTagsCache()->DeleteByFilename(targetFile);
+//	}
 }
 
 void ContextCpp::OnUserTypedXChars(const wxString &word)
@@ -2295,7 +2390,10 @@ void ContextCpp::MakeCppKeywordsTags(const wxString &word, std::vector<TagEntryP
 	                        "do double dynamic_cast else enum explicit export extern false float for friend goto if inline int long mutable namespace "
 	                        "new not not_eq operator or or_eq private protected public register reinterpret_cast return short signed sizeof size_t static "
 	                        "static_cast struct switch template this throw true try typedef typeid typename union unsigned using virtual void volatile "
-	                        "wchar_t while xor xor_eq");
+	                        "wchar_t while xor xor_eq ");
+
+	// add preprocessors
+	cppWords << wxT("ifdef undef define defined include endif elif ifndef ");
 
 	wxString s1(word);
 	wxArrayString wordsArr = wxStringTokenize(cppWords, wxT(" "));
@@ -2314,11 +2412,12 @@ void ContextCpp::MakeCppKeywordsTags(const wxString &word, std::vector<TagEntryP
 wxString ContextCpp::CallTipContent()
 {
 	// if we have an active call tip, return its content
-	if (GetCtrl().CallTipActive() && m_ct && m_ct->Count() > 0) {
-		return m_ct->All();
-	}
+	if (GetCtrl().GetFunctionTip()->IsActive())
+		return GetCtrl().GetFunctionTip()->GetText();
+
 	return wxEmptyString;
 }
+
 void ContextCpp::DoCodeComplete(long pos)
 {
 	long currentPosition = pos;
@@ -2429,18 +2528,19 @@ void ContextCpp::DoCodeComplete(long pos)
 		expr = GetExpression(rCtrl.PositionBefore(currentPosition), false);
 
 		//display function tooltip
-		int word_end = rCtrl.WordEndPosition(end, true);
+		int word_end   = rCtrl.WordEndPosition(end, true);
 		int word_start = rCtrl.WordStartPosition(end, true);
 
 		// get the token
 		wxString word = rCtrl.GetTextRange(word_start, word_end);
-		m_ct = TagsManagerST::Get()->GetFunctionTip(rCtrl.GetFileName(), line, expr, text, word);
-		if (m_ct && m_ct->Count() > 0) {
-			rCtrl.CallTipCancel();
-			int start(wxNOT_FOUND), len(wxNOT_FOUND);
-			m_ct->GetHighlightPos(DoGetCalltipParamterIndex(), start, len);
-			rCtrl.DoShowCalltip(currentPosition, m_ct->First(), ct_function_proto, start, len);
-		}
+		rCtrl.GetFunctionTip()->Add( TagsManagerST::Get()->GetFunctionTip(rCtrl.GetFileName(), line, expr, text, word) );
+		rCtrl.GetFunctionTip()->Highlight(DoGetCalltipParamterIndex());
+		
+		// In an ideal world, we would like our tooltip to be placed 
+		// on top of the caret.
+		wxPoint pt = rCtrl.PointFromPosition(currentPosition);
+		rCtrl.GetFunctionTip()->Activate(pt, rCtrl.GetCurrLineHeight(), rCtrl.StyleGetBackground(wxSCI_C_DEFAULT));
+		
 	} else {
 
 		if (TagsManagerST::Get()->AutoCompleteCandidates(rCtrl.GetFileName(), line, expr, text, candidates)) {
@@ -2484,6 +2584,8 @@ void ContextCpp::DoOpenWorkspaceFile()
 	wxString tmpName(m_selectedWord);
 
 	tmpName.Replace(wxT("\\"), wxT("/"));
+	if (tmpName.Contains(wxT("..")))
+		tmpName = fileName.GetFullName();
 
 	std::vector<wxFileName> files, files2;
 	TagsManagerST::Get()->GetFiles(fileName.GetFullName(), files);
@@ -2577,37 +2679,7 @@ void ContextCpp::OnGotoNextFunction(wxCommandEvent& event)
 
 void ContextCpp::OnCallTipClick(wxScintillaEvent& e)
 {
-	int pos = GetCtrl().GetCurrentPos();
-	switch (e.GetPosition()) {
-	case 1: // Up
-		if (m_ct) {
-			int index = DoGetCalltipParamterIndex();
-			int start(wxNOT_FOUND), len(wxNOT_FOUND);
-			wxString tip = m_ct->Next();
-
-			if (index != wxNOT_FOUND) {
-				m_ct->GetHighlightPos(index, start, len);
-			}
-			GetCtrl().CallTipCancel();
-			GetCtrl().DoShowCalltip(pos, tip, ct_function_proto, start, len);
-		}
-		break;
-	case 2: // down arrow
-		if (m_ct) {
-			int index = DoGetCalltipParamterIndex();
-			int start(wxNOT_FOUND), len(wxNOT_FOUND);
-			wxString tip = m_ct->Prev();
-			if (index != wxNOT_FOUND) {
-				m_ct->GetHighlightPos(index, start, len);
-			}
-			GetCtrl().CallTipCancel();
-			GetCtrl().DoShowCalltip(pos, tip, ct_function_proto, start, len);
-		}
-		break;
-	case 0: // elsewhere
-		GetCtrl().DoCancelCalltip();
-		break;
-	}
+	wxUnusedVar(e);
 }
 
 void ContextCpp::OnCalltipCancel()
@@ -2626,7 +2698,9 @@ int ContextCpp::DoGetCalltipParamterIndex()
 		bool exit_loop(false);
 
 		while ( pos < ctrl.GetCurrentPos() && !exit_loop ) {
-			wxChar ch = ctrl.SafeGetChar(pos);
+			wxChar ch        = ctrl.SafeGetChar(pos);
+			wxChar ch_before = ctrl.SafeGetChar(ctrl.PositionBefore(pos));
+
 			if (IsCommentOrString(pos)) {
 				pos = ctrl.PositionAfter(pos);
 				continue;
@@ -2647,10 +2721,15 @@ int ContextCpp::DoGetCalltipParamterIndex()
 							case wxT('['):
 									depth++;
 				break;
+			case wxT('>'):
+							if ( ch_before == wxT('-') ) {
+						// operator noting to do
+						break;
+					}
+				// fall through
 			case wxT(')'):
-						case wxT('>'):
-							case wxT(']'):
-									depth--;
+						case wxT(']'):
+								depth--;
 				break;
 			default:
 				break;
@@ -2664,13 +2743,40 @@ int ContextCpp::DoGetCalltipParamterIndex()
 void ContextCpp::DoUpdateCalltipHighlight()
 {
 	LEditor &ctrl = GetCtrl();
-	if (ctrl.CallTipActive() && m_ct && m_ct->Count() && ctrl.GetCalltipType() == ct_function_proto) {
-		int index = DoGetCalltipParamterIndex();
-		int start(wxNOT_FOUND), len(wxNOT_FOUND);
+	if (ctrl.GetFunctionTip()->IsActive()) {
+		ctrl.GetFunctionTip()->Highlight(DoGetCalltipParamterIndex());
+	}
+}
 
-		if (index != wxNOT_FOUND) {
-			m_ct->GetHighlightPos(index, start, len);
-			ctrl.CallTipSetHighlight(start, start + len);
+void ContextCpp::SemicolonShift()
+{
+	int foundPos    (wxNOT_FOUND);
+	int semiColonPos(wxNOT_FOUND);
+	LEditor &ctrl = GetCtrl();
+	if (ctrl.NextChar(ctrl.GetCurrentPos(), semiColonPos) == wxT(')')) {
+
+		// test to see if we are inside a 'for' statement
+		long openBracePos          (wxNOT_FOUND);
+		int  posWordBeforeOpenBrace(wxNOT_FOUND);
+
+		if (ctrl.MatchBraceBack(wxT(')'), semiColonPos, openBracePos)) {
+			ctrl.PreviousChar(openBracePos, posWordBeforeOpenBrace);
+			if (posWordBeforeOpenBrace != wxNOT_FOUND) {
+				wxString word = ctrl.PreviousWord(posWordBeforeOpenBrace, foundPos);
+
+				// c++ expression with single line and should be treated separatly
+				if ( word == wxT("for"))
+					return;
+
+				// At the current pos, we got a ';'
+				// at semiColonPos we got ;
+				// switch
+				ctrl.DeleteBack();
+				ctrl.SetCurrentPos(semiColonPos);
+				ctrl.InsertText(semiColonPos, wxT(";"));
+				ctrl.SetCaretAt(semiColonPos+1);
+				ctrl.GetFunctionTip()->Deactivate();
+			}
 		}
 	}
 }
