@@ -23,13 +23,14 @@
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 #include <wx/aui/framemanager.h>
+#include "debuggerasciiviewer.h"
+#include "localstable.h"
 #include "dockablepane.h"
 #include "editor_config.h"
 #include "detachedpanesinfo.h"
  #include "wx/dcbuffer.h"
 #include "memoryview.h"
 #include "debuggerpane.h"
-#include "localvarstree.h"
 #include "simpletable.h"
 #include "listctrlpanel.h"
 #include "wx/xrc/xmlres.h"
@@ -37,21 +38,13 @@
 #include "breakpointdlg.h"
 #include "threadlistpanel.h"
 
-const wxString DebuggerPane::LOCALS = wxT("Locals");
-const wxString DebuggerPane::WATCHES = wxT("Watches");
-const wxString DebuggerPane::FRAMES = wxT("Call Stack");
-const wxString DebuggerPane::BREAKPOINTS = wxT("Breakpoints");
-const wxString DebuggerPane::THREADS = wxT("Threads");
-const wxString DebuggerPane::MEMORY = wxT("Memory");
-
-/*#define ADD_DEBUGGER_PAGE(win, name, bmp) \
-	if( detachedPanes.Index(name) != wxNOT_FOUND ) {\
-		wxAuiPaneInfo info;\
-		DockablePane *pane = new DockablePane(GetParent(), m_book, win, name, bmp, wxSize(200, 200));\
-		m_mgr->AddPane(pane, info.Name(name).Caption(name));\
-	} else {\
-		m_book->AddPage(win, name, bmp, false);\
-	}*/
+const wxString DebuggerPane::LOCALS       = wxT("Locals");
+const wxString DebuggerPane::WATCHES      = wxT("Watches");
+const wxString DebuggerPane::FRAMES       = wxT("Call Stack");
+const wxString DebuggerPane::BREAKPOINTS  = wxT("Breakpoints");
+const wxString DebuggerPane::THREADS      = wxT("Threads");
+const wxString DebuggerPane::MEMORY       = wxT("Memory");
+const wxString DebuggerPane::ASCII_VIEWER = wxT("Ascii Viewer");
 
 #define ADD_DEBUGGER_PAGE(win, name, bmp) \
 	if( detachedPanes.Index(name) != wxNOT_FOUND ) {\
@@ -114,13 +107,16 @@ void DebuggerPane::CreateGUIControls()
 	EditorConfigST::Get()->ReadObject(wxT("DetachedPanesList"), &dpi);
 	detachedPanes = dpi.GetPanes();
 
-	m_localsTree = new LocalVarsTree(m_book, wxID_ANY);
-	ADD_DEBUGGER_PAGE(m_localsTree, LOCALS, wxXmlResource::Get()->LoadBitmap(wxT("locals_view")));
+	m_localsTable = new LocalsTable(m_book);
+	ADD_DEBUGGER_PAGE(m_localsTable, LOCALS, wxXmlResource::Get()->LoadBitmap(wxT("locals_view")));
 
 	//add the watches view
-	m_watchesTable = new SimpleTable(m_book);
+	m_watchesTable = new WatchesTable(m_book);
 	ADD_DEBUGGER_PAGE(m_watchesTable, WATCHES, wxXmlResource::Get()->LoadBitmap(wxT("watches")));
 
+	m_asciiViewer = new DebuggerAsciiViewer(this);
+	ADD_DEBUGGER_PAGE(m_asciiViewer, ASCII_VIEWER, wxXmlResource::Get()->LoadBitmap(wxT("text_view")));
+
 	m_frameList = new ListCtrlPanel(m_book);
 	ADD_DEBUGGER_PAGE(m_frameList, FRAMES, wxXmlResource::Get()->LoadBitmap(wxT("frames")));
 
@@ -148,7 +144,7 @@ void DebuggerPane::SelectTab(const wxString &tabName)
 
 void DebuggerPane::Clear()
 {
-	GetLocalsTree()->Clear();
+	GetLocalsTable()->Clear();
 	GetWatchesTable()->Clear();
 	GetFrameListView()->Clear();
 	GetThreadsView()->Clear();
