@@ -41,7 +41,8 @@ EditorOptionsGeneralGuidesPanel::EditorOptionsGeneralGuidesPanel( wxWindow* pare
 	m_highlightCaretLine->SetValue(options->GetHighlightCaretLine());
 	m_caretLineColourPicker->SetColour(options->GetCaretLineColour());
 	m_choiceEOL->SetStringSelection(options->GetEolMode());
-
+	m_checkBoxHideChangeMarkerMargin->SetValue( options->GetHideChangeMarkerMargin() );
+	m_checkBoxDisableSemicolonShift->SetValue( options->GetDisableSemicolonShift() );
     EnableDisableCaretLineColourPicker();
 
 	switch (options->GetShowWhitspaces()) {
@@ -72,6 +73,8 @@ void EditorOptionsGeneralGuidesPanel::Save(OptionsConfigPtr options)
 	options->SetCaretLineColour(m_caretLineColourPicker->GetColour());
 	options->SetAutoAddMatchedBraces(m_checkBoxAutoCompleteBraces->IsChecked());
 	options->SetEolMode(m_choiceEOL->GetStringSelection());
+	options->SetHideChangeMarkerMargin( m_checkBoxHideChangeMarkerMargin->IsChecked() );
+	options->SetDisableSemicolonShift( m_checkBoxDisableSemicolonShift->IsChecked() );
 
 	// save the whitespace visibility
 	int style(wxSCI_WS_INVISIBLE); // inivisble
