@@ -32,8 +32,8 @@
 #include "replaceinfilespanel.h"
 
 FindInFilesDialog::FindInFilesDialog(wxWindow* parent, wxWindowID id, const FindReplaceData& data)
-: FindInFilesDialogBase(parent, id)
-, m_data(data)
+		: FindInFilesDialogBase(parent, id)
+		, m_data(data)
 {
 	// DirPicker values
 	wxArrayString choices;
@@ -74,23 +74,23 @@ FindInFilesDialog::~FindInFilesDialog()
 
 void FindInFilesDialog::SetSearchData(const SearchData &data)
 {
-    m_data.SetFindString(data.GetFindString());
+	m_data.SetFindString(data.GetFindString());
 
-    size_t flags = 0;
-    flags |= data.IsMatchCase()         ? wxFRD_MATCHCASE         : 0;
-    flags |= data.IsMatchWholeWord()    ? wxFRD_MATCHWHOLEWORD    : 0;
-    flags |= data.IsRegularExpression() ? wxFRD_REGULAREXPRESSION : 0;
-    flags |= data.UseEditorFontConfig() ? wxFRD_USEFONTENCODING   : 0;
-    flags |= data.GetDisplayScope()     ? wxFRD_DISPLAYSCOPE      : 0;
-    m_data.SetFlags(flags);
+	size_t flags = 0;
+	flags |= data.IsMatchCase()         ? wxFRD_MATCHCASE         : 0;
+	flags |= data.IsMatchWholeWord()    ? wxFRD_MATCHWHOLEWORD    : 0;
+	flags |= data.IsRegularExpression() ? wxFRD_REGULAREXPRESSION : 0;
+	flags |= data.UseEditorFontConfig() ? wxFRD_USEFONTENCODING   : 0;
+	flags |= data.GetDisplayScope()     ? wxFRD_DISPLAYSCOPE      : 0;
+	m_data.SetFlags(flags);
 
-    m_findString->SetValue(data.GetFindString());
-    m_matchCase->SetValue(data.IsMatchCase());
-    m_matchWholeWord->SetValue(data.IsMatchWholeWord());
-    m_regualrExpression->SetValue(data.IsRegularExpression());
-    m_fontEncoding->SetValue(data.UseEditorFontConfig());
-    m_printScope->SetValue(data.GetDisplayScope());
-    m_fileTypes->SetValue(data.GetExtensions());
+	m_findString->SetValue(data.GetFindString());
+	m_matchCase->SetValue(data.IsMatchCase());
+	m_matchWholeWord->SetValue(data.IsMatchWholeWord());
+	m_regualrExpression->SetValue(data.IsRegularExpression());
+	m_fontEncoding->SetValue(data.UseEditorFontConfig());
+	m_printScope->SetValue(data.GetDisplayScope());
+	m_fileTypes->SetValue(data.GetExtensions());
 
 	m_listPaths->Clear();
 	const wxArrayString& rootDirs = data.GetRootDirs();
@@ -104,13 +104,13 @@ void FindInFilesDialog::SetSearchData(const SearchData &data)
 
 void FindInFilesDialog::SetRootDir(const wxString &rootDir)
 {
-    m_dirPicker->SetPath(rootDir);
+	m_dirPicker->SetPath(rootDir);
 }
 
 void FindInFilesDialog::DoSearchReplace()
 {
 	SearchData data = DoGetSearchData();
-    data.SetOwner(Frame::Get()->GetOutputPane()->GetReplaceResultsTab());
+	data.SetOwner(Frame::Get()->GetOutputPane()->GetReplaceResultsTab());
 
 	DoSaveOpenFiles();
 	SearchThreadST::Get()->PerformSearch(data);
@@ -136,7 +136,7 @@ SearchData FindInFilesDialog::DoGetSearchData()
 {
 	SearchData data;
 	wxString findStr(m_data.GetFindString());
-	if(m_findString->GetValue().IsEmpty() == false){
+	if (m_findString->GetValue().IsEmpty() == false) {
 		findStr = m_findString->GetValue();
 	}
 
@@ -145,7 +145,7 @@ SearchData FindInFilesDialog::DoGetSearchData()
 	data.SetMatchWholeWord((m_data.GetFlags() & wxFRD_MATCHWHOLEWORD) != 0);
 	data.SetRegularExpression((m_data.GetFlags() & wxFRD_REGULAREXPRESSION) != 0);
 	data.SetUseEditorFontConfig((m_data.GetFlags() & wxFRD_USEFONTENCODING) != 0);
-    data.SetDisplayScope((m_data.GetFlags() & wxFRD_DISPLAYSCOPE) != 0);
+	data.SetDisplayScope((m_data.GetFlags() & wxFRD_DISPLAYSCOPE) != 0);
 
 	wxArrayString rootDirs;
 	for (size_t i = 0; i < m_listPaths->GetCount(); ++i) {
@@ -160,18 +160,17 @@ SearchData FindInFilesDialog::DoGetSearchData()
 	data.SetRootDirs(rootDirs);
 
 	wxArrayString files;
-	for (size_t i = 0; i < rootDirs.GetCount(); ++i)
-	{
+	for (size_t i = 0; i < rootDirs.GetCount(); ++i) {
 		const wxString& rootDir = rootDirs.Item(i);
-		if (rootDir == SEARCH_IN_WORKSPACE){
+		if (rootDir == SEARCH_IN_WORKSPACE) {
 
 			ManagerST::Get()->GetWorkspaceFiles(files);
 
-		} else if (rootDir == SEARCH_IN_PROJECT){
+		} else if (rootDir == SEARCH_IN_PROJECT) {
 
 			ManagerST::Get()->GetProjectFiles(ManagerST::Get()->GetActiveProjectName(), files);
 
-		} else if (rootDir == SEARCH_IN_CURR_FILE_PROJECT){
+		} else if (rootDir == SEARCH_IN_CURR_FILE_PROJECT) {
 
 			wxString project = ManagerST::Get()->GetActiveProjectName();
 
@@ -200,58 +199,58 @@ void FindInFilesDialog::OnClick(wxCommandEvent &event)
 
 	m_data.SetFindString( m_findString->GetValue() );
 
-	if(btnClicked == m_stop){
+	if (btnClicked == m_stop) {
 		SearchThreadST::Get()->StopSearch();
 
-	} else if(btnClicked == m_find){
-		if( findWhat.IsEmpty() ) {
+	} else if (btnClicked == m_find) {
+		if ( findWhat.IsEmpty() ) {
 			return;
 		}
 		DoSearch();
 
-	} else if(btnClicked == m_replaceAll){
-		if( findWhat.IsEmpty() ) {
+	} else if (btnClicked == m_replaceAll) {
+		if ( findWhat.IsEmpty() ) {
 			return;
 		}
 		DoSearchReplace();
 
-	} else if(btnClicked == m_cancel){
+	} else if (btnClicked == m_cancel) {
 		// Hide the dialog
 		DoSaveSearchPaths();
 		Hide();
 
-	} else if(btnClicked == m_matchCase){
-		if(m_matchCase->IsChecked()) {
+	} else if (btnClicked == m_matchCase) {
+		if (m_matchCase->IsChecked()) {
 			flags |= wxFRD_MATCHCASE;
 		} else {
 			flags &= ~(wxFRD_MATCHCASE);
 		}
-	} else if(btnClicked == m_matchWholeWord){
-		if(m_matchWholeWord->IsChecked()) {
+	} else if (btnClicked == m_matchWholeWord) {
+		if (m_matchWholeWord->IsChecked()) {
 			flags |= wxFRD_MATCHWHOLEWORD;
 		} else {
 			flags &= ~(wxFRD_MATCHWHOLEWORD);
 		}
-	} else if(btnClicked == m_regualrExpression){
-		if(m_regualrExpression->IsChecked()) {
+	} else if (btnClicked == m_regualrExpression) {
+		if (m_regualrExpression->IsChecked()) {
 			flags |= wxFRD_REGULAREXPRESSION;
 		} else {
 			flags &= ~(wxFRD_REGULAREXPRESSION);
 		}
-	} else if(btnClicked == m_fontEncoding){
-		if(m_fontEncoding->IsChecked()) {
+	} else if (btnClicked == m_fontEncoding) {
+		if (m_fontEncoding->IsChecked()) {
 			flags |= wxFRD_USEFONTENCODING;
 		} else {
 			flags &= ~(wxFRD_USEFONTENCODING);
 		}
-    } else if(btnClicked == m_printScope){
-		if(m_printScope->IsChecked()) {
+	} else if (btnClicked == m_printScope) {
+		if (m_printScope->IsChecked()) {
 			flags |= wxFRD_DISPLAYSCOPE;
 		} else {
 			flags &= ~(wxFRD_DISPLAYSCOPE);
 		}
-	} else if(btnClicked == m_checkBoxSaveFilesBeforeSearching) {
-		if(m_checkBoxSaveFilesBeforeSearching->IsChecked()) {
+	} else if (btnClicked == m_checkBoxSaveFilesBeforeSearching) {
+		if (m_checkBoxSaveFilesBeforeSearching->IsChecked()) {
 			flags |= wxFRD_SAVE_BEFORE_SEARCH;
 		} else {
 			flags &= ~(wxFRD_SAVE_BEFORE_SEARCH);
@@ -271,11 +270,10 @@ void FindInFilesDialog::OnClose(wxCloseEvent &e)
 
 void FindInFilesDialog::OnCharEvent(wxKeyEvent &event)
 {
-	if(event.GetKeyCode() == WXK_ESCAPE){
+	if (event.GetKeyCode() == WXK_ESCAPE) {
 		Hide();
 		return;
-	}
-	else if(event.GetKeyCode() == WXK_RETURN || event.GetKeyCode() == WXK_NUMPAD_ENTER){
+	} else if (event.GetKeyCode() == WXK_RETURN || event.GetKeyCode() == WXK_NUMPAD_ENTER) {
 		m_data.SetFindString( m_findString->GetValue() );
 		DoSearch();
 		return;
@@ -306,26 +304,26 @@ void FindInFilesDialog::OnClearPaths( wxCommandEvent& event )
 
 bool FindInFilesDialog::Show()
 {
-    bool res = IsShown() || wxDialog::Show();
-    if (res) {
+	bool res = IsShown() || wxDialog::Show();
+	if (res) {
 
 		// update the combobox
 		m_findString->Clear();
 		m_findString->Append(m_data.GetFindStringArr());
 		m_findString->SetValue(m_data.GetFindString());
 
-        LEditor *editor = Frame::Get()->GetMainBook()->GetActiveEditor();
-        if (editor) {
-            //if we have an open editor, and a selected text, make this text the search string
-            wxString selText = editor->GetSelectedText();
-            if (!selText.IsEmpty()) {
-                m_findString->SetValue(selText);
-            }
-        }
+		LEditor *editor = Frame::Get()->GetMainBook()->GetActiveEditor();
+		if (editor) {
+			//if we have an open editor, and a selected text, make this text the search string
+			wxString selText = editor->GetSelectedText();
+			if (!selText.IsEmpty()) {
+				m_findString->SetValue(selText);
+			}
+		}
 
-        m_findString->SetSelection(-1, -1); // select all
-        m_findString->SetFocus();
-    }
+		m_findString->SetSelection(-1, -1); // select all
+		m_findString->SetFocus();
+	}
 	return res;
 }
 
@@ -333,15 +331,15 @@ void FindInFilesDialog::DoSaveSearchPaths()
 {
 	wxArrayString paths = m_dirPicker->GetValues();
 	int where = paths.Index(SEARCH_IN_PROJECT);
-	if(where != wxNOT_FOUND){
+	if (where != wxNOT_FOUND) {
 		paths.RemoveAt(where);
 	}
 	where = paths.Index(SEARCH_IN_WORKSPACE);
-	if(where != wxNOT_FOUND){
+	if (where != wxNOT_FOUND) {
 		paths.RemoveAt(where);
 	}
 	where = paths.Index(SEARCH_IN_CURR_FILE_PROJECT);
-	if(where != wxNOT_FOUND){
+	if (where != wxNOT_FOUND) {
 		paths.RemoveAt(where);
 	}
 
@@ -350,7 +348,22 @@ void FindInFilesDialog::DoSaveSearchPaths()
 
 void FindInFilesDialog::DoSaveOpenFiles()
 {
-	if(m_checkBoxSaveFilesBeforeSearching->IsChecked()){
+	if (m_checkBoxSaveFilesBeforeSearching->IsChecked()) {
 		Frame::Get()->GetMainBook()->SaveAll(false, false);
 	}
 }
+
+void FindInFilesDialog::OnClearPathsUI(wxUpdateUIEvent& event)
+{
+	event.Enable(m_listPaths->IsEmpty() == false);
+}
+
+void FindInFilesDialog::OnRemovePathUI(wxUpdateUIEvent& event)
+{
+	event.Enable(m_listPaths->GetSelection() != wxNOT_FOUND);
+}
+
+void FindInFilesDialog::OnFindWhatUI(wxUpdateUIEvent& event)
+{
+	event.Enable(m_findString->GetValue().IsEmpty() == false);
+}
