@@ -23,6 +23,8 @@
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 #include <wx/xrc/xmlres.h>
+#include <wx/wupdlock.h>
+#include "drawingutils.h"
 #include "custom_tabcontainer.h"
 #include "custom_tab.h"
 #include <wx/tokenzr.h>
@@ -34,6 +36,7 @@
 #include "editor_config.h"
 #include "globals.h"
 #include "findresultstab.h"
+#include "search_thread.h"
 
 
 BEGIN_EVENT_TABLE(FindResultsTab, OutputTabWindow)
@@ -90,6 +93,11 @@ FindResultsTab::FindResultsTab(wxWindow *parent, wxWindowID id, const wxString &
 	}
 
 	wxTheApp->Connect(XRCID("find_in_files"), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(FindResultsTab::OnFindInFiles), NULL, this);
+	m_tb->AddTool ( XRCID ( "stop_search" ), wxT ( "Stop current search" ), wxXmlResource::Get()->LoadBitmap ( wxT ( "stop_build16" ) ), wxT ( "Stop current search" ) );
+	Connect( XRCID ( "stop_search" ), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler ( FindResultsTab::OnStopSearch  ), NULL, this );
+	Connect( XRCID ( "stop_search" ), wxEVT_UPDATE_UI,             wxUpdateUIEventHandler( FindResultsTab::OnStopSearchUI), NULL, this );
+	m_tb->Realize();
+
 }
 
 FindResultsTab::~FindResultsTab()
@@ -133,19 +141,19 @@ void FindResultsTab::SetStyles(wxScintilla *sci)
 {
 	InitStyle(sci, wxSCI_LEX_FIF, true);
 
-	sci->StyleSetForeground(wxSCI_LEX_FIF_DEFAULT, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT));
+	sci->StyleSetForeground(wxSCI_LEX_FIF_DEFAULT, wxT("BLACK"));
 	sci->StyleSetBackground(wxSCI_LEX_FIF_DEFAULT, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
 
-	sci->StyleSetForeground(wxSCI_LEX_FIF_PROJECT, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT));
+	sci->StyleSetForeground(wxSCI_LEX_FIF_PROJECT, wxT("BLACK"));
 	sci->StyleSetBackground(wxSCI_LEX_FIF_PROJECT, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
 
 	sci->StyleSetForeground(wxSCI_LEX_FIF_FILE, wxT("BLUE"));
 	sci->StyleSetBackground(wxSCI_LEX_FIF_FILE, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
 
-	sci->StyleSetForeground(wxSCI_LEX_FIF_FILE_SHORT, wxSystemSettings::GetColour(wxSYS_COLOUR_3DDKSHADOW));
+	sci->StyleSetForeground(wxSCI_LEX_FIF_FILE_SHORT, wxSystemSettings::GetColour(wxSYS_COLOUR_GRAYTEXT));
 	sci->StyleSetBackground(wxSCI_LEX_FIF_FILE_SHORT, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
 
-	sci->StyleSetForeground(wxSCI_LEX_FIF_MATCH, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOWTEXT));
+	sci->StyleSetForeground(wxSCI_LEX_FIF_MATCH, wxT("BLACK"));
 	sci->StyleSetBackground(wxSCI_LEX_FIF_MATCH, wxSystemSettings::GetColour(wxSYS_COLOUR_WINDOW));
 
 	sci->StyleSetForeground(wxSCI_LEX_FIF_SCOPE, wxT("PURPLE"));
@@ -164,6 +172,9 @@ void FindResultsTab::SetStyles(wxScintilla *sci)
 
 	sci->StyleSetHotSpot(wxSCI_LEX_FIF_MATCH, true);
 	sci->StyleSetHotSpot(wxSCI_LEX_FIF_FILE,  true);
+
+	sci->MarkerDefine       (7, wxSCI_MARK_BACKGROUND);
+	sci->MarkerSetBackground(7, DrawingUtils::LightColour(wxT("BLUE"), 8));
 }
 
 size_t FindResultsTab::GetPageCount() const
@@ -237,11 +248,27 @@ void FindResultsTab::OnSearchStart(wxCommandEvent& e)
 	wxString label = data ? data->GetFindString() : wxT("");
 
 	if (e.GetInt() != 0 || m_sci == NULL) {
-		if(m_book){
+		if (m_book) {
+			wxWindowUpdateLocker locker(this);
 			wxScintilla *sci = new wxScintilla(m_book);
 			SetStyles(sci);
 
-			// add new page
+			// Make sure we can add more tabs, if not delete the last used tab and then add
+			// a new tab
+
+			long MaxBuffers(15);
+			EditorConfigST::Get()->GetLongValue(wxT("MaxOpenedTabs"), MaxBuffers);
+
+			if( (long)m_book->GetPageCount() >= MaxBuffers ) {
+				// We have reached the limit of the number of open buffers
+				// Close the last used buffer
+				const wxArrayPtrVoid &arr = m_book->GetHistory();
+				if ( arr.GetCount() ) {
+					CustomTab *tab = static_cast<CustomTab*>(arr.Item(arr.GetCount()-1));
+					m_book->DeletePage( m_book->GetTabContainer()->TabToIndex(tab) );
+				}
+			}
+
 			m_book->AddPage(sci, label, label, wxNullBitmap, true);
 			size_t where = m_book->GetPageCount() - 1;
 
@@ -254,7 +281,7 @@ void FindResultsTab::OnSearchStart(wxCommandEvent& e)
 			m_matchInfo.push_back(MatchInfo());
 			m_sci = sci;
 		}
-	} else if(m_book){
+	} else if (m_book) {
 		// using current tab, update the tab title and the search data
 		size_t where = m_book->GetPageIndex(m_sci);
 		if (where != Notebook::npos) {
@@ -396,13 +423,8 @@ void FindResultsTab::OnMouseDClick(wxScintillaEvent &e)
 		size_t n = m_book ? m_book->GetSelection() : 0;
 		const MatchInfo& matchInfo = GetMatchInfo(n);
 		MatchInfo::const_iterator m = matchInfo.find(line);
-		if (m != matchInfo.end() && !m->second.GetFileName().IsEmpty()) {
-			LEditor *editor = Frame::Get()->GetMainBook()->OpenFile(m->second.GetFileName(), wxEmptyString, m->second.GetLineNumber()-1);
-			if (editor && m->second.GetColumn() >= 0 && m->second.GetLen() >= 0) {
-				int offset = editor->PositionFromLine(m->second.GetLineNumber()-1) + m->second.GetColumn();
-				editor->EnsureVisible(m->second.GetLineNumber()-1);
-				editor->SetSelection(offset, offset + m->second.GetLen());
-			}
+		if (m != matchInfo.end()) {
+			DoOpenSearchResult( m->second, m_sci, m->first );
 		}
 	}
 
@@ -438,7 +460,7 @@ SearchData FindResultsTab::GetSearchData(wxScintilla* sci)
 void FindResultsTab::OnCloseAllTabs(wxCommandEvent& e)
 {
 	wxUnusedVar( e );
-	if(m_book) {
+	if (m_book) {
 		m_book->DeleteAllPages(true);
 	}
 }
@@ -446,15 +468,15 @@ void FindResultsTab::OnCloseAllTabs(wxCommandEvent& e)
 void FindResultsTab::OnCloseOtherTab(wxCommandEvent& e)
 {
 	wxUnusedVar( e );
-	if(m_book) {
+	if (m_book) {
 		size_t idx = m_book->GetSelection();
-		if(idx != Notebook::npos) {
-			for(size_t i=0; i<idx; i++){
+		if (idx != Notebook::npos) {
+			for (size_t i=0; i<idx; i++) {
 				m_book->DeletePage((size_t)0);
 			}
 
 			size_t number = m_book->GetPageCount();
-			for(size_t i = number - 1; i>0; i--){
+			for (size_t i = number - 1; i>0; i--) {
 				m_book->DeletePage(i);
 			}
 		}
@@ -464,9 +486,9 @@ void FindResultsTab::OnCloseOtherTab(wxCommandEvent& e)
 void FindResultsTab::OnCloseTab(wxCommandEvent& e)
 {
 	wxUnusedVar( e );
-	if(m_book) {
+	if (m_book) {
 		size_t idx = m_book->GetSelection();
-		if(idx != Notebook::npos) {
+		if (idx != Notebook::npos) {
 			m_book->DeletePage(idx);
 		}
 	}
@@ -476,3 +498,107 @@ void FindResultsTab::OnTabMenuUI(wxUpdateUIEvent& e)
 {
 	e.Enable( !m_searchInProgress );
 }
+
+void FindResultsTab::NextMatch()
+{
+	// m_sci holds to the selected tab's scintilla editor
+	if ( m_sci ) {
+		const MatchInfo& matchInfo = GetMatchInfo( m_book ? m_book->GetSelection() : 0 );
+
+		// locate the last match
+		int firstLine = m_sci->MarkerNext(0, 255);
+		if ( firstLine == wxNOT_FOUND ) {
+			firstLine = 0;
+		}
+
+		// We found the last marker
+		for (int i=firstLine+1; i<m_sci->GetLineCount(); i++) {
+
+			// Find the next match
+			MatchInfo::const_iterator iter = matchInfo.find(i);
+			if ( iter != matchInfo.end() ) {
+				SearchResult sr = iter->second;
+
+				// open the new searchresult in the editor
+				DoOpenSearchResult ( sr, m_sci, i );
+				return;
+			}
+		}
+		// if we are here, it means we are the end of the search results list, add a status message
+		wxCommandEvent e(wxEVT_UPDATE_STATUS_BAR);
+		e.SetEventObject(this);
+		e.SetString(wxString::Format(wxT("Reached the end of 'find in files' search results list" )));
+		e.SetInt(0);
+		Frame::Get()->AddPendingEvent(e);
+	}
+}
+
+void FindResultsTab::PrevMatch()
+{
+	// m_sci holds to the selected tab's scintilla editor
+	if ( m_sci ) {
+		const MatchInfo& matchInfo = GetMatchInfo( m_book ? m_book->GetSelection() : 0 );
+
+		// locate the last match
+		int firstLine = m_sci->MarkerPrevious(m_sci->GetLineCount()-1, 255);
+		if ( firstLine == wxNOT_FOUND ) {
+			firstLine = m_sci->GetLineCount();
+		}
+
+		// We found the last marker
+		for (int i=firstLine-1; i>=0; i--) {
+
+			// Find the next match
+			MatchInfo::const_iterator iter = matchInfo.find(i);
+			if ( iter != matchInfo.end() ) {
+				SearchResult sr = iter->second;
+
+				// open the new searchresult in the editor
+				DoOpenSearchResult ( sr, m_sci, i );
+				return;
+			}
+		}
+		// if we are here, it means we are the top of the search results list, add a status message
+		wxCommandEvent e(wxEVT_UPDATE_STATUS_BAR);
+		e.SetEventObject(this);
+		e.SetString(wxString::Format(wxT("Reached the begining of 'find in files' search results list" )));
+		e.SetInt(0);
+		Frame::Get()->AddPendingEvent(e);
+	}
+}
+
+void FindResultsTab::DoOpenSearchResult(const SearchResult &result, wxScintilla *sci, int markerLine)
+{
+	if (!result.GetFileName().IsEmpty()) {
+		LEditor *editor = Frame::Get()->GetMainBook()->OpenFile(result.GetFileName(), wxEmptyString, result.GetLineNumber()-1);
+		if (editor && result.GetColumn() >= 0 && result.GetLen() >= 0) {
+			int offset = editor->PositionFromLine(result.GetLineNumber()-1) + result.GetColumn();
+			editor->EnsureVisible(result.GetLineNumber()-1);
+			editor->SetSelection(offset, offset + result.GetLen());
+
+			if ( sci ) {
+				// remove the previous marker and add the new one
+				sci->MarkerDeleteAll( 7 );
+				sci->MarkerAdd(markerLine, 7 );
+
+				// make the marked line visible
+				int pos = sci->PositionFromLine(markerLine);
+				sci->SetCurrentPos     (pos);
+				sci->SetSelectionStart (pos);
+				sci->SetSelectionEnd   (pos);
+				sci->EnsureCaretVisible(   );
+			}
+		}
+	}
+}
+
+void FindResultsTab::OnStopSearch(wxCommandEvent& e)
+{
+	// stop the search thread
+	SearchThreadST::Get()->StopSearch();
+}
+
+void FindResultsTab::OnStopSearchUI(wxUpdateUIEvent& e)
+{
+	e.Enable(m_searchInProgress);
+}
