@@ -54,11 +54,13 @@ class WorkspaceTab;
 class FileExplorer;
 class OutputTabWindow;
 class DockablePaneMenuManager;
+class OutputViewControlBar;
 
 //--------------------------------
 // Helper class
 //--------------------------------
 extern const wxEventType wxEVT_UPDATE_STATUS_BAR;
+extern const wxEventType wxEVT_LOAD_PERSPECTIVE ;
 
 struct StartPageData {
 	wxString name;
@@ -68,26 +70,27 @@ struct StartPageData {
 
 class Frame : public wxFrame
 {
-	MainBook *m_mainBook;
-	static Frame* m_theFrame;
-	wxAuiManager m_mgr;
-	OutputPane *m_outputPane;
-	WorkspacePane *m_workspacePane;
-	wxArrayString m_files;
-	wxTimer *m_timer;
-	std::map<int, wxString> m_viewAsMap;
-	TagsOptionsData m_tagsOptionsData;
-	DebuggerPane *m_debuggerPane;
-	wxToolBar *m_debuggerTb;
-	bool m_buildAndRun;
-
-	GeneralInfo m_frameGeneralInfo;
-	std::map<int, wxString> m_toolbars;
-	std::map<int, wxString> m_panes;
+	MainBook *                            m_mainBook;
+	static Frame*                         m_theFrame;
+	wxAuiManager                          m_mgr;
+	OutputPane *                          m_outputPane;
+	WorkspacePane *                       m_workspacePane;
+	wxArrayString                         m_files;
+	wxTimer *                             m_timer;
+	std::map<int, wxString>               m_viewAsMap;
+	TagsOptionsData                       m_tagsOptionsData;
+	DebuggerPane *                        m_debuggerPane;
+	wxToolBar *                           m_debuggerTb;
+	bool                                  m_buildAndRun;
+	GeneralInfo                           m_frameGeneralInfo;
+	std::map<int, wxString>               m_toolbars;
+	std::map<int, wxString>               m_panes;
     std::vector<std::map<int, wxString> > m_status;
-	wxMenu *m_cppMenu;
-	bool m_highlightWord;
-	DockablePaneMenuManager *m_DPmenuMgr;
+	wxMenu *                              m_cppMenu;
+	bool                                  m_highlightWord;
+	DockablePaneMenuManager *             m_DPmenuMgr;
+	OutputViewControlBar *                m_controlBar;
+	wxPanel*                              m_mainPanel;
 
 public:
 	static Frame* Get();
@@ -97,6 +100,14 @@ public:
 		return m_DPmenuMgr;
 	}
 
+	wxPanel *GetMainPanel() {return m_mainPanel;}
+	
+	/**
+	 * @brief update the environment status message:
+	 * which displays to the user the current environment set used + the active builder
+	 */
+	void SetEnvStatusMessage();
+	
 	virtual ~Frame(void);
 	/**
 	 * @brief set frame option flag
@@ -105,6 +116,19 @@ public:
 	 */
 	void SetFrameFlag(bool set, int flag);
 
+	/**
+	 * @brief update the tags options data on the disc as well as
+	 * the parser thread
+	 * @param tod
+	 */
+	void UpdateTagsOptions(const TagsOptionsData &tod);
+
+	/**
+	 * @brief return the current tags options data
+	 * @return
+	 */
+	TagsOptionsData& GetTagsOptions() {return m_tagsOptionsData;}
+
 	/**
 	 * @brief return true if the word under the caret should be highlighted
 	 * @return
@@ -174,6 +198,10 @@ public:
 		return m_mgr;
 	}
 
+	wxAuiManager* GetDockingManagerPtr() {
+		return &m_mgr;
+	}
+
 	/**
 	 * Return the debugger toolbar
 	 */
@@ -187,9 +215,11 @@ public:
 	void LoadSession(const wxString &sessionName);
 
 	/**
-	 * load all available plugins
+	 * Compelete the main frame initialization
+	 * this method is called immediatly after the
+	 * main frame construction is over.
 	 */
-	void LoadPlugins();
+	void CompleteInitialization();
 
 	void RegisterToolbar(int menuItemId, const wxString &name);
 	void RegisterDockWindow(int menuItemId, const wxString &name);
@@ -222,11 +252,17 @@ public:
      */
     void SetStatusMessage(const wxString &msg, int col, int id = wxID_ANY);
 
+	/**
+	 * @brief save the current IDE layout and session
+	 */
+	void SaveLayoutAndSession();
+
 private:
 	// make our frame's constructor private
 	Frame(wxWindow *pParent, wxWindowID id, const wxString& title, const wxPoint& pos, const wxSize& size, long style = wxMINIMIZE_BOX | wxMAXIMIZE_BOX | wxCLOSE_BOX | wxCAPTION | wxSYSTEM_MENU | wxRESIZE_BORDER | wxCLIP_CHILDREN);
 	wxString CreateWorkspaceTable();
 	wxString CreateFilesTable();
+	void     StartTimer();
 
 private:
 	/**
@@ -254,11 +290,6 @@ private:
 	void CreateRecentlyOpenedFilesMenu();
 	void CreateRecentlyOpenedWorkspacesMenu();
 	void CreateWelcomePage();
-	void CreateMenuBar();
-	void UpgradeExternalDbExt();
-	void AutoLoadExternalDb();
-	void DoBuildExternalDatabase(const wxString &dir = wxEmptyString);
-
 	void ReloadExternallyModifiedProjectFiles();
 
 protected:
@@ -274,18 +305,18 @@ protected:
 	void OnSave(wxCommandEvent& event);
 	void OnSaveAs(wxCommandEvent& event);
 	void OnFileReload(wxCommandEvent& event);
+	void OnFileLoadTabGroup(wxCommandEvent& event);
 	void OnCompleteWord(wxCommandEvent& event);
 	void OnFunctionCalltip(wxCommandEvent& event);
 	void OnDeleteProject(wxCommandEvent& event);
-	void OnBuildExternalDatabase(wxCommandEvent& event);
-	void OnUseExternalDatabase(wxCommandEvent& event);
-	void OnCloseExternalDatabase(wxCommandEvent& event);
 	void OnAbout(wxCommandEvent& event);
+	void OnCheckForUpdate(wxCommandEvent &e);
 	void OnFileNew(wxCommandEvent &event);
 	void OnFileOpen(wxCommandEvent &event);
 	void OnFileClose(wxCommandEvent &event);
 	void OnFileCloseUI(wxUpdateUIEvent &event);
 	void OnFileSaveAll(wxCommandEvent &event);
+	void OnFileSaveTabGroup(wxCommandEvent &event);
 	void OnFileFindAndReplace(wxCommandEvent &event);
 	void OnFileExistUpdateUI(wxUpdateUIEvent &event);
 	void OnCompleteWordUpdateUI(wxUpdateUIEvent &event);
@@ -337,6 +368,10 @@ protected:
 	void OnOpenShellFromFilePath(wxCommandEvent &e);
 	void OnQuickDebug(wxCommandEvent &e);
 	void OnQuickDebugUI(wxUpdateUIEvent &e);
+	void OnNextFiFMatch      (wxCommandEvent &e);
+	void OnPreviousFiFMatch  (wxCommandEvent &e);
+	void OnNextFiFMatchUI    (wxUpdateUIEvent &e);
+	void OnPreviousFiFMatchUI(wxUpdateUIEvent &e);
 
 	// this event is sent from the notebook container to the frame
 	void OnFileClosing(NotebookEvent &event);
@@ -344,10 +379,12 @@ protected:
 	void OnPageClosed(NotebookEvent &event);
 
 	//handle symbol tree events
-	void OnAddSymbols(SymbolTreeEvent &event);
-	void OnDeleteSymbols(SymbolTreeEvent &event);
-	void OnUpdateSymbols(SymbolTreeEvent &event);
-	void OnParsingThreadDone(wxCommandEvent &e);
+	void OnAddSymbols                 (SymbolTreeEvent &event);
+	void OnDeleteSymbols              (SymbolTreeEvent &event);
+	void OnUpdateSymbols              (SymbolTreeEvent &event);
+	void OnParsingThreadDone          (wxCommandEvent  &e);
+	void OnParsingThreadMessage       (wxCommandEvent  &e);
+	void OnDatabaseUpgrade            (wxCommandEvent  &e);
 
 	void OnRecentFile(wxCommandEvent &event);
 	void OnRecentWorkspace(wxCommandEvent &event);
@@ -374,6 +411,7 @@ protected:
 	void OnLoadWelcomePageUI(wxUpdateUIEvent &event);
 	void OnAppActivated(wxActivateEvent &event);
 	void OnReloadExternallModified(wxCommandEvent &e);
+	void OnReloadExternallModifiedNoPrompt(wxCommandEvent &e);
 	void OnCompileFile(wxCommandEvent &e);
 	void OnCompileFileUI(wxUpdateUIEvent &e);
 	void OnCloseAllButThis(wxCommandEvent &e);
@@ -416,8 +454,11 @@ protected:
 	void OnShowWhitespace(wxCommandEvent &e);
 	void OnShowFullScreen(wxCommandEvent &e);
 	void OnSetStatusMessage(wxCommandEvent &e);
-
-// Any class wishing to process wxWindows events must use this macro
+	void OnShowQuickFinder  (wxCommandEvent &e);
+	void OnShowQuickFinderUI(wxUpdateUIEvent &e);
+	void OnShowActiveProjectSettings(wxCommandEvent &e);
+	void OnShowActiveProjectSettingsUI(wxUpdateUIEvent &e);
+	void OnLoadPerspective(wxCommandEvent &e);
 	DECLARE_EVENT_TABLE()
 };
 
