@@ -24,6 +24,8 @@
 //////////////////////////////////////////////////////////////////////////////
 
 #include "options_dlg2.h"
+#include "plugin.h"
+#include "editorsettingsdockingwidows.h"
 #include "editorsettingsterminal.h"
 #include "editorsettingscaret.h"
 
@@ -33,6 +35,7 @@
 #include "editoroptionsgeneralindentationpanel.h"
 #include "editoroptionsgeneralrightmarginpanel.h"
 #include "editoroptionsgeneralsavepanel.h"
+#include "globals.h"
 
 #include "editorsettingscomments.h"
 #include "editorsettingscommentsdoxygenpanel.h"
@@ -44,8 +47,9 @@
 #include "../Plugin/windowattrmanager.h"
 
 OptionsDlg2::OptionsDlg2( wxWindow* parent )
-: OptionsBaseDlg2( parent )
-, m_contentObjects()
+		: OptionsBaseDlg2( parent )
+		, m_contentObjects()
+		, restartRquired(false)
 {
 	Initialize();
 	WindowAttrManager::Load(this, wxT("OptionsDlgAttr"), NULL);
@@ -54,7 +58,8 @@ OptionsDlg2::OptionsDlg2( wxWindow* parent )
 
 OptionsDlg2::~OptionsDlg2()
 {
-	WindowAttrManager::Save(this, wxT("OptionsDlgAttr"), NULL);
+	if ( !this->restartRquired )
+		WindowAttrManager::Save(this, wxT("OptionsDlgAttr"), NULL);
 }
 
 void OptionsDlg2::OnButtonOK( wxCommandEvent&)
@@ -82,9 +87,14 @@ void OptionsDlg2::DoSave()
 	// file
 	EditorConfigST::Get()->Begin();
 	typedef std::list<TreeBookNodeBase*>::iterator ContentIter;
-	for(ContentIter it = m_contentObjects.begin(), end = m_contentObjects.end(); it != end; ++it){
-		if(*it){
-			(*it)->Save(options);
+	for (ContentIter it = m_contentObjects.begin(), end = m_contentObjects.end(); it != end; ++it) {
+		if (*it) {
+			TreeBookNodeBase* child = *it;
+			child->Save( options );
+
+			if ( !this->restartRquired ) {
+				this->restartRquired = child->IsRestartRequired();
+			}
 		}
 	}
 
@@ -94,6 +104,13 @@ void OptionsDlg2::DoSave()
 	EditorConfigST::Get()->Save();
 
 	Frame::Get()->GetMainBook()->ApplySettingsChanges();
+
+	// Notify plugins about settings changed
+	PostCmdEvent( wxEVT_EDITOR_SETTINGS_CHANGED );
+
+	if ( this->restartRquired ) {
+		WindowAttrManager::Save(this, wxT("OptionsDlgAttr"), NULL);
+	}
 }
 
 void OptionsDlg2::Initialize()
@@ -108,13 +125,18 @@ void OptionsDlg2::Initialize()
 	m_treeBook->AddPage(0, wxT("C++"));
 	AddSubPage(new EditorSettingsComments(m_treeBook),             wxT("Comments"));
 	AddSubPage(new EditorSettingsCommentsDoxygenPanel(m_treeBook), wxT("Doxygen"));
+
+#ifdef __WXMSW__
 	AddSubPage(new EditorOptionsGeneralCodeNavPanel(m_treeBook),   wxT("Quick Code Navigation"));
+#endif
 
 	AddPage(new EditorSettingsFolding(m_treeBook),        wxT("Folding"));
-	AddPage(new EditorSettingsBookmarksPanel(m_treeBook), wxT("Bookmarks"));
+	AddPage(new EditorSettingsBookmarksPanel(m_treeBook), wxT("Bookmarks & Breakpoints"));
 	AddPage(new EditorSettingsDialogs(m_treeBook),        wxT("Dialogs"));
+	AddPage(new EditorSettingsDockingWindows(m_treeBook), wxT("Docking Windows"));
+
 #ifndef __WXMSW__
-	// the Terminal page should be added under Windows
+	// the Terminal page should NOT be added under Windows
 	AddPage(new EditorSettingsTerminal(m_treeBook),       wxT("Terminal"));
 #endif
 	AddPage(new EditorSettingsMiscPanel(m_treeBook),      wxT("Misc"));
