@@ -40,7 +40,8 @@ enum {
 	wxVB_BORDER                 = 0x00000100,
 	wxVB_NODND                  = 0x00000200,
 	wxVB_NO_DROPBUTTON          = 0x00000400,
-	wxVB_FIXED_WIDTH            = 0x00000800
+	wxVB_FIXED_WIDTH            = 0x00000800,
+	wxVB_NO_TABS                = 0x00001000
 };
 
 class wxTabContainer;
@@ -85,7 +86,7 @@ public:
 	 * \brief set page at given index to be the selected page. this function does not trigger an event
 	 * \param page
 	 */
-	void SetSelection(size_t page);
+	void SetSelection(size_t page, bool notify = true);
 
 	/**
 	 * \brief add page to the book
@@ -95,12 +96,14 @@ public:
 	 * \param selected set the page as the selected page
 	 */
 	void AddPage(wxWindow *win, const wxString &text, const wxString &tooltip, const wxBitmap &bmp = wxNullBitmap, bool selected = false);
+	void InsertPage(size_t index, wxWindow *win, const wxString &text, const wxString &tooltip, const wxBitmap &bmp = wxNullBitmap, bool selected = false);
 
 	/**
 	 * \brief add page to the book
 	 * \param tab to be added
 	 */
 	void AddPage(CustomTab *tab);
+	void InsertPage(CustomTab* tab, size_t index);
 
 	/**
 	 * \brief return page at give position
