@@ -22,10 +22,15 @@
 //
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
+#include <wx/app.h>
+#include <wx/panel.h>
 #include "wx/settings.h"
 #include "drawingutils.h"
 #include "wx/dc.h"
 
+#ifdef __WXGTK__
+#include <gtk/gtk.h>
+#endif
 //////////////////////////////////////////////////
 // Colour methods to convert HSL <-> RGB
 //////////////////////////////////////////////////
@@ -316,3 +321,47 @@ wxColour DrawingUtils::GetGradient()
 //	return LightColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DDKSHADOW), GetDdkShadowLightFactor());
 //#endif
 }
+
+
+wxColor DrawingUtils::DarkColour(const wxColour& color, float percent)
+{
+	if(percent == 0){
+		return color;
+	}
+
+	float h, s, l, r, g, b;
+	RGB_2_HSL(color.Red(), color.Green(), color.Blue(), &h, &s, &l);
+
+	// reduce the Lum value
+	l -= (float)((percent * 5.0)/100.0);
+	if (l < 0) l = 0.0;
+
+	HSL_2_RGB(h, s, l, &r, &g, &b);
+	return wxColour((unsigned char)r, (unsigned char)g, (unsigned char)b);
+}
+
+wxColor DrawingUtils::GetPanelBgColour()
+{
+#ifdef __WXGTK__
+	static bool     intitialized(false);
+	static wxColour bgColour(wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE));
+	
+	if( !intitialized ) {
+		// try to get the background colour from a menu
+		GtkWidget *menu = gtk_window_new(GTK_WINDOW_TOPLEVEL);
+		GtkStyle   *def = gtk_rc_get_style( menu );
+		if(!def)
+			def = gtk_widget_get_default_style();
+		
+		if(def) {
+			GdkColor col = def->bg[GTK_STATE_NORMAL];
+			bgColour = wxColour(col);
+		}
+		gtk_widget_destroy( menu );
+		intitialized = true;
+	}
+	return bgColour;
+#else
+	return wxSystemSettings::GetColour(wxSYS_COLOUR_3DFACE);
+#endif
+}
