@@ -72,6 +72,7 @@ void SimpleStringValue::DeSerialize(Archive &arch)
 //-------------------------------------------------------------------------------------------
 
 wxString EditorConfig::m_svnRevision;
+wxString EditorConfig::m_version;
 
 EditorConfig::EditorConfig()
 		: m_transcation(false)
@@ -84,39 +85,64 @@ EditorConfig::~EditorConfig()
 	delete m_doc;
 }
 
+bool EditorConfig::DoLoadDefaultSettings()
+{
+	//try to load the default settings
+	m_fileName = wxFileName(wxT("config/codelite.xml.default"));
+	m_fileName.MakeAbsolute();
+
+	if ( !m_fileName.FileExists() ) {
+		//create a new empty file with this name so the load function will not
+		//fail
+		wxFFile file(m_fileName.GetFullPath(), wxT("a"));
+		wxString content;
+		content << wxT("<CodeLite Revision=\"")
+		<< m_svnRevision
+		<< wxT("\"")
+		<< wxT(" Version=\"")
+		<< m_version
+		<< wxT("\">")
+		<< wxT("</CodeLite>");
+
+		if (file.IsOpened()) {
+			file.Write(content);
+			file.Close();
+		}
+	}
+	return m_doc->Load(m_fileName.GetFullPath());
+}
+
 bool EditorConfig::Load()
 {
 	//first try to load the user's settings
 	m_fileName = wxFileName(wxT("config/codelite.xml"));
 	m_fileName.MakeAbsolute();
+	bool userSettingsLoaded(false);
+	bool loadSuccess       (false);
 
 	if (!m_fileName.FileExists()) {
-		//try to load the default settings
-		m_fileName = wxFileName(wxT("config/codelite.xml.default"));
-		m_fileName.MakeAbsolute();
+		loadSuccess = DoLoadDefaultSettings();
 
-		if ( !m_fileName.FileExists() ) {
-			//create a new empty file with this name so the load function will not
-			//fail
-			wxFFile file(m_fileName.GetFullPath(), wxT("a"));
-			wxString content;
-			content << wxT("<LiteEditor Revision=\"")
-			<< m_svnRevision
-			<< wxT("\">")
-			<< wxT("</LiteEditor>");
-
-			if (file.IsOpened()) {
-				file.Write(content);
-				file.Close();
-			}
-		}
+	} else {
+		userSettingsLoaded = true;
+		loadSuccess = m_doc->Load(m_fileName.GetFullPath());
 	}
 
-	// load the main configuration file
-	if (!m_doc->Load(m_fileName.GetFullPath())) {
+	if ( !loadSuccess ) {
 		return false;
 	}
 
+	// Check the codelite-version for this file
+	wxString version;
+	bool found = m_doc->GetRoot()->GetPropVal(wxT("Version"), &version);
+	if ( userSettingsLoaded ) {
+		if(!found || (found && version != this->m_version)) {
+			if(DoLoadDefaultSettings() == false) {
+				return false;
+			}
+		}
+	}
+
 	// load CodeLite lexers
 	LoadLexers(false);
 
@@ -153,30 +179,6 @@ LexerConfPtr EditorConfig::GetLexer(const wxString &lexerName)
 	return m_lexers.find(lexerName)->second;
 }
 
-wxString EditorConfig::LoadPerspective(const wxString &Name) const
-{
-	wxXmlNode *layoutNode = XmlUtils::FindFirstByTagName(m_doc->GetRoot(), wxT("Layout"));
-	if ( !layoutNode ) {
-		//add an Layout node
-		wxXmlNode *newChild = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, wxT("Layout"));
-		m_doc->GetRoot()->AddChild(newChild);
-		DoSave();
-		layoutNode = newChild;
-	}
-
-	wxXmlNode *child = layoutNode->GetChildren();
-	while ( child ) {
-		if ( child->GetName() == wxT("Perspective") ) {
-			if (child->GetPropVal(wxT("Name"), wxEmptyString) == Name) {
-				return child->GetPropVal(wxT("Value"), wxEmptyString);
-			}
-		}
-		child = child->GetNext();
-	}
-
-	return wxEmptyString;
-}
-
 //long EditorConfig::LoadNotebookStyle(const wxString &nbName)
 //{
 //	long style = wxNOT_FOUND;
@@ -305,51 +307,13 @@ wxString EditorConfig::GetTagsDatabase() const
 	}
 }
 
-void EditorConfig::GetRecentlyOpenedFies(wxArrayString &files)
+void EditorConfig::GetRecentItems(wxArrayString &files, const wxString nodeName)
 {
-	//find the root node of the recent files
-	wxXmlNode *node = XmlUtils::FindFirstByTagName(m_doc->GetRoot(), wxT("RecentFiles"));
-	if (node) {
-		wxXmlNode *child = node->GetChildren();
-		while (child) {
-			if (child->GetName() == wxT("File")) {
-				wxString fileName = XmlUtils::ReadString(child, wxT("Name"));
-				// wxXmlDocument Saves/Loads items in reverse order, so prepend, not add.
-				if( wxFileExists( fileName) ) files.Insert(fileName, 0);
-			}
-			child = child->GetNext();
-		}
-	}
+	if (nodeName.IsEmpty()) {
+		return;
 }
-
-void EditorConfig::SetRecentlyOpenedFies(const wxArrayString &files)
-{
-	wxString nodeName = wxT("RecentFiles");
+	//find the root node
 	wxXmlNode *node = XmlUtils::FindFirstByTagName(m_doc->GetRoot(), nodeName);
-	if (node) {
-		wxXmlNode *root = m_doc->GetRoot();
-		root->RemoveChild(node);
-		delete node;
-	}
-	//create new entry in the configuration file
-	node = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, nodeName);
-	m_doc->GetRoot()->AddChild(node);
-	for (size_t i=0; i<files.GetCount(); i++) {
-		wxXmlNode *child = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, wxT("File"));
-		child->AddProperty(wxT("Name"), files.Item(i));
-		node->AddChild(child);
-	}
-
-	//save the data to disk
-	DoSave();
-	SendCmdEvent(wxEVT_EDITOR_CONFIG_CHANGED, (void*) &nodeName);
-}
-
-
-void EditorConfig::GetRecentlyOpenedWorkspaces(wxArrayString &files)
-{
-	//find the root node of the recent files
-	wxXmlNode *node = XmlUtils::FindFirstByTagName(m_doc->GetRoot(), wxT("RecentWorkspaces"));
 	if (node) {
 		wxXmlNode *child = node->GetChildren();
 		while (child) {
@@ -363,15 +327,19 @@ void EditorConfig::GetRecentlyOpenedWorkspaces(wxArrayString &files)
 	}
 }
 
-void EditorConfig::SetRecentlyOpenedWorkspaces(const wxArrayString &files)
+void EditorConfig::SetRecentItems(const wxArrayString &files, const wxString nodeName)
 {
-	wxString nodeName = wxT("RecentWorkspaces");
+	if (nodeName.IsEmpty()) {
+		return;
+	}
+	//find the root node
 	wxXmlNode *node = XmlUtils::FindFirstByTagName(m_doc->GetRoot(), nodeName);
 	if (node) {
 		wxXmlNode *root = m_doc->GetRoot();
 		root->RemoveChild(node);
 		delete node;
 	}
+	//create new entry in the configuration file
 	node = new wxXmlNode(NULL, wxXML_ELEMENT_NODE, nodeName);
 	m_doc->GetRoot()->AddChild(node);
 	for (size_t i=0; i<files.GetCount(); i++) {
@@ -512,7 +480,7 @@ void EditorConfig::LoadLexers(bool loadDefault)
 
 		//try to locate a file with the same name but with the user extension
 		wxFileName fn(files.Item(i));
-		wxString userLexer( fn.GetPath(wxPATH_GET_VOLUME|wxPATH_GET_SEPARATOR) + fn.GetName() +  wxT(".") + wxGetUserName() + wxT("_xml"));
+		wxString userLexer( fn.GetPath(wxPATH_GET_VOLUME|wxPATH_GET_SEPARATOR) + fn.GetName() +  wxT(".") + clGetUserName() + wxT("_xml"));
 		if ( wxFileName::FileExists( userLexer ) ) {
 			if (!loadDefault) {
 				fileToLoad = userLexer;
@@ -574,3 +542,4 @@ void SimpleRectValue::Serialize(Archive& arch)
 	arch.Write(wxT("TopLeft"), m_rect.GetTopLeft());
 	arch.Write(wxT("Size"), m_rect.GetSize());
 }
+
