@@ -33,6 +33,8 @@
 #include <map>
 //#include "ctags_manager.h"
 #include "configuration_mapping.h"
+#include "optionsconfig.h"
+#include "localworkspace.h"
 
 
 #ifdef WXMAKINGDLL_LE_SDK
@@ -217,6 +219,16 @@ public:
 	 */
 	void Save();
 
+	/**
+	 * Return a node pointing to any workspace-wide editor preferences
+	 */
+	wxXmlNode* GetWorkspaceEditorOptions() const;
+
+	/**
+	 * Add or update local workspace options
+	 */
+	void SetWorkspaceEditorOptions(LocalOptionsConfigPtr opts);
+
 	/**
 	 * Return the configuration mapping for the workspace. 'Configuration Mapping' is
 	 * the build matrix that should be used for this workspace, it contains all possible
