@@ -41,6 +41,8 @@ public:
 	void OnGetBuildCommand     (wxCommandEvent &event);
 	void OnGetIsPluginMakefile (wxCommandEvent &event);
 	void OnNewQmakeBasedProject(wxCommandEvent &event);
+	void OnOpenFile            (wxCommandEvent &event);
+	void OnExportMakefile      (wxCommandEvent &event);
 };
 
 #endif //QMakePlugin
