@@ -2,7 +2,7 @@
 
 fix_shared_object_depends() {
 	search_string=$1
-	
+
 	## Get list of files to work on
 	file_list=`ls ../lib/*.so`
 	file_list="${file_list} ${exe_name} "
@@ -11,7 +11,7 @@ fix_shared_object_depends() {
 	## executable, we can run the following command only once and
 	## use the results to manipulate the plugins as well
 	orig_path=`otool -L ${exe_name}  | grep ${search_string} | awk '{print $1;}'`
-	
+
 	## Loop over the files, and update the path of the wx library
 	for file in ${file_list}
 	do
@@ -52,7 +52,7 @@ fix_shared_object_depends libplugin
 fix_shared_object_depends libcodelite
 fix_shared_object_depends libwxscintilla
 fix_shared_object_depends libwxsqlite
-fix_shared_object_depends libwxpropgrid
+#fix_shared_object_depends libwxpropgrid
 
 #echo install_name_tool -change /usr/lib/libcurl.4.dylib @executable_path/libcurl.4.dylib ${exe_name}
 #install_name_tool -change /usr/lib/libcurl.4.dylib @executable_path/libcurl.4.dylib ${exe_name}
@@ -103,22 +103,23 @@ cp config/debuggers.xml.default ./CodeLite.app/Contents/SharedSupport/config
 cp ../lib/CodeFormatter.so ./CodeLite.app/Contents/SharedSupport/plugins/
 cp ../lib/Debugger.so ./CodeLite.app/Contents/SharedSupport/debuggers/
 cp ../lib/Gizmos.so ./CodeLite.app/Contents/SharedSupport/plugins/
-cp ../lib/Subversion.so ./CodeLite.app/Contents/SharedSupport/plugins/
+cp ../lib/Subversion2.so ./CodeLite.app/Contents/SharedSupport/plugins/
 cp ../lib/cscope.so ./CodeLite.app/Contents/SharedSupport/plugins/
 cp ../lib/Copyright.so ./CodeLite.app/Contents/SharedSupport/plugins/
 cp ../lib/UnitTestCPP.so ./CodeLite.app/Contents/SharedSupport/plugins/
 cp ../lib/ExternalTools.so ./CodeLite.app/Contents/SharedSupport/plugins/
-cp ../lib/SymbolView.so ./CodeLite.app/Contents/SharedSupport/plugins/
+#cp ../lib/SymbolView.so ./CodeLite.app/Contents/SharedSupport/plugins/
 cp ../lib/ContinuousBuild.so ./CodeLite.app/Contents/SharedSupport/plugins/
 cp ../lib/SnipWiz.so ./CodeLite.app/Contents/SharedSupport/plugins/
 cp ../lib/wxformbuilder.so ./CodeLite.app/Contents/SharedSupport/plugins/
 cp ../lib/abbreviation.so ./CodeLite.app/Contents/SharedSupport/plugins/
 cp ../lib/QmakePlugin.so ./CodeLite.app/Contents/SharedSupport/plugins/
+cp ../lib/CppCheck.so ./CodeLite.app/Contents/SharedSupport/plugins/
 cp ../lib/libwxscintillau.so ./CodeLite.app/Contents/MacOS/
 cp ../lib/libpluginu.so ./CodeLite.app/Contents/MacOS/
 cp ../lib/libcodeliteu.so ./CodeLite.app/Contents/MacOS/
 cp ../lib/libwxsqlite3u.so ./CodeLite.app/Contents/MacOS/
-cp ../lib/libwxpropgridu.so ./CodeLite.app/Contents/MacOS/
 
-cp ../sdk/codelite_indexer/codelite_indexer ./CodeLite.app/Contents/SharedSupport/
-cp plugins/resources/*.* ./CodeLite.app/Contents/SharedSupport/plugins/resources/
+cp ../sdk/codelite_indexer/codelite_indexer   ./CodeLite.app/Contents/SharedSupport/
+cp ../sdk/codelite_cppcheck/codelite_cppcheck ./CodeLite.app/Contents/SharedSupport/
+cp plugins/resources/*.*                      ./CodeLite.app/Contents/SharedSupport/plugins/resources/
