@@ -22,7 +22,7 @@
 //
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
-
+#include <wx/app.h>
 #include "unittestspage.h"
 #include "macros.h"
 #include "unittestdata.h"
@@ -40,11 +40,15 @@
 #include <wx/ffile.h>
 #include <wx/msgdlg.h>
 #include "unittestpp.h"
-#include <wx/app.h>
 #include <wx/xrc/xmlres.h>
 #include <wx/menuitem.h>
 #include <wx/menu.h>
 
+#ifdef __WXMSW__
+#include <wx/msw/registry.h>
+#include "evnvarlist.h"
+#endif
+
 static UnitTestPP* thePlugin = NULL;
 
 //Define the plugin entry point
@@ -78,6 +82,25 @@ UnitTestPP::UnitTestPP(IManager *manager)
 	m_longName = wxT("A Unit test plugin based on the UnitTest++ framework");
 	m_shortName = wxT("UnitTestPP");
 	m_topWindow = m_mgr->GetTheApp();
+
+#ifdef __WXMSW__
+	wxRegKey rk(wxT("HKEY_CURRENT_USER\\Software\\CodeLite"));
+	if(rk.Exists()) {
+		wxString strUnitTestPP;
+		if(rk.HasValue(wxT("unittestpp"))){
+			rk.QueryValue(wxT("unittestpp"), strUnitTestPP);
+		}
+
+		if(strUnitTestPP.IsEmpty() == false) {
+			// Add the UnitTestPP environment variable to codelite's
+			// environment variables
+			EvnVarList vars;
+			m_mgr->GetEnv()->ReadObject(wxT("Variables"), &vars);
+			vars.AddVariable(wxT("Default"), wxT("UNIT_TEST_PP_SRC_DIR"), strUnitTestPP);
+			m_mgr->GetEnv()->WriteObject(wxT("Variables"), &vars);
+		}
+	}
+#endif
 }
 
 UnitTestPP::~UnitTestPP()
@@ -335,19 +358,15 @@ void UnitTestPP::OnRunUnitTests(wxCommandEvent& e)
 	if (m_proc) {
 
 		//set the environment variables
-		m_mgr->GetEnv()->ApplyEnv(NULL);
+		EnvSetter env(m_mgr->GetEnv());
 
 		if (m_proc->Start() == 0) {
 
-			//set the environment variables
-			m_mgr->GetEnv()->UnApplyEnv();
-
 			//failed to start the process
 			delete m_proc;
 			m_proc = NULL;
 			return;
 		}
-		m_mgr->GetEnv()->UnApplyEnv();
 		m_proc->Connect(wxEVT_END_PROCESS, wxProcessEventHandler(UnitTestPP::OnProcessTerminated), NULL, this);
 	}
 }
@@ -459,10 +478,10 @@ IEditor *UnitTestPP::DoAddTestFile(const wxString& filename, const wxString &pro
 		// Search the target file, if it is already exist in the project, open the file
 		// and return
 		for (size_t i=0; i<files.size(); i++) {
-			if (files.at(i).GetFullPath() == fn.GetFullPath()) {
+			if (files.at(i) == fn) {
 				m_mgr->OpenFile(fn.GetFullPath());
 				editor = m_mgr->GetActiveEditor();
-				if (editor && editor->GetFileName().GetFullPath() == fn.GetFullPath()) {
+				if (editor && editor->GetFileName() == fn) {
 					return editor;
 				} else {
 					return NULL;
@@ -479,7 +498,7 @@ IEditor *UnitTestPP::DoAddTestFile(const wxString& filename, const wxString &pro
 		// open the file
 		m_mgr->OpenFile(fn.GetFullPath());
 		editor = m_mgr->GetActiveEditor();
-		if (editor && editor->GetFileName().GetFullPath() == fn.GetFullPath()) {
+		if (editor && editor->GetFileName() == fn) {
 			return editor;
 		}
 	}
