@@ -11,12 +11,12 @@ cur_rev=`svn info | grep Revision | awk '{print $2;}'`
 curdir=`pwd`
 rm -fr /tmp/codelite-${cur_rev}/
 
-svn export . /tmp/codelite-1.0.${cur_rev}/
+svn export . /tmp/codelite-2.2.0.${cur_rev}/
 
 ## set correct permissions
 
-cp ./LiteEditor/svninfo.cpp /tmp/codelite-1.0.${cur_rev}/LiteEditor/
+cp ./LiteEditor/svninfo.cpp /tmp/codelite-2.2.0.${cur_rev}/LiteEditor/
 echo "Creating tar-ball"
 cd /tmp/
-rm -fr /tmp/codelite-1.0.${cur_rev}/sdk/curl
-tar cvfz ${curdir}/codelite-1.0.${cur_rev}.tar.gz codelite-1.0.${cur_rev}/*
+rm -fr /tmp/codelite-2.2.0.${cur_rev}/sdk/curl
+tar cvfz ${curdir}/codelite-2.2.0.${cur_rev}.tar.gz codelite-2.2.0.${cur_rev}/*
