@@ -1,29 +1,31 @@
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 //
-// copyright            : (C) 2008 by Eran Ifrah                            
-// file name            : wxformbuilder.cpp              
-//                                                                          
+// copyright            : (C) 2008 by Eran Ifrah
+// file name            : wxformbuilder.cpp
+//
 // -------------------------------------------------------------------------
-// A                                                                        
-//              _____           _      _     _ _                            
-//             /  __ \         | |    | |   (_) |                           
-//             | /  \/ ___   __| | ___| |    _| |_ ___                      
-//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )                     
-//             | \__/\ (_) | (_| |  __/ |___| | ||  __/                     
-//              \____/\___/ \__,_|\___\_____/_|\__\___|                     
-//                                                                          
-//                                                  F i l e                 
-//                                                                          
-//    This program is free software; you can redistribute it and/or modify  
-//    it under the terms of the GNU General Public License as published by  
-//    the Free Software Foundation; either version 2 of the License, or     
-//    (at your option) any later version.                                   
-//                                                                          
+// A
+//              _____           _      _     _ _
+//             /  __ \         | |    | |   (_) |
+//             | /  \/ ___   __| | ___| |    _| |_ ___
+//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
+//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
+//              \____/\___/ \__,_|\___\_____/_|\__\___|
+//
+//                                                  F i l e
+//
+//    This program is free software; you can redistribute it and/or modify
+//    it under the terms of the GNU General Public License as published by
+//    the Free Software Foundation; either version 2 of the License, or
+//    (at your option) any later version.
+//
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 
+#include <wx/app.h>
 #include "confformbuilder.h"
+#include <wx/mimetype.h>
 #include "procutils.h"
 #include "workspace.h"
 #include "project.h"
@@ -31,7 +33,6 @@
 #include "globals.h"
 #include "formbuildsettingsdlg.h"
 #include "wxfbitemdlg.h"
-#include <wx/app.h>
 #include <wx/menu.h>
 #include <wx/xrc/xmlres.h>
 #include "wxformbuilder.h"
@@ -42,8 +43,7 @@ static wxFormBuilder* thePlugin = NULL;
 //Define the plugin entry point
 extern "C" EXPORT IPlugin *CreatePlugin(IManager *manager)
 {
-	if (thePlugin == 0)
-	{
+	if (thePlugin == 0) {
 		thePlugin = new wxFormBuilder(manager);
 	}
 	return thePlugin;
@@ -74,11 +74,12 @@ wxFormBuilder::wxFormBuilder(IManager *manager)
 	m_shortName = wxT("wxFormBuilder");
 	m_topWin = m_mgr->GetTheApp();
 
-	m_topWin->Connect(XRCID("wxfb_new_dialog"), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(wxFormBuilder::OnNewDialog), NULL, this);
+	m_topWin->Connect(XRCID("wxfb_new_dialog"),              wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(wxFormBuilder::OnNewDialog),            NULL, this);
 	m_topWin->Connect(XRCID("wxfb_new_dialog_with_buttons"), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(wxFormBuilder::OnNewDialogWithButtons), NULL, this);
-	m_topWin->Connect(XRCID("wxfb_new_frame"), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(wxFormBuilder::OnNewFrame), NULL, this);
-	m_topWin->Connect(XRCID("wxfb_new_panel"), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(wxFormBuilder::OnNewPanel), NULL, this);
-	m_topWin->Connect(XRCID("wxfb_open"), wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(wxFormBuilder::OpenWithWxFb), NULL, this);
+	m_topWin->Connect(XRCID("wxfb_new_frame"),               wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(wxFormBuilder::OnNewFrame),             NULL, this);
+	m_topWin->Connect(XRCID("wxfb_new_panel"),               wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(wxFormBuilder::OnNewPanel),             NULL, this);
+	m_topWin->Connect(XRCID("wxfb_open"),                    wxEVT_COMMAND_MENU_SELECTED, wxCommandEventHandler(wxFormBuilder::OpenWithWxFb),           NULL, this);
+	m_topWin->Connect(wxEVT_TREE_ITEM_FILE_ACTIVATED,         wxCommandEventHandler(wxFormBuilder::OnOpenFile),                                          NULL, this);
 }
 
 wxFormBuilder::~wxFormBuilder()
@@ -105,28 +106,21 @@ void wxFormBuilder::CreatePluginMenu(wxMenu *pluginsMenu)
 
 void wxFormBuilder::HookPopupMenu(wxMenu *menu, MenuType type)
 {
-	if (type == MenuTypeFileView_Folder)
-	{
-		if (!menu->FindItem(XRCID("WXFB_POPUP")))
-		{
+	if (type == MenuTypeFileView_Folder) {
+		if (!menu->FindItem(XRCID("WXFB_POPUP"))) {
 			m_separatorItem = menu->AppendSeparator();
 			menu->Append(XRCID("WXFB_POPUP"), wxT("wxFormBuilder"), CreatePopupMenu());
 		}
-	}
-	else if (type == MenuTypeFileView_File)
-	{
+	} else if (type == MenuTypeFileView_File) {
 		bool isFbpFile(false);
 		TreeItemInfo item = m_mgr->GetSelectedTreeItemInfo( TreeFileView );
-		if ( item.m_item.IsOk() && item.m_itemType == ProjectItem::TypeFile )
-		{
-			if (item.m_fileName.GetExt() == wxT("fbp"))
-			{
+		if ( item.m_item.IsOk() && item.m_itemType == ProjectItem::TypeFile ) {
+			if (item.m_fileName.GetExt() == wxT("fbp")) {
 				isFbpFile = true;
 			}
 		}
 
-		if (!m_openWithWxFbItem && isFbpFile)
-		{
+		if (!m_openWithWxFbItem && isFbpFile) {
 			m_openWithWxFbSepItem = menu->PrependSeparator();
 			m_openWithWxFbItem = menu->Prepend(XRCID("wxfb_open"), wxT("Open with wxFormBuilder..."));
 		}
@@ -135,20 +129,15 @@ void wxFormBuilder::HookPopupMenu(wxMenu *menu, MenuType type)
 
 void wxFormBuilder::UnHookPopupMenu(wxMenu *menu, MenuType type)
 {
-	if (type == MenuTypeFileView_Folder)
-	{
+	if (type == MenuTypeFileView_Folder) {
 		wxMenuItem *item = menu->FindItem(XRCID("WXFB_POPUP"));
-		if (item)
-		{
+		if (item) {
 			menu->Destroy(item);
 			menu->Destroy(m_separatorItem);
 			m_separatorItem = NULL;
 		}
-	}
-	else if (type == MenuTypeFileView_File)
-	{
-		if ( m_openWithWxFbItem && m_openWithWxFbSepItem )
-		{
+	} else if (type == MenuTypeFileView_File) {
+		if ( m_openWithWxFbItem && m_openWithWxFbSepItem ) {
 			menu->Destroy(m_openWithWxFbItem);
 			m_openWithWxFbItem = NULL;
 			menu->Destroy(m_openWithWxFbSepItem);
@@ -176,7 +165,7 @@ wxMenu* wxFormBuilder::CreatePopupMenu()
 
 	item = new wxMenuItem(menu, XRCID("wxfb_new_dialog"), wxT("New wxDialog..."), wxEmptyString, wxITEM_NORMAL);
 	menu->Append(item);
-	
+
 	item = new wxMenuItem(menu, XRCID("wxfb_new_dialog_with_buttons"), wxT("New wxDialog with Default Buttons..."), wxEmptyString, wxITEM_NORMAL);
 	menu->Append(item);
 
@@ -193,8 +182,7 @@ void wxFormBuilder::OnNewDialog(wxCommandEvent& e)
 {
 	wxFBItemDlg dlg(m_mgr->GetTheApp()->GetTopWindow(), m_mgr);
 	dlg.SetTitle(_("New wxDialog"));
-	if (dlg.ShowModal() == wxID_OK)
-	{
+	if (dlg.ShowModal() == wxID_OK) {
 		wxFBItemInfo info;
 		info = dlg.GetData();
 		info.kind = wxFBItemKind_Dialog;
@@ -207,8 +195,7 @@ void wxFormBuilder::OnNewFrame(wxCommandEvent& e)
 {
 	wxFBItemDlg dlg(m_mgr->GetTheApp()->GetTopWindow(), m_mgr);
 	dlg.SetTitle(_("New wxFrame"));
-	if (dlg.ShowModal() == wxID_OK)
-	{
+	if (dlg.ShowModal() == wxID_OK) {
 		wxFBItemInfo info;
 		info = dlg.GetData();
 		info.kind = wxFBItemKind_Frame;
@@ -220,12 +207,11 @@ void wxFormBuilder::OnNewPanel(wxCommandEvent& e)
 {
 	wxFBItemDlg dlg(m_mgr->GetTheApp()->GetTopWindow(), m_mgr);
 	dlg.SetTitle(_("New wxPanel"));
-	
+
 	// no need for title in wxPanel
 	dlg.DisableTitleField();
-	
-	if (dlg.ShowModal() == wxID_OK)
-	{
+
+	if (dlg.ShowModal() == wxID_OK) {
 		wxFBItemInfo info;
 		info = dlg.GetData();
 		info.kind = wxFBItemKind_Panel;
@@ -242,8 +228,7 @@ void wxFormBuilder::DoCreateWxFormBuilderProject(const wxFBItemInfo& data)
 	m_mgr->CreateVirtualDirectory(formbuilderVD, wxT("formbuilder"));
 	wxString templateFile(m_mgr->GetInstallDirectory() + wxT("/templates/formbuilder/"));
 
-	switch (data.kind)
-	{
+	switch (data.kind) {
 	default:
 	case wxFBItemKind_Dialog:
 		templateFile << wxT("DialogTemplate.fbp");
@@ -260,8 +245,7 @@ void wxFormBuilder::DoCreateWxFormBuilderProject(const wxFBItemInfo& data)
 	}
 
 	wxFileName tmplFile(templateFile);
-	if (!tmplFile.FileExists())
-	{
+	if (!tmplFile.FileExists()) {
 		wxMessageBox(wxString::Format(wxT("Cant find wxFormBuilder template file '%s'"), tmplFile.GetFullPath().c_str()), wxT("CodeLite"), wxOK|wxCENTER|wxICON_WARNING);
 		return;
 	}
@@ -270,31 +254,27 @@ void wxFormBuilder::DoCreateWxFormBuilderProject(const wxFBItemInfo& data)
 	wxString err_msg;
 	wxString project = data.virtualFolder.BeforeFirst(wxT(':'));
 	ProjectPtr proj = m_mgr->GetWorkspace()->FindProjectByName(project, err_msg);
-	if (proj)
-	{
+	if (proj) {
 		wxString files_path = proj->GetFileName().GetPath(wxPATH_GET_SEPARATOR|wxPATH_GET_VOLUME);
 		// copy the file to here
 		wxFileName fbpFile(files_path, data.file + wxT(".fbp"));
-		if (!wxCopyFile(tmplFile.GetFullPath(), fbpFile.GetFullPath()))
-		{
+		if (!wxCopyFile(tmplFile.GetFullPath(), fbpFile.GetFullPath())) {
 			wxMessageBox(wxString::Format(wxT("Failed to copy tempalte file to '%s'"), fbpFile.GetFullPath().c_str()), wxT("CodeLite"), wxOK|wxCENTER|wxICON_WARNING);
 			return;
 		}
 		// open the file, and replace expand its macros
 		wxString content;
-		if (!ReadFileWithConversion(fbpFile.GetFullPath().c_str(), content))
-		{
+		if (!ReadFileWithConversion(fbpFile.GetFullPath().c_str(), content)) {
 			wxMessageBox(wxString::Format(wxT("Failed to read file '%s'"), fbpFile.GetFullPath().c_str()), wxT("CodeLite"), wxOK|wxCENTER|wxICON_WARNING);
 			return;
 		}
-		
+
 		content.Replace(wxT("$(BaseFileName)"), data.file);
 		content.Replace(wxT("$(ProjectName)"), data.className);
 		content.Replace(wxT("$(Title)"), data.title);
 		content.Replace(wxT("$(ClassName)"), data.className);
-		
-		if (!WriteFileWithBackup(fbpFile.GetFullPath().c_str(), content, false))
-		{
+
+		if (!WriteFileWithBackup(fbpFile.GetFullPath().c_str(), content, false)) {
 			wxMessageBox(wxString::Format(wxT("Failed to write file '%s'"), fbpFile.GetFullPath().c_str()), wxT("CodeLite"), wxOK|wxCENTER|wxICON_WARNING);
 			return;
 		}
@@ -314,18 +294,15 @@ void wxFormBuilder::DoCreateWxFormBuilderProject(const wxFBItemInfo& data)
 		wxFileName cppFile(fbpFile.GetPath(), data.file + wxT(".cpp"));
 		wxFileName headerFile(fbpFile.GetPath(), data.file + wxT(".h"));
 
-		if (cppFile.FileExists())
-		{
+		if (cppFile.FileExists()) {
 			filesToAdd.Add(cppFile.GetFullPath());
 		}
 
-		if (headerFile.FileExists())
-		{
+		if (headerFile.FileExists()) {
 			filesToAdd.Add(headerFile.GetFullPath());
 		}
 
-		if (filesToAdd.GetCount())
-		{
+		if (filesToAdd.GetCount()) {
 			m_mgr->AddFilesToVirtualFolder(data.virtualFolder, filesToAdd);
 		}
 
@@ -337,14 +314,10 @@ void wxFormBuilder::OpenWithWxFb(wxCommandEvent& e)
 {
 	// get the file name
 	TreeItemInfo item = m_mgr->GetSelectedTreeItemInfo( TreeFileView );
-	if ( item.m_item.IsOk() && item.m_itemType == ProjectItem::TypeFile )
-	{
-		if (item.m_fileName.GetExt() == wxT("fbp"))
-		{
+	if ( item.m_item.IsOk() && item.m_itemType == ProjectItem::TypeFile ) {
+		if (item.m_fileName.GetExt() == wxT("fbp")) {
 			DoLaunchWxFB(item.m_fileName.GetFullPath());
-		}
-		else
-		{
+		} else {
 			wxMessageBox(_("Please select a 'fbp' (Form Builder Project) file only"), wxT("CodeLite"), wxOK|wxCENTER|wxICON_INFORMATION);
 			return;
 		}
@@ -354,8 +327,7 @@ void wxFormBuilder::OpenWithWxFb(wxCommandEvent& e)
 void wxFormBuilder::DoLaunchWxFB(const wxString& file)
 {
 	wxString fbpath = GetWxFBPath();
-	if (fbpath.IsEmpty())
-	{
+	if (fbpath.IsEmpty()) {
 		wxMessageBox(_("Failed to launch wxFormBuilder, no path specified\nPlease set wxFormBuilder path from Plugins -> wxFormBuilder -> Settings..."),
 		             wxT("CodeLite"), wxOK|wxCENTER|wxICON_WARNING);
 		return;
@@ -378,26 +350,20 @@ wxString wxFormBuilder::GetWxFBPath()
 	wxString fbpath = confData.GetFbPath();
 
 #ifdef __WXGTK__
-	if (fbpath.IsEmpty())
-	{
+	if (fbpath.IsEmpty()) {
 		// try to locate the file at '/usr/bin' or '/usr/local/bin'
-		if (wxFileName::FileExists(wxT("/usr/local/bin/wxformbuilder")))
-		{
+		if (wxFileName::FileExists(wxT("/usr/local/bin/wxformbuilder"))) {
 			fbpath = wxT("/usr/local/bin/wxformbuilder");
-		}
-		else if (wxFileName::FileExists(wxT("/usr/bin/wxformbuilder")))
-		{
+		} else if (wxFileName::FileExists(wxT("/usr/bin/wxformbuilder"))) {
 			fbpath = wxT("/usr/bin/wxformbuilder");
 		}
 	}
 #endif
 
 #ifdef __WXMSW__
-	if (fbpath.IsEmpty())
-	{
+	if (fbpath.IsEmpty()) {
 		// try to locate the file at '/usr/bin' or '/usr/local/bin'
-		if (wxFileName::FileExists(wxT("C:\\Program Files\\wxFormBuilder\\wxFormBuilder.exe")))
-		{
+		if (wxFileName::FileExists(wxT("C:\\Program Files\\wxFormBuilder\\wxFormBuilder.exe"))) {
 			fbpath = wxT("C:\\Program Files\\wxFormBuilder\\wxFormBuilder.exe");
 		}
 	}
@@ -409,8 +375,7 @@ void wxFormBuilder::OnNewDialogWithButtons(wxCommandEvent& e)
 {
 	wxFBItemDlg dlg(m_mgr->GetTheApp()->GetTopWindow(), m_mgr);
 	dlg.SetTitle(_("New wxDialog with Default Buttons"));
-	if (dlg.ShowModal() == wxID_OK)
-	{
+	if (dlg.ShowModal() == wxID_OK) {
 		wxFBItemInfo info;
 		info = dlg.GetData();
 		info.kind = wxFBItemKind_Dialog_With_Buttons;
@@ -418,3 +383,30 @@ void wxFormBuilder::OnNewDialogWithButtons(wxCommandEvent& e)
 		DoCreateWxFormBuilderProject(info);
 	}
 }
+
+void wxFormBuilder::OnOpenFile(wxCommandEvent& e)
+{
+	wxString *fn = (wxString*)e.GetClientData();
+	if ( fn ) {
+		// launch it with the default application
+		wxFileName fullpath ( *fn );
+		if ( fullpath.GetExt().MakeLower() != wxT("fbp") ) {
+			e.Skip();
+			return;
+		}
+
+		wxMimeTypesManager *mgr = wxTheMimeTypesManager;
+		wxFileType *type = mgr->GetFileTypeFromExtension(fullpath.GetExt());
+		if ( type ) {
+			wxString cmd = type->GetOpenCommand(fullpath.GetFullPath());
+			delete type;
+
+			if ( cmd.IsEmpty() == false ) {
+				wxExecute(cmd);
+				return;
+			}
+		}
+	}
+	// we failed, call event.Skip()
+	e.Skip();
+}
