@@ -1,25 +1,25 @@
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 //
-// copyright            : (C) 2008 by Eran Ifrah                            
-// file name            : wxformbuilder.h              
-//                                                                          
+// copyright            : (C) 2008 by Eran Ifrah
+// file name            : wxformbuilder.h
+//
 // -------------------------------------------------------------------------
-// A                                                                        
-//              _____           _      _     _ _                            
-//             /  __ \         | |    | |   (_) |                           
-//             | /  \/ ___   __| | ___| |    _| |_ ___                      
-//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )                     
-//             | \__/\ (_) | (_| |  __/ |___| | ||  __/                     
-//              \____/\___/ \__,_|\___\_____/_|\__\___|                     
-//                                                                          
-//                                                  F i l e                 
-//                                                                          
-//    This program is free software; you can redistribute it and/or modify  
-//    it under the terms of the GNU General Public License as published by  
-//    the Free Software Foundation; either version 2 of the License, or     
-//    (at your option) any later version.                                   
-//                                                                          
+// A
+//              _____           _      _     _ _
+//             /  __ \         | |    | |   (_) |
+//             | /  \/ ___   __| | ___| |    _| |_ ___
+//             | |    / _ \ / _  |/ _ \ |   | | __/ _ )
+//             | \__/\ (_) | (_| |  __/ |___| | ||  __/
+//              \____/\___/ \__,_|\___\_____/_|\__\___|
+//
+//                                                  F i l e
+//
+//    This program is free software; you can redistribute it and/or modify
+//    it under the terms of the GNU General Public License as published by
+//    the Free Software Foundation; either version 2 of the License, or
+//    (at your option) any later version.
+//
 //////////////////////////////////////////////////////////////////////////////
 //////////////////////////////////////////////////////////////////////////////
 
@@ -36,11 +36,11 @@ class wxFormBuilder : public IPlugin
 	wxMenuItem* m_separatorItem;
 	wxMenuItem* m_openWithWxFbItem;
 	wxMenuItem* m_openWithWxFbSepItem;
-	
+
 public:
 	wxFormBuilder(IManager *manager);
 	~wxFormBuilder();
-	
+
 protected:
 	void OnSettings(wxCommandEvent &e);
 	void OnNewDialog(wxCommandEvent &e);
@@ -48,12 +48,13 @@ protected:
 	void OnNewFrame(wxCommandEvent &e);
 	void OnNewPanel(wxCommandEvent &e);
 	void OpenWithWxFb(wxCommandEvent &e);
-	
+	void OnOpenFile(wxCommandEvent &e);
+
 	wxMenu *CreatePopupMenu();
 	void DoCreateWxFormBuilderProject(const wxFBItemInfo &data);
 	void DoLaunchWxFB(const wxString &file);
 	wxString GetWxFBPath();
-	
+
 public:
 	//--------------------------------------------
 	//Abstract methods
